@@ -2,7 +2,6 @@ package lxd
 
 import (
 	"bytes"
-	"crypto/sha256"
 	"crypto/x509"
 	"encoding/base64"
 	"encoding/json"
@@ -25,6 +24,9 @@ import (
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/ioprogress"
+	"github.com/lxc/lxd/shared/simplestreams"
+	"github.com/lxc/lxd/shared/version"
 )
 
 // Client can talk to a LXD daemon.
@@ -39,7 +41,7 @@ type Client struct {
 
 	Http            http.Client
 	websocketDialer websocket.Dialer
-	simplestreams   *shared.SimpleStreams
+	simplestreams   *simplestreams.SimpleStreams
 }
 
 type ResponseType string
@@ -272,7 +274,10 @@ func connectViaUnix(c *Client, remote *RemoteConfig) error {
 		}
 		return net.DialUnix("unix", nil, raddr)
 	}
-	c.Http.Transport = &http.Transport{Dial: uDial}
+	c.Http.Transport = &http.Transport{
+		Dial:              uDial,
+		DisableKeepAlives: true,
+	}
 	c.websocketDialer.NetDial = uDial
 	c.Remote = remote
 
@@ -291,9 +296,10 @@ func connectViaHttp(c *Client, remote *RemoteConfig, clientCert, clientKey, clie
 	}
 
 	tr := &http.Transport{
-		TLSClientConfig: tlsconfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           shared.ProxyFromEnvironment,
+		TLSClientConfig:   tlsconfig,
+		Dial:              shared.RFC3493Dialer,
+		Proxy:             shared.ProxyFromEnvironment,
+		DisableKeepAlives: true,
 	}
 
 	c.websocketDialer.NetDial = shared.RFC3493Dialer
@@ -334,11 +340,20 @@ func NewClientFromInfo(info ConnectInfo) (*Client, error) {
 	}
 
 	if info.RemoteConfig.Protocol == "simplestreams" {
-		ss, err := shared.SimpleStreamsClient(c.Remote.Addr, shared.ProxyFromEnvironment)
+		tlsconfig, err := shared.GetTLSConfig("", "", "", nil)
 		if err != nil {
 			return nil, err
 		}
 
+		tr := &http.Transport{
+			TLSClientConfig:   tlsconfig,
+			Dial:              shared.RFC3493Dialer,
+			Proxy:             shared.ProxyFromEnvironment,
+			DisableKeepAlives: true,
+		}
+		c.Http.Transport = tr
+
+		ss := simplestreams.NewClient(c.Remote.Addr, c.Http, version.UserAgent)
 		c.simplestreams = ss
 	}
 
@@ -353,9 +368,19 @@ func (c *Client) Addresses() ([]string, error) {
 		if err != nil {
 			return nil, err
 		}
+
 		addresses = serverStatus.Environment.Addresses
 	} else if c.Transport == "https" {
 		addresses = append(addresses, c.BaseURL[8:])
+
+		if !c.Remote.Public {
+			serverStatus, err := c.ServerStatus()
+			if err != nil {
+				return nil, err
+			}
+
+			addresses = append(addresses, serverStatus.Environment.Addresses...)
+		}
 	} else {
 		return nil, fmt.Errorf("unknown transport type: %s", c.Transport)
 	}
@@ -368,7 +393,7 @@ func (c *Client) Addresses() ([]string, error) {
 }
 
 func (c *Client) get(base string) (*Response, error) {
-	uri := c.url(shared.APIVersion, base)
+	uri := c.url(version.APIVersion, base)
 
 	return c.baseGet(uri)
 }
@@ -379,7 +404,7 @@ func (c *Client) baseGet(getUrl string) (*Response, error) {
 		return nil, err
 	}
 
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 
 	resp, err := c.Http.Do(req)
 	if err != nil {
@@ -389,8 +414,8 @@ func (c *Client) baseGet(getUrl string) (*Response, error) {
 	return HoistResponse(resp, Sync)
 }
 
-func (c *Client) put(base string, args interface{}, rtype ResponseType) (*Response, error) {
-	uri := c.url(shared.APIVersion, base)
+func (c *Client) doUpdateMethod(method string, base string, args interface{}, rtype ResponseType) (*Response, error) {
+	uri := c.url(version.APIVersion, base)
 
 	buf := bytes.Buffer{}
 	err := json.NewEncoder(&buf).Encode(args)
@@ -398,13 +423,13 @@ func (c *Client) put(base string, args interface{}, rtype ResponseType) (*Respon
 		return nil, err
 	}
 
-	shared.LogDebugf("Putting %s to %s", buf.String(), uri)
+	shared.LogDebugf("%s %s to %s", method, buf.String(), uri)
 
-	req, err := http.NewRequest("PUT", uri, &buf)
+	req, err := http.NewRequest(method, uri, &buf)
 	if err != nil {
 		return nil, err
 	}
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 	req.Header.Set("Content-Type", "application/json")
 
 	resp, err := c.Http.Do(req)
@@ -415,30 +440,20 @@ func (c *Client) put(base string, args interface{}, rtype ResponseType) (*Respon
 	return HoistResponse(resp, rtype)
 }
 
+func (c *Client) put(base string, args interface{}, rtype ResponseType) (*Response, error) {
+	return c.doUpdateMethod("PUT", base, args, rtype)
+}
+
+func (c *Client) patch(base string, args interface{}, rtype ResponseType) (*Response, error) {
+	return c.doUpdateMethod("PATCH", base, args, rtype)
+}
+
 func (c *Client) post(base string, args interface{}, rtype ResponseType) (*Response, error) {
-	uri := c.url(shared.APIVersion, base)
+	return c.doUpdateMethod("POST", base, args, rtype)
+}
 
-	buf := bytes.Buffer{}
-	err := json.NewEncoder(&buf).Encode(args)
-	if err != nil {
-		return nil, err
-	}
-
-	shared.LogDebugf("Posting %s to %s", buf.String(), uri)
-
-	req, err := http.NewRequest("POST", uri, &buf)
-	if err != nil {
-		return nil, err
-	}
-	req.Header.Set("User-Agent", shared.UserAgent)
-	req.Header.Set("Content-Type", "application/json")
-
-	resp, err := c.Http.Do(req)
-	if err != nil {
-		return nil, err
-	}
-
-	return HoistResponse(resp, rtype)
+func (c *Client) delete(base string, args interface{}, rtype ResponseType) (*Response, error) {
+	return c.doUpdateMethod("DELETE", base, args, rtype)
 }
 
 func (c *Client) getRaw(uri string) (*http.Response, error) {
@@ -446,7 +461,7 @@ func (c *Client) getRaw(uri string) (*http.Response, error) {
 	if err != nil {
 		return nil, err
 	}
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 
 	raw, err := c.Http.Do(req)
 	if err != nil {
@@ -465,32 +480,6 @@ func (c *Client) getRaw(uri string) (*http.Response, error) {
 	return raw, nil
 }
 
-func (c *Client) delete(base string, args interface{}, rtype ResponseType) (*Response, error) {
-	uri := c.url(shared.APIVersion, base)
-
-	buf := bytes.Buffer{}
-	err := json.NewEncoder(&buf).Encode(args)
-	if err != nil {
-		return nil, err
-	}
-
-	shared.LogDebugf("Deleting %s to %s", buf.String(), uri)
-
-	req, err := http.NewRequest("DELETE", uri, &buf)
-	if err != nil {
-		return nil, err
-	}
-	req.Header.Set("User-Agent", shared.UserAgent)
-	req.Header.Set("Content-Type", "application/json")
-
-	resp, err := c.Http.Do(req)
-	if err != nil {
-		return nil, err
-	}
-
-	return HoistResponse(resp, rtype)
-}
-
 func (c *Client) Websocket(operation string, secret string) (*websocket.Conn, error) {
 	query := url.Values{"secret": []string{secret}}
 	url := c.BaseWSURL + path.Join(operation, "websocket") + "?" + query.Encode()
@@ -533,7 +522,7 @@ func (c *Client) GetServerConfig() (*Response, error) {
 		return nil, fmt.Errorf("This function isn't supported by simplestreams remote.")
 	}
 
-	return c.baseGet(c.url(shared.APIVersion))
+	return c.baseGet(c.url(version.APIVersion))
 }
 
 // GetLocalLXDErr determines whether or not an error is likely due to a
@@ -781,7 +770,7 @@ func (c *Client) ExportImage(image string, target string) (string, error) {
 		return c.simplestreams.ExportImage(image, target)
 	}
 
-	uri := c.url(shared.APIVersion, "images", image, "export")
+	uri := c.url(version.APIVersion, "images", image, "export")
 	raw, err := c.getRaw(uri)
 	if err != nil {
 		return "", err
@@ -984,12 +973,12 @@ func (c *Client) PostImageURL(imageFile string, properties []string, public bool
 	return fingerprint, nil
 }
 
-func (c *Client) PostImage(imageFile string, rootfsFile string, properties []string, public bool, aliases []string, progressHandler func(percent int)) (string, error) {
+func (c *Client) PostImage(imageFile string, rootfsFile string, properties []string, public bool, aliases []string, progressHandler func(int64, int64)) (string, error) {
 	if c.Remote.Public {
 		return "", fmt.Errorf("This function isn't supported by public remotes.")
 	}
 
-	uri := c.url(shared.APIVersion, "images")
+	uri := c.url(version.APIVersion, "images")
 
 	var err error
 	var fImage *os.File
@@ -1051,7 +1040,13 @@ func (c *Client) PostImage(imageFile string, rootfsFile string, properties []str
 			return "", err
 		}
 
-		progress := &shared.TransferProgress{Reader: body, Length: size, Handler: progressHandler}
+		progress := &ioprogress.ProgressReader{
+			ReadCloser: body,
+			Tracker: &ioprogress.ProgressTracker{
+				Length:  size,
+				Handler: progressHandler,
+			},
+		}
 
 		req, err = http.NewRequest("POST", uri, progress)
 		req.Header.Set("Content-Type", w.FormDataContentType())
@@ -1067,7 +1062,13 @@ func (c *Client) PostImage(imageFile string, rootfsFile string, properties []str
 			return "", err
 		}
 
-		progress := &shared.TransferProgress{Reader: fImage, Length: stat.Size(), Handler: progressHandler}
+		progress := &ioprogress.ProgressReader{
+			ReadCloser: fImage,
+			Tracker: &ioprogress.ProgressTracker{
+				Length:  stat.Size(),
+				Handler: progressHandler,
+			},
+		}
 
 		req, err = http.NewRequest("POST", uri, progress)
 		req.Header.Set("X-LXD-filename", filepath.Base(imageFile))
@@ -1077,7 +1078,7 @@ func (c *Client) PostImage(imageFile string, rootfsFile string, properties []str
 	if err != nil {
 		return "", err
 	}
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 
 	if public {
 		req.Header.Set("X-LXD-public", "1")
@@ -1686,10 +1687,10 @@ func (c *Client) ServerStatus() (*shared.ServerState, error) {
 
 	// Fill in certificate fingerprint if not provided
 	if ss.Environment.CertificateFingerprint == "" && ss.Environment.Certificate != "" {
-		pemCertificate, _ := pem.Decode([]byte(ss.Environment.Certificate))
-		if pemCertificate != nil {
-			digest := sha256.Sum256(pemCertificate.Bytes)
-			ss.Environment.CertificateFingerprint = fmt.Sprintf("%x", digest)
+		var err error
+		ss.Environment.CertificateFingerprint, err = shared.CertFingerprintStr(ss.Environment.Certificate)
+		if err != nil {
+			return nil, err
 		}
 	}
 
@@ -1739,7 +1740,7 @@ func (c *Client) GetLog(container string, log string) (io.Reader, error) {
 		return nil, fmt.Errorf("This function isn't supported by public remotes.")
 	}
 
-	uri := c.url(shared.APIVersion, "containers", container, "logs", log)
+	uri := c.url(version.APIVersion, "containers", container, "logs", log)
 	resp, err := c.getRaw(uri)
 	if err != nil {
 		return nil, err
@@ -1773,13 +1774,13 @@ func (c *Client) PushFile(container string, p string, gid int, uid int, mode str
 	}
 
 	query := url.Values{"path": []string{p}}
-	uri := c.url(shared.APIVersion, "containers", container, "files") + "?" + query.Encode()
+	uri := c.url(version.APIVersion, "containers", container, "files") + "?" + query.Encode()
 
 	req, err := http.NewRequest("POST", uri, buf)
 	if err != nil {
 		return err
 	}
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 	req.Header.Set("X-LXD-type", "file")
 
 	if mode != "" {
@@ -1807,14 +1808,14 @@ func (c *Client) Mkdir(container string, p string, mode os.FileMode) error {
 	}
 
 	query := url.Values{"path": []string{p}}
-	uri := c.url(shared.APIVersion, "containers", container, "files") + "?" + query.Encode()
+	uri := c.url(version.APIVersion, "containers", container, "files") + "?" + query.Encode()
 
 	req, err := http.NewRequest("POST", uri, nil)
 	if err != nil {
 		return err
 	}
 
-	req.Header.Set("User-Agent", shared.UserAgent)
+	req.Header.Set("User-Agent", version.UserAgent)
 	req.Header.Set("X-LXD-type", "directory")
 	req.Header.Set("X-LXD-mode", fmt.Sprintf("%04o", mode.Perm()))
 
@@ -1832,6 +1833,11 @@ func (c *Client) MkdirP(container string, p string, mode os.FileMode) error {
 		return fmt.Errorf("This function isn't supported by public remotes.")
 	}
 
+	/* special case, every container has a /, we don't need to do anything */
+	if p == "/" {
+		return nil
+	}
+
 	parts := strings.Split(p, "/")
 	i := len(parts)
 
@@ -1872,7 +1878,12 @@ func (c *Client) RecursivePushFile(container string, source string, target strin
 			return fmt.Errorf("got error sending path %s: %s", p, err)
 		}
 
-		targetPath := path.Join(target, p[len(sourceDir):])
+		appendLen := len(sourceDir)
+		if sourceDir == "." {
+			appendLen--
+		}
+
+		targetPath := path.Join(target, p[appendLen:])
 		if fInfo.IsDir() {
 			return c.Mkdir(container, targetPath, fInfo.Mode())
 		}
@@ -1896,7 +1907,7 @@ func (c *Client) PullFile(container string, p string) (int, int, int, string, io
 		return 0, 0, 0, "", nil, nil, fmt.Errorf("This function isn't supported by public remotes.")
 	}
 
-	uri := c.url(shared.APIVersion, "containers", container, "files")
+	uri := c.url(version.APIVersion, "containers", container, "files")
 	query := url.Values{"path": []string{p}}
 
 	r, err := c.getRaw(uri + "?" + query.Encode())
