@@ -2,7 +2,6 @@ package main
 
 import (
 	"crypto/x509"
-	"encoding/base64"
 	"encoding/pem"
 	"fmt"
 	"io/ioutil"
@@ -14,10 +13,8 @@ import (
 	"github.com/olekukonko/tablewriter"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
@@ -58,91 +55,51 @@ func (c *configCmd) configEditHelp() string {
 
 func (c *configCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc config <subcommand> [options]
+		`Manage configuration.
 
-Change container or server configuration options.
+lxc config device add [<remote>:]<container> <device> <type> [key=value...]   Add a device to a container.
+lxc config device get [<remote>:]<container> <device> <key>                   Get a device property.
+lxc config device set [<remote>:]<container> <device> <key> <value>           Set a device property.
+lxc config device unset [<remote>:]<container> <device> <key>                 Unset a device property.
+lxc config device list [<remote>:]<container>                                 List devices for container.
+lxc config device show [<remote>:]<container>                                 Show full device details for container.
+lxc config device remove [<remote>:]<container> <name>                        Remove device from container.
 
-*Container configuration*
-
-lxc config get [<remote>:][container] <key>
-    Get container or server configuration key.
-
-lxc config set [<remote>:][container] <key> <value>
-    Set container or server configuration key.
-
-lxc config unset [<remote>:][container] <key>
-    Unset container or server configuration key.
-
-lxc config show [<remote>:][container] [--expanded]
-    Show container or server configuration.
-
-lxc config edit [<remote>:][container]
+lxc config get [<remote>:][container] <key>                                   Get container or server configuration key.
+lxc config set [<remote>:][container] <key> <value>                           Set container or server configuration key.
+lxc config unset [<remote>:][container] <key>                                 Unset container or server configuration key.
+lxc config show [<remote>:][container] [--expanded]                           Show container or server configuration.
+lxc config edit [<remote>:][container]                                        Edit container or server configuration in external editor.
     Edit configuration, either by launching external editor or reading STDIN.
+    Example: lxc config edit <container> # launch editor
+             cat config.yaml | lxc config edit <container> # read from config.yaml
 
-*Device management*
+lxc config trust list [<remote>:]                                             List all trusted certs.
+lxc config trust add [<remote>:] <certfile.crt>                               Add certfile.crt to trusted hosts.
+lxc config trust remove [<remote>:] [hostname|fingerprint]                    Remove the cert from trusted hosts.
 
-lxc config device add [<remote>:]<container> <device> <type> [key=value...]
-    Add a device to a container.
+Examples:
+To mount host's /share/c1 onto /opt in the container:
+    lxc config device add [<remote>:]container1 <device-name> disk source=/share/c1 path=opt
 
-lxc config device get [<remote>:]<container> <device> <key>
-    Get a device property.
+To set an lxc config value:
+    lxc config set [<remote>:]<container> raw.lxc 'lxc.aa_allow_incomplete = 1'
 
-lxc config device set [<remote>:]<container> <device> <key> <value>
-    Set a device property.
+To listen on IPv4 and IPv6 port 8443 (you can omit the 8443 its the default):
+    lxc config set core.https_address [::]:8443
 
-lxc config device unset [<remote>:]<container> <device> <key>
-    Unset a device property.
-
-lxc config device list [<remote>:]<container>
-    List devices for container.
-
-lxc config device show [<remote>:]<container>
-    Show full device details for container.
-
-lxc config device remove [<remote>:]<container> <name>...
-    Remove device from container.
-
-*Client trust store management*
-
-lxc config trust list [<remote>:]
-    List all trusted certs.
-
-lxc config trust add [<remote>:] <certfile.crt>
-    Add certfile.crt to trusted hosts.
-
-lxc config trust remove [<remote>:] [hostname|fingerprint]
-    Remove the cert from trusted hosts.
-
-*Examples*
-
-cat config.yaml | lxc config edit <container>
-    Update the container configuration from config.yaml.
-
-lxc config device add [<remote>:]container1 <device-name> disk source=/share/c1 path=opt
-    Will mount the host's /share/c1 onto /opt in the container.
-
-lxc config set [<remote>:]<container> limits.cpu 2
-    Will set a CPU limit of "2" for the container.
-
-lxc config set core.https_address [::]:8443
-    Will have LXD listen on IPv4 and IPv6 port 8443.
-
-lxc config set core.trust_password blah
-    Will set the server's trust password to blah.`)
+To set the server trust password:
+    lxc config set core.trust_password blah`)
 }
 
-func (c *configCmd) doSet(conf *config.Config, args []string, unset bool) error {
+func (c *configCmd) doSet(config *lxd.Config, args []string, unset bool) error {
 	if len(args) != 4 {
 		return errArgs
 	}
 
 	// [[lxc config]] set dakara:c1 limits.memory 200000
-	remote, name, err := conf.ParseRemote(args[1])
-	if err != nil {
-		return err
-	}
-
-	d, err := conf.GetContainerServer(remote)
+	remote, container := config.ParseRemoteAndContainer(args[1])
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -158,33 +115,24 @@ func (c *configCmd) doSet(conf *config.Config, args []string, unset bool) error
 		value = string(buf[:])
 	}
 
-	container, etag, err := d.GetContainer(name)
-	if err != nil {
-		return err
-	}
-
 	if unset {
-		_, ok := container.Config[key]
-		if !ok {
-			return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set"), key)
+		st, err := d.ContainerInfo(container)
+		if err != nil {
+			return err
 		}
 
-		delete(container.Config, key)
-	} else {
-		container.Config[key] = value
+		_, ok := st.Config[key]
+		if !ok {
+			return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), key)
+		}
 	}
 
-	op, err := d.UpdateContainer(name, container.Writable(), etag)
-	if err != nil {
-		return err
-	}
-
-	return op.Wait()
+	return d.SetContainerConfig(container, key, value)
 }
 
-func (c *configCmd) run(conf *config.Config, args []string) error {
+func (c *configCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
-		return errUsage
+		return errArgs
 	}
 
 	switch args[0] {
@@ -196,54 +144,50 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 
 		// Deal with local server
 		if len(args) == 2 {
-			c, err := conf.GetContainerServer(conf.DefaultRemote)
+			c, err := lxd.NewClient(config, config.DefaultRemote)
 			if err != nil {
 				return err
 			}
 
-			server, etag, err := c.GetServer()
+			ss, err := c.ServerStatus()
 			if err != nil {
 				return err
 			}
 
-			_, ok := server.Config[args[1]]
+			_, ok := ss.Config[args[1]]
 			if !ok {
 				return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), args[1])
 			}
 
-			delete(server.Config, args[1])
-			return c.UpdateServer(server.Writable(), etag)
-		}
-
-		// Deal with remote server
-		remote, container, err := conf.ParseRemote(args[1])
-		if err != nil {
+			_, err = c.SetServerConfig(args[1], "")
 			return err
 		}
 
+		// Deal with remote server
+		remote, container := config.ParseRemoteAndContainer(args[1])
 		if container == "" {
-			c, err := conf.GetContainerServer(remote)
+			c, err := lxd.NewClient(config, remote)
 			if err != nil {
 				return err
 			}
 
-			server, etag, err := c.GetServer()
+			ss, err := c.ServerStatus()
 			if err != nil {
 				return err
 			}
 
-			_, ok := server.Config[args[2]]
+			_, ok := ss.Config[args[1]]
 			if !ok {
 				return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), args[1])
 			}
 
-			delete(server.Config, args[2])
-			return c.UpdateServer(server.Writable(), etag)
+			_, err = c.SetServerConfig(args[2], "")
+			return err
 		}
 
 		// Deal with container
 		args = append(args, "")
-		return c.doSet(conf, args, true)
+		return c.doSet(config, args, true)
 
 	case "set":
 		if len(args) < 3 {
@@ -252,45 +196,29 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 
 		// Deal with local server
 		if len(args) == 3 {
-			c, err := conf.GetContainerServer(conf.DefaultRemote)
+			c, err := lxd.NewClient(config, config.DefaultRemote)
 			if err != nil {
 				return err
 			}
 
-			server, etag, err := c.GetServer()
-			if err != nil {
-				return err
-			}
-
-			server.Config[args[1]] = args[2]
-
-			return c.UpdateServer(server.Writable(), etag)
-		}
-
-		// Deal with remote server
-		remote, container, err := conf.ParseRemote(args[1])
-		if err != nil {
+			_, err = c.SetServerConfig(args[1], args[2])
 			return err
 		}
 
+		// Deal with remote server
+		remote, container := config.ParseRemoteAndContainer(args[1])
 		if container == "" {
-			c, err := conf.GetContainerServer(remote)
+			c, err := lxd.NewClient(config, remote)
 			if err != nil {
 				return err
 			}
 
-			server, etag, err := c.GetServer()
-			if err != nil {
-				return err
-			}
-
-			server.Config[args[2]] = args[3]
-
-			return c.UpdateServer(server.Writable(), etag)
+			_, err = c.SetServerConfig(args[2], args[3])
+			return err
 		}
 
 		// Deal with container
-		return c.doSet(conf, args, false)
+		return c.doSet(config, args, false)
 
 	case "trust":
 		if len(args) < 2 {
@@ -301,21 +229,17 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 		case "list":
 			var remote string
 			if len(args) == 3 {
-				var err error
-				remote, _, err = conf.ParseRemote(args[2])
-				if err != nil {
-					return err
-				}
+				remote = config.ParseRemote(args[2])
 			} else {
-				remote = conf.DefaultRemote
+				remote = config.DefaultRemote
 			}
 
-			d, err := conf.GetContainerServer(remote)
+			d, err := lxd.NewClient(config, remote)
 			if err != nil {
 				return err
 			}
 
-			trust, err := d.GetCertificates()
+			trust, err := d.CertificateList()
 			if err != nil {
 				return err
 			}
@@ -359,69 +283,52 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 			if len(args) < 3 {
 				return fmt.Errorf(i18n.G("No certificate provided to add"))
 			} else if len(args) == 4 {
-				var err error
-				remote, _, err = conf.ParseRemote(args[2])
-				if err != nil {
-					return err
-				}
+				remote = config.ParseRemote(args[2])
 			} else {
-				remote = conf.DefaultRemote
+				remote = config.DefaultRemote
 			}
 
-			d, err := conf.GetContainerServer(remote)
+			d, err := lxd.NewClient(config, remote)
 			if err != nil {
 				return err
 			}
 
 			fname := args[len(args)-1]
-			x509Cert, err := shared.ReadCert(fname)
+			cert, err := shared.ReadCert(fname)
 			if err != nil {
 				return err
 			}
+
 			name, _ := shared.SplitExt(fname)
-
-			cert := api.CertificatesPost{}
-			cert.Certificate = base64.StdEncoding.EncodeToString(x509Cert.Raw)
-			cert.Name = name
-			cert.Type = "client"
-
-			return d.CreateCertificate(cert)
+			return d.CertificateAdd(cert, name)
 		case "remove":
 			var remote string
 			if len(args) < 3 {
 				return fmt.Errorf(i18n.G("No fingerprint specified."))
 			} else if len(args) == 4 {
-				var err error
-				remote, _, err = conf.ParseRemote(args[2])
-				if err != nil {
-					return err
-				}
+				remote = config.ParseRemote(args[2])
 			} else {
-				remote = conf.DefaultRemote
+				remote = config.DefaultRemote
 			}
 
-			d, err := conf.GetContainerServer(remote)
+			d, err := lxd.NewClient(config, remote)
 			if err != nil {
 				return err
 			}
 
-			return d.DeleteCertificate(args[len(args)-1])
+			return d.CertificateRemove(args[len(args)-1])
 		default:
 			return errArgs
 		}
 
 	case "show":
-		remote := conf.DefaultRemote
+		remote := config.DefaultRemote
 		container := ""
 		if len(args) > 1 {
-			var err error
-			remote, container, err = conf.ParseRemote(args[1])
-			if err != nil {
-				return err
-			}
+			remote, container = config.ParseRemoteAndContainer(args[1])
 		}
 
-		d, err := conf.GetContainerServer(remote)
+		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
@@ -429,50 +336,44 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 		var data []byte
 
 		if len(args) == 1 || container == "" {
-			server, _, err := d.GetServer()
+			config, err := d.ServerStatus()
 			if err != nil {
 				return err
 			}
 
-			brief := server.Writable()
+			brief := config.Brief()
 			data, err = yaml.Marshal(&brief)
-			if err != nil {
-				return err
-			}
 		} else {
-			var brief api.ContainerPut
+			var brief shared.BriefContainerInfo
 			if shared.IsSnapshot(container) {
-				fields := strings.Split(container, shared.SnapshotDelimiter)
-
-				snap, _, err := d.GetContainerSnapshot(fields[0], fields[1])
+				config, err := d.SnapshotInfo(container)
 				if err != nil {
 					return err
 				}
 
-				brief = api.ContainerPut{
-					Profiles:  snap.Profiles,
-					Config:    snap.Config,
-					Devices:   snap.Devices,
-					Ephemeral: snap.Ephemeral,
+				brief = shared.BriefContainerInfo{
+					Profiles:  config.Profiles,
+					Config:    config.Config,
+					Devices:   config.Devices,
+					Ephemeral: config.Ephemeral,
 				}
 				if c.expanded {
-					brief = api.ContainerPut{
-						Profiles:  snap.Profiles,
-						Config:    snap.ExpandedConfig,
-						Devices:   snap.ExpandedDevices,
-						Ephemeral: snap.Ephemeral,
+					brief = shared.BriefContainerInfo{
+						Profiles:  config.Profiles,
+						Config:    config.ExpandedConfig,
+						Devices:   config.ExpandedDevices,
+						Ephemeral: config.Ephemeral,
 					}
 				}
 			} else {
-				container, _, err := d.GetContainer(container)
+				config, err := d.ContainerInfo(container)
 				if err != nil {
 					return err
 				}
 
-				brief = container.Writable()
+				brief = config.Brief()
 				if c.expanded {
-					brief.Config = container.ExpandedConfig
-					brief.Devices = container.ExpandedDevices
+					brief = config.BriefExpanded()
 				}
 			}
 
@@ -491,31 +392,27 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		remote := conf.DefaultRemote
+		remote := config.DefaultRemote
 		container := ""
 		key := args[1]
 		if len(args) > 2 {
-			var err error
-			remote, container, err = conf.ParseRemote(args[1])
-			if err != nil {
-				return err
-			}
+			remote, container = config.ParseRemoteAndContainer(args[1])
 			key = args[2]
 		}
 
-		d, err := conf.GetContainerServer(remote)
+		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
 
 		if container != "" {
-			resp, _, err := d.GetContainer(container)
+			resp, err := d.ContainerInfo(container)
 			if err != nil {
 				return err
 			}
 			fmt.Println(resp.Config[key])
 		} else {
-			resp, _, err := d.GetServer()
+			resp, err := d.ServerStatus()
 			if err != nil {
 				return err
 			}
@@ -540,19 +437,19 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 		}
 		switch args[1] {
 		case "list":
-			return c.deviceList(conf, "container", args)
+			return c.deviceList(config, "container", args)
 		case "add":
-			return c.deviceAdd(conf, "container", args)
+			return c.deviceAdd(config, "container", args)
 		case "remove":
-			return c.deviceRm(conf, "container", args)
+			return c.deviceRm(config, "container", args)
 		case "get":
-			return c.deviceGet(conf, "container", args)
+			return c.deviceGet(config, "container", args)
 		case "set":
-			return c.deviceSet(conf, "container", args)
+			return c.deviceSet(config, "container", args)
 		case "unset":
-			return c.deviceUnset(conf, "container", args)
+			return c.deviceUnset(config, "container", args)
 		case "show":
-			return c.deviceShow(conf, "container", args)
+			return c.deviceShow(config, "container", args)
 		default:
 			return errArgs
 		}
@@ -562,17 +459,13 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		remote := conf.DefaultRemote
+		remote := config.DefaultRemote
 		container := ""
 		if len(args) > 1 {
-			var err error
-			remote, container, err = conf.ParseRemote(args[1])
-			if err != nil {
-				return err
-			}
+			remote, container = config.ParseRemoteAndContainer(args[1])
 		}
 
-		d, err := conf.GetContainerServer(remote)
+		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
@@ -590,7 +483,7 @@ func (c *configCmd) run(conf *config.Config, args []string) error {
 	return errArgs
 }
 
-func (c *configCmd) doContainerConfigEdit(client lxd.ContainerServer, cont string) error {
+func (c *configCmd) doContainerConfigEdit(client *lxd.Client, cont string) error {
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(int(syscall.Stdin)) {
 		contents, err := ioutil.ReadAll(os.Stdin)
@@ -598,27 +491,21 @@ func (c *configCmd) doContainerConfigEdit(client lxd.ContainerServer, cont strin
 			return err
 		}
 
-		newdata := api.ContainerPut{}
+		newdata := shared.BriefContainerInfo{}
 		err = yaml.Unmarshal(contents, &newdata)
 		if err != nil {
 			return err
 		}
-
-		op, err := client.UpdateContainer(cont, newdata, "")
-		if err != nil {
-			return err
-		}
-
-		return op.Wait()
+		return client.UpdateContainerConfig(cont, newdata)
 	}
 
 	// Extract the current value
-	container, etag, err := client.GetContainer(cont)
+	config, err := client.ContainerInfo(cont)
 	if err != nil {
 		return err
 	}
 
-	brief := container.Writable()
+	brief := config.Brief()
 	data, err := yaml.Marshal(&brief)
 	if err != nil {
 		return err
@@ -632,14 +519,10 @@ func (c *configCmd) doContainerConfigEdit(client lxd.ContainerServer, cont strin
 
 	for {
 		// Parse the text received from the editor
-		newdata := api.ContainerPut{}
+		newdata := shared.BriefContainerInfo{}
 		err = yaml.Unmarshal(content, &newdata)
 		if err == nil {
-			var op *lxd.Operation
-			op, err = client.UpdateContainer(cont, newdata, etag)
-			if err == nil {
-				err = op.Wait()
-			}
+			err = client.UpdateContainerConfig(cont, newdata)
 		}
 
 		// Respawn the editor
@@ -660,11 +543,10 @@ func (c *configCmd) doContainerConfigEdit(client lxd.ContainerServer, cont strin
 		}
 		break
 	}
-
 	return nil
 }
 
-func (c *configCmd) doDaemonConfigEdit(client lxd.ContainerServer) error {
+func (c *configCmd) doDaemonConfigEdit(client *lxd.Client) error {
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(int(syscall.Stdin)) {
 		contents, err := ioutil.ReadAll(os.Stdin)
@@ -672,22 +554,23 @@ func (c *configCmd) doDaemonConfigEdit(client lxd.ContainerServer) error {
 			return err
 		}
 
-		newdata := api.ServerPut{}
+		newdata := shared.BriefServerState{}
 		err = yaml.Unmarshal(contents, &newdata)
 		if err != nil {
 			return err
 		}
 
-		return client.UpdateServer(newdata, "")
+		_, err = client.UpdateServerConfig(newdata)
+		return err
 	}
 
 	// Extract the current value
-	server, etag, err := client.GetServer()
+	config, err := client.ServerStatus()
 	if err != nil {
 		return err
 	}
 
-	brief := server.Writable()
+	brief := config.Brief()
 	data, err := yaml.Marshal(&brief)
 	if err != nil {
 		return err
@@ -701,10 +584,10 @@ func (c *configCmd) doDaemonConfigEdit(client lxd.ContainerServer) error {
 
 	for {
 		// Parse the text received from the editor
-		newdata := api.ServerPut{}
+		newdata := shared.BriefServerState{}
 		err = yaml.Unmarshal(content, &newdata)
 		if err == nil {
-			err = client.UpdateServer(newdata, etag)
+			_, err = client.UpdateServerConfig(newdata)
 		}
 
 		// Respawn the editor
@@ -725,96 +608,55 @@ func (c *configCmd) doDaemonConfigEdit(client lxd.ContainerServer) error {
 		}
 		break
 	}
-
 	return nil
 }
 
-func (c *configCmd) deviceAdd(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceAdd(config *lxd.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	remote, name, err := conf.ParseRemote(args[2])
-	if err != nil {
-		return err
-	}
-
-	client, err := conf.GetContainerServer(remote)
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
 	devname := args[3]
-	device := map[string]string{}
-	device["type"] = args[4]
+	devtype := args[4]
+	var props []string
 	if len(args) > 5 {
-		for _, prop := range args[5:] {
-			results := strings.SplitN(prop, "=", 2)
-			if len(results) != 2 {
-				return fmt.Errorf("No value found in %q", prop)
-			}
-			k := results[0]
-			v := results[1]
-			device[k] = v
-		}
-	}
-
-	if which == "profile" {
-		profile, etag, err := client.GetProfile(name)
-		if err != nil {
-			return err
-		}
-
-		_, ok := profile.Devices[devname]
-		if ok {
-			return fmt.Errorf(i18n.G("The device already exists"))
-		}
-
-		profile.Devices[devname] = device
-
-		err = client.UpdateProfile(name, profile.Writable(), etag)
-		if err != nil {
-			return err
-		}
+		props = args[5:]
 	} else {
-		container, etag, err := client.GetContainer(name)
-		if err != nil {
-			return err
-		}
-
-		_, ok := container.Devices[devname]
-		if ok {
-			return fmt.Errorf(i18n.G("The device already exists"))
-		}
-
-		container.Devices[devname] = device
-
-		op, err := client.UpdateContainer(name, container.Writable(), etag)
-		if err != nil {
-			return err
-		}
-
-		err = op.Wait()
-		if err != nil {
-			return err
-		}
+		props = []string{}
 	}
 
-	fmt.Printf(i18n.G("Device %s added to %s")+"\n", devname, name)
-	return nil
+	var resp *lxd.Response
+	if which == "profile" {
+		resp, err = client.ProfileDeviceAdd(name, devname, devtype, props)
+	} else {
+		resp, err = client.ContainerDeviceAdd(name, devname, devtype, props)
+	}
+	if err != nil {
+		return err
+	}
+	if which != "profile" {
+		err = client.WaitForSuccess(resp.Operation)
+	}
+	if err == nil {
+		fmt.Printf(i18n.G("Device %s added to %s")+"\n", devname, name)
+	}
+	return err
 }
 
-func (c *configCmd) deviceGet(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceGet(config *lxd.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
 
-	remote, name, err := conf.ParseRemote(args[2])
-	if err != nil {
-		return err
-	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	client, err := conf.GetContainerServer(remote)
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -823,24 +665,24 @@ func (c *configCmd) deviceGet(conf *config.Config, which string, args []string)
 	key := args[4]
 
 	if which == "profile" {
-		profile, _, err := client.GetProfile(name)
+		st, err := client.ProfileConfig(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := profile.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		fmt.Println(dev[key])
 	} else {
-		container, _, err := client.GetContainer(name)
+		st, err := client.ContainerInfo(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := container.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
@@ -851,17 +693,14 @@ func (c *configCmd) deviceGet(conf *config.Config, which string, args []string)
 	return nil
 }
 
-func (c *configCmd) deviceSet(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceSet(config *lxd.Config, which string, args []string) error {
 	if len(args) < 6 {
 		return errArgs
 	}
 
-	remote, name, err := conf.ParseRemote(args[2])
-	if err != nil {
-		return err
-	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	client, err := conf.GetContainerServer(remote)
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -871,62 +710,54 @@ func (c *configCmd) deviceSet(conf *config.Config, which string, args []string)
 	value := args[5]
 
 	if which == "profile" {
-		profile, etag, err := client.GetProfile(name)
+		st, err := client.ProfileConfig(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := profile.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		dev[key] = value
-		profile.Devices[devname] = dev
+		st.Devices[devname] = dev
 
-		err = client.UpdateProfile(name, profile.Writable(), etag)
+		err = client.PutProfile(name, *st)
 		if err != nil {
 			return err
 		}
 	} else {
-		container, etag, err := client.GetContainer(name)
+		st, err := client.ContainerInfo(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := container.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		dev[key] = value
-		container.Devices[devname] = dev
+		st.Devices[devname] = dev
 
-		op, err := client.UpdateContainer(name, container.Writable(), etag)
-		if err != nil {
-			return err
-		}
-
-		err = op.Wait()
+		err = client.UpdateContainerConfig(name, st.Brief())
 		if err != nil {
 			return err
 		}
 	}
 
-	return nil
+	return err
 }
 
-func (c *configCmd) deviceUnset(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceUnset(config *lxd.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
 
-	remote, name, err := conf.ParseRemote(args[2])
-	if err != nil {
-		return err
-	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	client, err := conf.GetContainerServer(remote)
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -935,181 +766,127 @@ func (c *configCmd) deviceUnset(conf *config.Config, which string, args []string
 	key := args[4]
 
 	if which == "profile" {
-		profile, etag, err := client.GetProfile(name)
+		st, err := client.ProfileConfig(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := profile.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
-		delete(dev, key)
-		profile.Devices[devname] = dev
 
-		err = client.UpdateProfile(name, profile.Writable(), etag)
+		delete(dev, key)
+		st.Devices[devname] = dev
+
+		err = client.PutProfile(name, *st)
 		if err != nil {
 			return err
 		}
 	} else {
-		container, etag, err := client.GetContainer(name)
+		st, err := client.ContainerInfo(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := container.Devices[devname]
+		dev, ok := st.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
+
 		delete(dev, key)
-		container.Devices[devname] = dev
+		st.Devices[devname] = dev
 
-		op, err := client.UpdateContainer(name, container.Writable(), etag)
-		if err != nil {
-			return err
-		}
-
-		err = op.Wait()
+		err = client.UpdateContainerConfig(name, st.Brief())
 		if err != nil {
 			return err
 		}
 	}
 
-	return nil
+	return err
 }
 
-func (c *configCmd) deviceRm(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceRm(config *lxd.Config, which string, args []string) error {
 	if len(args) < 4 {
 		return errArgs
 	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	remote, name, err := conf.ParseRemote(args[2])
-	if err != nil {
-		return err
-	}
-
-	client, err := conf.GetContainerServer(remote)
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
+	devname := args[3]
+	var resp *lxd.Response
 	if which == "profile" {
-		profile, etag, err := client.GetProfile(name)
-		if err != nil {
-			return err
-		}
-
-		for _, devname := range args[3:] {
-			_, ok := profile.Devices[devname]
-			if !ok {
-				return fmt.Errorf(i18n.G("The device doesn't exist"))
-			}
-			delete(profile.Devices, devname)
-		}
-
-		err = client.UpdateProfile(name, profile.Writable(), etag)
-		if err != nil {
-			return err
-		}
+		resp, err = client.ProfileDeviceDelete(name, devname)
 	} else {
-		container, etag, err := client.GetContainer(name)
-		if err != nil {
-			return err
-		}
-
-		for _, devname := range args[3:] {
-			_, ok := container.Devices[devname]
-			if !ok {
-				return fmt.Errorf(i18n.G("The device doesn't exist"))
-			}
-			delete(container.Devices, devname)
-		}
-
-		op, err := client.UpdateContainer(name, container.Writable(), etag)
-		if err != nil {
-			return err
-		}
-
-		err = op.Wait()
-		if err != nil {
-			return err
-		}
+		resp, err = client.ContainerDeviceDelete(name, devname)
 	}
-
-	fmt.Printf(i18n.G("Device %s removed from %s")+"\n", strings.Join(args[3:], ", "), name)
-	return nil
+	if err != nil {
+		return err
+	}
+	if which != "profile" {
+		err = client.WaitForSuccess(resp.Operation)
+	}
+	if err == nil {
+		fmt.Printf(i18n.G("Device %s removed from %s")+"\n", devname, name)
+	}
+	return err
 }
 
-func (c *configCmd) deviceList(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceList(config *lxd.Config, which string, args []string) error {
 	if len(args) < 3 {
 		return errArgs
 	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	remote, name, err := conf.ParseRemote(args[2])
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
-	client, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
-	}
-
-	var devices []string
+	var resp []string
 	if which == "profile" {
-		profile, _, err := client.GetProfile(name)
-		if err != nil {
-			return err
-		}
-
-		for k := range profile.Devices {
-			devices = append(devices, k)
-		}
+		resp, err = client.ProfileListDevices(name)
 	} else {
-		container, _, err := client.GetContainer(name)
-		if err != nil {
-			return err
-		}
-
-		for k := range container.Devices {
-			devices = append(devices, k)
-		}
+		resp, err = client.ContainerListDevices(name)
 	}
+	if err != nil {
+		return err
+	}
+	fmt.Printf("%s\n", strings.Join(resp, "\n"))
 
-	fmt.Printf("%s\n", strings.Join(devices, "\n"))
 	return nil
 }
 
-func (c *configCmd) deviceShow(conf *config.Config, which string, args []string) error {
+func (c *configCmd) deviceShow(config *lxd.Config, which string, args []string) error {
 	if len(args) < 3 {
 		return errArgs
 	}
+	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	remote, name, err := conf.ParseRemote(args[2])
+	client, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
-	client, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
-	}
-
-	var devices map[string]map[string]string
+	var devices map[string]shared.Device
 	if which == "profile" {
-		profile, _, err := client.GetProfile(name)
+		resp, err := client.ProfileConfig(name)
 		if err != nil {
 			return err
 		}
 
-		devices = profile.Devices
+		devices = resp.Devices
 	} else {
-		container, _, err := client.GetContainer(name)
+		resp, err := client.ContainerInfo(name)
 		if err != nil {
 			return err
 		}
 
-		devices = container.Devices
+		devices = resp.Devices
 	}
 
 	data, err := yaml.Marshal(&devices)
@@ -1118,5 +895,6 @@ func (c *configCmd) deviceShow(conf *config.Config, which string, args []string)
 	}
 
 	fmt.Printf(string(data))
+
 	return nil
 }
