@@ -6,10 +6,8 @@ import (
 	"os"
 	"strings"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -25,14 +23,16 @@ func (c *deleteCmd) showByDefault() bool {
 
 func (c *deleteCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc delete [<remote>:]<container>[/<snapshot>] [[<remote>:]<container>[/<snapshot>]...]
+		`Delete containers or snapshots.
 
-Delete containers and snapshots.`)
+lxc delete [<remote>:]<container>[/<snapshot>] [[<remote>:]<container>[/<snapshot>]...]
+
+Destroy containers or snapshots with any attached data (configuration, snapshots, ...).`)
 }
 
 func (c *deleteCmd) flags() {
-	gnuflag.BoolVar(&c.force, "f", false, i18n.G("Force the removal of running containers"))
-	gnuflag.BoolVar(&c.force, "force", false, i18n.G("Force the removal of running containers"))
+	gnuflag.BoolVar(&c.force, "f", false, i18n.G("Force the removal of stopped containers"))
+	gnuflag.BoolVar(&c.force, "force", false, i18n.G("Force the removal of stopped containers"))
 	gnuflag.BoolVar(&c.interactive, "i", false, i18n.G("Require user confirmation"))
 	gnuflag.BoolVar(&c.interactive, "interactive", false, i18n.G("Require user confirmation"))
 }
@@ -49,37 +49,24 @@ func (c *deleteCmd) promptDelete(name string) error {
 	return nil
 }
 
-func (c *deleteCmd) doDelete(d lxd.ContainerServer, name string) error {
-	var op *lxd.Operation
-	var err error
-
-	if shared.IsSnapshot(name) {
-		// Snapshot delete
-		fields := strings.SplitN(name, shared.SnapshotDelimiter, 2)
-		op, err = d.DeleteContainerSnapshot(fields[0], fields[1])
-	} else {
-		// Container delete
-		op, err = d.DeleteContainer(name)
-	}
+func (c *deleteCmd) doDelete(d *lxd.Client, name string) error {
+	resp, err := d.Delete(name)
 	if err != nil {
 		return err
 	}
 
-	return op.Wait()
+	return d.WaitForSuccess(resp.Operation)
 }
 
-func (c *deleteCmd) run(conf *config.Config, args []string) error {
+func (c *deleteCmd) run(config *lxd.Config, args []string) error {
 	if len(args) == 0 {
 		return errArgs
 	}
 
 	for _, nameArg := range args {
-		remote, name, err := conf.ParseRemote(nameArg)
-		if err != nil {
-			return err
-		}
+		remote, name := config.ParseRemoteAndContainer(nameArg)
 
-		d, err := conf.GetContainerServer(remote)
+		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
@@ -95,30 +82,28 @@ func (c *deleteCmd) run(conf *config.Config, args []string) error {
 			return c.doDelete(d, name)
 		}
 
-		ct, _, err := d.GetContainer(name)
+		ct, err := d.ContainerInfo(name)
 		if err != nil {
 			return err
 		}
 
-		if ct.StatusCode != 0 && ct.StatusCode != api.Stopped {
+		if ct.StatusCode != 0 && ct.StatusCode != shared.Stopped {
 			if !c.force {
 				return fmt.Errorf(i18n.G("The container is currently running, stop it first or pass --force."))
 			}
 
-			req := api.ContainerStatePut{
-				Action:  "stop",
-				Timeout: -1,
-				Force:   true,
-			}
-
-			op, err := d.UpdateContainerState(name, req, "")
+			resp, err := d.Action(name, shared.Stop, -1, true, false)
 			if err != nil {
 				return err
 			}
 
-			err = op.Wait()
+			op, err := d.WaitFor(resp.Operation)
 			if err != nil {
-				return fmt.Errorf(i18n.G("Stopping the container failed: %s"), err)
+				return err
+			}
+
+			if op.StatusCode == shared.Failure {
+				return fmt.Errorf(i18n.G("Stopping container failed!"))
 			}
 
 			if ct.Ephemeral == true {
