@@ -10,24 +10,22 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/termios"
 )
 
-type envList []string
+type envFlag []string
 
-func (f *envList) String() string {
+func (f *envFlag) String() string {
 	return fmt.Sprint(*f)
 }
 
-func (f *envList) Set(value string) error {
+func (f *envFlag) Set(value string) error {
 	if f == nil {
-		*f = make(envList, 1)
+		*f = make(envFlag, 1)
 	} else {
 		*f = append(*f, value)
 	}
@@ -36,7 +34,7 @@ func (f *envList) Set(value string) error {
 
 type execCmd struct {
 	modeFlag string
-	envArgs  envList
+	envArgs  envFlag
 }
 
 func (c *execCmd) showByDefault() bool {
@@ -45,9 +43,9 @@ func (c *execCmd) showByDefault() bool {
 
 func (c *execCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc exec [<remote>:]<container> [--mode=auto|interactive|non-interactive] [--env KEY=VALUE...] [--] <command line>
+		`Execute the specified command in a container.
 
-Execute commands in containers.
+lxc exec [<remote>:]<container> [--mode=auto|interactive|non-interactive] [--env KEY=VALUE...] [--] <command line>
 
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`)
 }
@@ -63,14 +61,14 @@ func (c *execCmd) sendTermSize(control *websocket.Conn) error {
 		return err
 	}
 
-	logger.Debugf("Window size is now: %dx%d", width, height)
+	shared.LogDebugf("Window size is now: %dx%d", width, height)
 
 	w, err := control.NextWriter(websocket.TextMessage)
 	if err != nil {
 		return err
 	}
 
-	msg := api.ContainerExecControl{}
+	msg := shared.ContainerExecControl{}
 	msg.Command = "window-resize"
 	msg.Args = make(map[string]string)
 	msg.Args["width"] = strconv.Itoa(width)
@@ -86,24 +84,39 @@ func (c *execCmd) sendTermSize(control *websocket.Conn) error {
 	return err
 }
 
-func (c *execCmd) run(conf *config.Config, args []string) error {
+func (c *execCmd) forwardSignal(control *websocket.Conn, sig syscall.Signal) error {
+	shared.LogDebugf("Forwarding signal: %s", sig)
+
+	w, err := control.NextWriter(websocket.TextMessage)
+	if err != nil {
+		return err
+	}
+
+	msg := shared.ContainerExecControl{}
+	msg.Command = "signal"
+	msg.Signal = sig
+
+	buf, err := json.Marshal(msg)
+	if err != nil {
+		return err
+	}
+	_, err = w.Write(buf)
+
+	w.Close()
+	return err
+}
+
+func (c *execCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
 
-	remote, name, err := conf.ParseRemote(args[0])
+	remote, name := config.ParseRemoteAndContainer(args[0])
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
-	d, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
-	}
-
-	/* FIXME: Default values for HOME and USER are now handled by LXD.
-	   This code should be removed after most users upgraded.
-	*/
 	env := map[string]string{"HOME": "/root", "USER": "root"}
 	if myTerm, ok := c.getTERM(); ok {
 		env["TERM"] = myTerm
@@ -152,39 +165,11 @@ func (c *execCmd) run(conf *config.Config, args []string) error {
 	}
 
 	stdout := c.getStdout()
-
-	req := api.ContainerExecPost{
-		Command:     args[1:],
-		WaitForWS:   true,
-		Interactive: interactive,
-		Environment: env,
-		Width:       width,
-		Height:      height,
-	}
-
-	execArgs := lxd.ContainerExecArgs{
-		Stdin:    os.Stdin,
-		Stdout:   stdout,
-		Stderr:   os.Stderr,
-		Control:  handler,
-		DataDone: make(chan bool),
-	}
-
-	// Run the command in the container
-	op, err := d.ExecContainer(name, req, &execArgs)
+	ret, err := d.Exec(name, args[1:], env, os.Stdin, stdout, os.Stderr, handler, width, height)
 	if err != nil {
 		return err
 	}
 
-	// Wait for the operation to complete
-	err = op.Wait()
-	if err != nil {
-		return err
-	}
-
-	// Wait for any remaining I/O to be flushed
-	<-execArgs.DataDone
-
 	if oldttystate != nil {
 		/* A bit of a special case here: we want to exit with the same code as
 		 * the process inside the container, so we explicitly exit here
@@ -196,6 +181,6 @@ func (c *execCmd) run(conf *config.Config, args []string) error {
 		termios.Restore(cfd, oldttystate)
 	}
 
-	os.Exit(int(op.Metadata["return"].(float64)))
-	return nil
+	os.Exit(ret)
+	return fmt.Errorf(i18n.G("unreachable return reached"))
 }
