@@ -11,8 +11,7 @@ import (
 	"strings"
 	"syscall"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
@@ -23,6 +22,10 @@ type fileCmd struct {
 	uid  int
 	gid  int
 	mode string
+
+	recursive bool
+
+	mkdirs bool
 }
 
 func (c *fileCmd) showByDefault() bool {
@@ -31,34 +34,33 @@ func (c *fileCmd) showByDefault() bool {
 
 func (c *fileCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc file <subcommand> [options]
-
-Manage files in containers.
-
-lxc file pull [<remote>:]<container>/<path> [[<remote>:]<container>/<path>...] <target path>
-    Pull files from containers.
-
-lxc file push [--uid=UID] [--gid=GID] [--mode=MODE] <source path> [<source path>...] [<remote>:]<container>/<path>
-    Push files into containers.
+		`Manage files in a container.
 
+lxc file pull [-r|--recursive] [<remote>:]<container> [[<remote>:]<container>...] <target path>
+lxc file push [-r|--recursive] [-p|--create-dirs] [--uid=UID] [--gid=GID] [--mode=MODE] <source path> [<source path>...] [<remote>:]<container>
 lxc file edit [<remote>:]<container>/<path>
-    Edit files in containers using the default text editor.
 
-*Examples*
-lxc file push /etc/hosts foo/etc/hosts
-   To push /etc/hosts into the container "foo".
+<source> in the case of pull, <target> in the case of push and <file> in the case of edit are <container name>/<path>
 
-lxc file pull foo/etc/hosts .
-   To pull /etc/hosts from the container and write it to the current directory.`)
+Examples:
+To push /etc/hosts into the container foo:
+    lxc file push /etc/hosts foo/etc/hosts
+
+To pull /etc/hosts from the container:
+    lxc file pull foo/etc/hosts .`)
 }
 
 func (c *fileCmd) flags() {
 	gnuflag.IntVar(&c.uid, "uid", -1, i18n.G("Set the file's uid on push"))
 	gnuflag.IntVar(&c.gid, "gid", -1, i18n.G("Set the file's gid on push"))
 	gnuflag.StringVar(&c.mode, "mode", "", i18n.G("Set the file's perms on push"))
+	gnuflag.BoolVar(&c.recursive, "recursive", false, i18n.G("Recursively push or pull files"))
+	gnuflag.BoolVar(&c.recursive, "r", false, i18n.G("Recursively push or pull files"))
+	gnuflag.BoolVar(&c.mkdirs, "create-dirs", false, i18n.G("Create any directories necessary"))
+	gnuflag.BoolVar(&c.mkdirs, "p", false, i18n.G("Create any directories necessary"))
 }
 
-func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string) error {
+func (c *fileCmd) push(config *lxd.Config, send_file_perms bool, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
@@ -70,15 +72,33 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 		return fmt.Errorf(i18n.G("Invalid target %s"), target)
 	}
 
-	targetPath := pathSpec[1]
-	remote, container, err := conf.ParseRemote(pathSpec[0])
+	remote, container := config.ParseRemoteAndContainer(pathSpec[0])
+
+	targetIsDir := strings.HasSuffix(target, "/")
+	// re-add leading / that got stripped by the SplitN
+	targetPath := "/" + pathSpec[1]
+	// clean various /./, /../, /////, etc. that users add (#2557)
+	targetPath, err := filepath.Abs(targetPath)
+	if err != nil {
+		return fmt.Errorf(i18n.G("Could not sanitize path %s"), targetPath)
+	}
+	// normalization may reveal that path is still a dir, e.g. /.
+	if strings.HasSuffix(targetPath, "/") {
+		targetIsDir = true
+	}
+
+	shared.LogDebugf("Pushing to: %s  (isdir: %t)", targetPath, targetIsDir)
+
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
-	d, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
+	var sourcefilenames []string
+	for _, fname := range args[:len(args)-1] {
+		if !strings.HasPrefix(fname, "--") {
+			sourcefilenames = append(sourcefilenames, fname)
+		}
 	}
 
 	mode := os.FileMode(0755)
@@ -94,6 +114,26 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 		mode = os.FileMode(m)
 	}
 
+	if c.recursive {
+		if c.uid != -1 || c.gid != -1 || c.mode != "" {
+			return fmt.Errorf(i18n.G("can't supply uid/gid/mode in recursive mode"))
+		}
+
+		if c.mkdirs {
+			if err := d.MkdirP(container, targetPath, mode); err != nil {
+				return err
+			}
+		}
+
+		for _, fname := range sourcefilenames {
+			if err := d.RecursivePushFile(container, fname, targetPath); err != nil {
+				return err
+			}
+		}
+
+		return nil
+	}
+
 	uid := 0
 	if c.uid >= 0 {
 		uid = c.uid
@@ -104,16 +144,7 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 		gid = c.gid
 	}
 
-	_, targetfilename := filepath.Split(targetPath)
-
-	var sourcefilenames []string
-	for _, fname := range args[:len(args)-1] {
-		if !strings.HasPrefix(fname, "--") {
-			sourcefilenames = append(sourcefilenames, shared.HostPath(filepath.Clean(fname)))
-		}
-	}
-
-	if (targetfilename != "") && (len(sourcefilenames) > 1) {
+	if (len(sourcefilenames) > 1) && !targetIsDir {
 		return errArgs
 	}
 
@@ -137,15 +168,14 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 
 	for _, f := range files {
 		fpath := targetPath
-		if targetfilename == "" {
+		if targetIsDir {
 			fpath = path.Join(fpath, path.Base(f.Name()))
 		}
 
-		args := lxd.ContainerFileArgs{
-			Content: f,
-			UID:     -1,
-			GID:     -1,
-			Mode:    -1,
+		if c.mkdirs {
+			if err := d.MkdirP(container, path.Dir(fpath), mode); err != nil {
+				return err
+			}
 		}
 
 		if send_file_perms {
@@ -168,12 +198,11 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 				}
 			}
 
-			args.UID = int64(uid)
-			args.GID = int64(gid)
-			args.Mode = int(mode.Perm())
+			err = d.PushFile(container, fpath, gid, uid, fmt.Sprintf("%04o", mode.Perm()), f)
+		} else {
+			err = d.PushFile(container, fpath, -1, -1, "", f)
 		}
 
-		err = d.CreateContainerFile(container, fpath, args)
 		if err != nil {
 			return err
 		}
@@ -182,12 +211,12 @@ func (c *fileCmd) push(conf *config.Config, send_file_perms bool, args []string)
 	return nil
 }
 
-func (c *fileCmd) pull(conf *config.Config, args []string) error {
+func (c *fileCmd) pull(config *lxd.Config, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
 
-	target := shared.HostPath(filepath.Clean(args[len(args)-1]))
+	target := args[len(args)-1]
 	targetIsDir := false
 	sb, err := os.Stat(target)
 	if err != nil && !os.IsNotExist(err) {
@@ -196,17 +225,18 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 
 	/*
 	 * If the path exists, just use it. If it doesn't exist, it might be a
-	 * directory in one of two cases:
+	 * directory in one of three cases:
 	 *   1. Someone explicitly put "/" at the end
 	 *   2. Someone provided more than one source. In this case the target
 	 *      should be a directory so we can save all the files into it.
+	 *   3. We are dealing with recursive copy
 	 */
 	if err == nil {
 		targetIsDir = sb.IsDir()
 		if !targetIsDir && len(args)-1 > 1 {
 			return fmt.Errorf(i18n.G("More than one file to download, but target is not a directory"))
 		}
-	} else if strings.HasSuffix(target, string(os.PathSeparator)) || len(args)-1 > 1 {
+	} else if strings.HasSuffix(target, string(os.PathSeparator)) || len(args)-1 > 1 || c.recursive {
 		if err := os.MkdirAll(target, 0755); err != nil {
 			return err
 		}
@@ -219,19 +249,27 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 			return fmt.Errorf(i18n.G("Invalid source %s"), f)
 		}
 
-		remote, container, err := conf.ParseRemote(pathSpec[0])
+		remote, container := config.ParseRemoteAndContainer(pathSpec[0])
+		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
 
-		d, err := conf.GetContainerServer(remote)
+		if c.recursive {
+			if err := d.RecursivePullFile(container, pathSpec[1], target); err != nil {
+				return err
+			}
+
+			continue
+		}
+
+		_, _, mode, type_, buf, _, err := d.PullFile(container, pathSpec[1])
 		if err != nil {
 			return err
 		}
 
-		buf, _, err := d.GetContainerFile(container, pathSpec[1])
-		if err != nil {
-			return err
+		if type_ == "directory" {
+			return fmt.Errorf(i18n.G("can't pull a directory without --recursive"))
 		}
 
 		var targetPath string
@@ -250,6 +288,11 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 				return err
 			}
 			defer f.Close()
+
+			err = f.Chmod(os.FileMode(mode))
+			if err != nil {
+				return err
+			}
 		}
 
 		_, err = io.Copy(f, buf)
@@ -261,14 +304,18 @@ func (c *fileCmd) pull(conf *config.Config, args []string) error {
 	return nil
 }
 
-func (c *fileCmd) edit(conf *config.Config, args []string) error {
+func (c *fileCmd) edit(config *lxd.Config, args []string) error {
 	if len(args) != 1 {
 		return errArgs
 	}
 
+	if c.recursive {
+		return fmt.Errorf(i18n.G("recursive edit doesn't make sense :("))
+	}
+
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(int(syscall.Stdin)) {
-		return c.push(conf, false, append([]string{os.Stdin.Name()}, args[0]))
+		return c.push(config, false, append([]string{os.Stdin.Name()}, args[0]))
 	}
 
 	// Create temp file
@@ -279,7 +326,7 @@ func (c *fileCmd) edit(conf *config.Config, args []string) error {
 	defer os.Remove(fname)
 
 	// Extract current value
-	err = c.pull(conf, append([]string{args[0]}, fname))
+	err = c.pull(config, append([]string{args[0]}, fname))
 	if err != nil {
 		return err
 	}
@@ -289,7 +336,7 @@ func (c *fileCmd) edit(conf *config.Config, args []string) error {
 		return err
 	}
 
-	err = c.push(conf, false, append([]string{fname}, args[0]))
+	err = c.push(config, false, append([]string{fname}, args[0]))
 	if err != nil {
 		return err
 	}
@@ -297,18 +344,18 @@ func (c *fileCmd) edit(conf *config.Config, args []string) error {
 	return nil
 }
 
-func (c *fileCmd) run(conf *config.Config, args []string) error {
+func (c *fileCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
-		return errUsage
+		return errArgs
 	}
 
 	switch args[0] {
 	case "push":
-		return c.push(conf, true, args[1:])
+		return c.push(config, true, args[1:])
 	case "pull":
-		return c.pull(conf, args[1:])
+		return c.pull(config, args[1:])
 	case "edit":
-		return c.edit(conf, args[1:])
+		return c.edit(config, args[1:])
 	default:
 		return errArgs
 	}
