@@ -5,9 +5,8 @@ import (
 	"os"
 	"strings"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -61,10 +60,10 @@ func (f *profileList) Set(value string) error {
 var initRequestedEmptyProfiles bool
 
 type initCmd struct {
-	profArgs     profileList
-	confArgs     configList
-	ephem        bool
-	instanceType string
+	profArgs profileList
+	confArgs configList
+	ephem    bool
+	network  string
 }
 
 func (c *initCmd) showByDefault() bool {
@@ -73,14 +72,16 @@ func (c *initCmd) showByDefault() bool {
 
 func (c *initCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc init [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--type|-t <instance type>]
+		`Initialize a container from a particular image.
 
-Create containers from images.
+lxc init [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--network|-n <network>]
+
+Initializes a container using the specified image and name.
 
 Not specifying -p will result in the default profile.
 Specifying "-p" with no argument will result in no profile.
 
-Examples:
+Example:
     lxc init ubuntu:16.04 u1`)
 }
 
@@ -125,7 +126,6 @@ func (c *initCmd) massage_args() {
 		initRequestedEmptyProfiles = true
 		newargs := os.Args[0 : l-2]
 		newargs = append(newargs, os.Args[l-1])
-		os.Args = newargs
 		return
 	}
 }
@@ -138,43 +138,32 @@ func (c *initCmd) flags() {
 	gnuflag.Var(&c.profArgs, "p", i18n.G("Profile to apply to the new container"))
 	gnuflag.BoolVar(&c.ephem, "ephemeral", false, i18n.G("Ephemeral container"))
 	gnuflag.BoolVar(&c.ephem, "e", false, i18n.G("Ephemeral container"))
-	gnuflag.StringVar(&c.instanceType, "t", "", i18n.G("Instance type"))
+	gnuflag.StringVar(&c.network, "network", "", i18n.G("Network name"))
+	gnuflag.StringVar(&c.network, "n", "", i18n.G("Network name"))
 }
 
-func (c *initCmd) run(conf *config.Config, args []string) error {
-	_, _, err := c.create(conf, args)
-	return err
-}
-
-func (c *initCmd) create(conf *config.Config, args []string) (lxd.ContainerServer, string, error) {
+func (c *initCmd) run(config *lxd.Config, args []string) error {
 	if len(args) > 2 || len(args) < 1 {
-		return nil, "", errArgs
+		return errArgs
 	}
 
-	iremote, image, err := conf.ParseRemote(args[0])
-	if err != nil {
-		return nil, "", err
-	}
+	iremote, image := config.ParseRemoteAndContainer(args[0])
 
 	var name string
 	var remote string
 	if len(args) == 2 {
-		remote, name, err = conf.ParseRemote(args[1])
-		if err != nil {
-			return nil, "", err
-		}
+		remote, name = config.ParseRemoteAndContainer(args[1])
 	} else {
-		remote, name, err = conf.ParseRemote("")
-		if err != nil {
-			return nil, "", err
-		}
+		remote, name = config.ParseRemoteAndContainer("")
 	}
 
-	d, err := conf.GetContainerServer(remote)
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
-		return nil, "", err
+		return err
 	}
 
+	// TODO: implement the syntax for supporting other image types/remotes
+
 	/*
 	 * initRequestedEmptyProfiles means user requested empty
 	 * !initRequestedEmptyProfiles but len(profArgs) == 0 means use profile default
@@ -184,123 +173,122 @@ func (c *initCmd) create(conf *config.Config, args []string) (lxd.ContainerServe
 		profiles = append(profiles, p)
 	}
 
+	var resp *lxd.Response
 	if name == "" {
 		fmt.Printf(i18n.G("Creating the container") + "\n")
 	} else {
 		fmt.Printf(i18n.G("Creating %s")+"\n", name)
 	}
 
-	// Get the image server and image info
-	iremote, image = c.guessImage(conf, d, remote, iremote, image)
-	var imgRemote lxd.ImageServer
-	var imgInfo *api.Image
+	iremote, image = c.guessImage(config, d, remote, iremote, image)
 
-	// Connect to the image server
-	if iremote == remote {
-		imgRemote = d
-	} else {
-		imgRemote, err = conf.GetImageServer(iremote)
+	devicesMap := map[string]shared.Device{}
+	if c.network != "" {
+		network, err := d.NetworkGet(c.network)
 		if err != nil {
-			return nil, "", err
+			return err
+		}
+
+		if network.Type == "bridge" {
+			devicesMap[c.network] = shared.Device{"type": "nic", "nictype": "bridged", "parent": c.network}
+		} else {
+			devicesMap[c.network] = shared.Device{"type": "nic", "nictype": "macvlan", "parent": c.network}
 		}
 	}
 
-	// Deal with the default image
-	if image == "" {
-		image = "default"
-	}
-
-	// Setup container creation request
-	req := api.ContainersPost{
-		Name:         name,
-		InstanceType: c.instanceType,
-	}
-	req.Config = configMap
 	if !initRequestedEmptyProfiles && len(profiles) == 0 {
-		req.Profiles = nil
+		resp, err = d.Init(name, iremote, image, nil, configMap, devicesMap, c.ephem)
 	} else {
-		req.Profiles = profiles
+		resp, err = d.Init(name, iremote, image, &profiles, configMap, devicesMap, c.ephem)
 	}
-	req.Ephemeral = c.ephem
-
-	// Optimisation for simplestreams
-	if conf.Remotes[iremote].Protocol == "simplestreams" {
-		imgInfo = &api.Image{}
-		imgInfo.Fingerprint = image
-		imgInfo.Public = true
-		req.Source.Alias = image
-	} else {
-		// Attempt to resolve an image alias
-		alias, _, err := imgRemote.GetImageAlias(image)
-		if err == nil {
-			req.Source.Alias = image
-			image = alias.Target
-		}
-
-		// Get the image info
-		imgInfo, _, err = imgRemote.GetImage(image)
-		if err != nil {
-			return nil, "", err
-		}
-	}
-
-	// Create the container
-	op, err := d.CreateContainerFromImage(imgRemote, *imgInfo, req)
 	if err != nil {
-		return nil, "", err
+		return err
 	}
 
-	// Watch the background operation
-	progress := ProgressRenderer{Format: i18n.G("Retrieving image: %s")}
-	_, err = op.AddHandler(progress.UpdateOp)
-	if err != nil {
-		progress.Done("")
-		return nil, "", err
-	}
+	progress := ProgressRenderer{}
+	c.initProgressTracker(d, &progress, resp.Operation)
 
-	err = cancelableWait(op, &progress)
-	if err != nil {
-		progress.Done("")
-		return nil, "", err
-	}
+	err = d.WaitForSuccess(resp.Operation)
 	progress.Done("")
 
-	// Extract the container name
-	opInfo, err := op.GetTarget()
 	if err != nil {
-		return nil, "", err
+		return err
+	} else {
+		op, err := resp.MetadataAsOperation()
+		if err != nil {
+			return fmt.Errorf(i18n.G("didn't get any affected image, container or snapshot from server"))
+		}
+
+		containers, ok := op.Resources["containers"]
+		if !ok || len(containers) == 0 {
+			return fmt.Errorf(i18n.G("didn't get any affected image, container or snapshot from server"))
+		}
+
+		if len(containers) == 1 && name == "" {
+			fields := strings.Split(containers[0], "/")
+			fmt.Printf(i18n.G("Container name is: %s")+"\n", fields[len(fields)-1])
+		}
 	}
 
-	containers, ok := opInfo.Resources["containers"]
-	if !ok || len(containers) == 0 {
-		return nil, "", fmt.Errorf(i18n.G("didn't get any affected image, container or snapshot from server"))
-	}
+	c.checkNetwork(d, name)
 
-	if len(containers) == 1 && name == "" {
-		fields := strings.Split(containers[0], "/")
-		name = fields[len(fields)-1]
-		fmt.Printf(i18n.G("Container name is: %s")+"\n", name)
-	}
-
-	return d, name, nil
+	return nil
 }
 
-func (c *initCmd) guessImage(conf *config.Config, d lxd.ContainerServer, remote string, iremote string, image string) (string, string) {
+func (c *initCmd) initProgressTracker(d *lxd.Client, progress *ProgressRenderer, operation string) {
+	progress.Format = i18n.G("Retrieving image: %s")
+	handler := func(msg interface{}) {
+		if msg == nil {
+			return
+		}
+
+		event := msg.(map[string]interface{})
+		if event["type"].(string) != "operation" {
+			return
+		}
+
+		if event["metadata"] == nil {
+			return
+		}
+
+		md := event["metadata"].(map[string]interface{})
+		if !strings.HasSuffix(operation, md["id"].(string)) {
+			return
+		}
+
+		if md["metadata"] == nil {
+			return
+		}
+
+		if shared.StatusCode(md["status_code"].(float64)).IsFinal() {
+			return
+		}
+
+		opMd := md["metadata"].(map[string]interface{})
+		_, ok := opMd["download_progress"]
+		if ok {
+			progress.Update(opMd["download_progress"].(string))
+		}
+	}
+	go d.Monitor([]string{"operation"}, handler)
+}
+
+func (c *initCmd) guessImage(config *lxd.Config, d *lxd.Client, remote string, iremote string, image string) (string, string) {
 	if remote != iremote {
 		return iremote, image
 	}
 
-	_, ok := conf.Remotes[image]
+	_, ok := config.Remotes[image]
 	if !ok {
 		return iremote, image
 	}
 
-	_, _, err := d.GetImageAlias(image)
-	if err == nil {
+	target := d.GetAlias(image)
+	if target != "" {
 		return iremote, image
 	}
 
-	_, _, err = d.GetImage(image)
+	_, err := d.GetImageInfo(image)
 	if err == nil {
 		return iremote, image
 	}
@@ -308,3 +296,20 @@ func (c *initCmd) guessImage(conf *config.Config, d lxd.ContainerServer, remote
 	fmt.Fprintf(os.Stderr, i18n.G("The local image '%s' couldn't be found, trying '%s:' instead.")+"\n", image, image)
 	return image, "default"
 }
+
+func (c *initCmd) checkNetwork(d *lxd.Client, name string) {
+	ct, err := d.ContainerInfo(name)
+	if err != nil {
+		return
+	}
+
+	for _, d := range ct.ExpandedDevices {
+		if d["type"] == "nic" {
+			return
+		}
+	}
+
+	fmt.Fprintf(os.Stderr, "\n"+i18n.G("The container you are starting doesn't have any network attached to it.")+"\n")
+	fmt.Fprintf(os.Stderr, "  "+i18n.G("To create a new network, use: lxc network create")+"\n")
+	fmt.Fprintf(os.Stderr, "  "+i18n.G("To attach a network to a container, use: lxc network attach")+"\n\n")
+}
