@@ -4,31 +4,24 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
-	"os/user"
 	"path"
-	"path/filepath"
 	"strings"
 	"syscall"
 
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
-	"github.com/lxc/lxd/shared/version"
 )
 
 var configPath string
-var execName string
 
 func main() {
-	execName = os.Args[0]
-
 	if err := run(); err != nil {
 		msg := fmt.Sprintf(i18n.G("error: %v"), err)
 
-		lxdErr := getLocalErr(err)
+		lxdErr := lxd.GetLocalLXDErr(err)
 		switch lxdErr {
 		case syscall.ENOENT:
 			msg = i18n.G("LXD socket not found; is LXD installed and running?")
@@ -49,18 +42,9 @@ func run() error {
 	forceLocal := gnuflag.Bool("force-local", false, i18n.G("Force using the local unix socket"))
 	noAlias := gnuflag.Bool("no-alias", false, i18n.G("Ignore aliases when determining what command to run"))
 
-	var configDir string
+	configDir := "$HOME/.config/lxc"
 	if os.Getenv("LXD_CONF") != "" {
 		configDir = os.Getenv("LXD_CONF")
-	} else if os.Getenv("HOME") != "" {
-		configDir = path.Join(os.Getenv("HOME"), ".config", "lxc")
-	} else {
-		user, err := user.Current()
-		if err != nil {
-			return err
-		}
-
-		configDir = path.Join(user.HomeDir, ".config", "lxc")
 	}
 	configPath = os.ExpandEnv(path.Join(configDir, "config.yml"))
 
@@ -74,11 +58,16 @@ func run() error {
 	}
 
 	if len(os.Args) >= 2 && (os.Args[1] == "--all") {
-		os.Args = []string{os.Args[0], "help", "--all"}
+		os.Args[1] = "help"
+		os.Args = append(os.Args, "--all")
 	}
 
-	if shared.StringInSlice("--version", os.Args) {
-		os.Args = []string{os.Args[0], "version"}
+	if len(os.Args) == 2 && os.Args[1] == "--version" {
+		os.Args[1] = "version"
+	}
+
+	if len(os.Args) == 2 && os.Args[1] == "--man" {
+		os.Args[1] = "manpage"
 	}
 
 	if len(os.Args) < 2 {
@@ -86,24 +75,18 @@ func run() error {
 		os.Exit(1)
 	}
 
-	var conf *config.Config
+	var config *lxd.Config
 	var err error
 
 	if *forceLocal {
-		conf = &config.DefaultConfig
-	} else if shared.PathExists(configPath) {
-		conf, err = config.LoadConfig(configPath)
+		config = &lxd.DefaultConfig
+	} else {
+		config, err = lxd.LoadConfig(configPath)
 		if err != nil {
 			return err
 		}
-	} else {
-		conf = &config.DefaultConfig
-		conf.ConfigDir = filepath.Dir(configPath)
 	}
 
-	// Set the user agent
-	conf.UserAgent = version.UserAgent
-
 	// This is quite impolite, but it seems gnuflag needs us to shift our
 	// own exename out of the arguments before parsing them. However, this
 	// is useful for execIfAlias, which wants to know exactly the command
@@ -116,7 +99,7 @@ func run() error {
 	 * --no-alias by hand.
 	 */
 	if !shared.StringInSlice("--no-alias", origArgs) {
-		execIfAliases(conf, origArgs)
+		execIfAliases(config, origArgs)
 	}
 	cmd, ok := commands[name]
 	if !ok {
@@ -126,69 +109,45 @@ func run() error {
 	}
 	cmd.flags()
 	gnuflag.Usage = func() {
-		fmt.Print(cmd.usage())
-		fmt.Printf("\n\n%s\n", i18n.G("Options:"))
-
-		gnuflag.SetOut(os.Stdout)
+		fmt.Fprintf(os.Stderr, i18n.G("Usage: %s")+"\n\n"+i18n.G("Options:")+"\n\n", strings.TrimSpace(cmd.usage()))
 		gnuflag.PrintDefaults()
-		os.Exit(0)
 	}
 
 	os.Args = os.Args[1:]
 	gnuflag.Parse(true)
 
-	logger.Log, err = logging.GetLogger("", "", *verbose, *debug, nil)
+	shared.Log, err = logging.GetLogger("", "", *verbose, *debug, nil)
 	if err != nil {
 		return err
 	}
 
-	certf := conf.ConfigPath("client.crt")
-	keyf := conf.ConfigPath("client.key")
+	// If the user is running a command that may attempt to connect to the local daemon
+	// and this is the first time the client has been run by the user, then check to see
+	// if LXD has been properly configured.  Don't display the message if the var path
+	// does not exist (LXD not installed), as the user may be targeting a remote daemon.
+	if os.Args[0] != "help" && os.Args[0] != "version" && shared.PathExists(shared.VarPath("")) && !shared.PathExists(config.ConfigDir) {
 
-	if !*forceLocal && os.Args[0] != "help" && os.Args[0] != "version" && (!shared.PathExists(certf) || !shared.PathExists(keyf)) {
-		fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
-
-		err = shared.FindOrGenCert(certf, keyf, true)
+		// Create the config dir so that we don't get in here again for this user.
+		err = os.MkdirAll(config.ConfigDir, 0750)
 		if err != nil {
 			return err
 		}
 
-		if shared.PathExists("/var/lib/lxd/") {
-			fmt.Fprintf(os.Stderr, i18n.G("If this is your first time using LXD, you should also run: sudo lxd init")+"\n")
-			fmt.Fprintf(os.Stderr, i18n.G("To start your first container, try: lxc launch ubuntu:16.04")+"\n\n")
-		}
+		fmt.Fprintf(os.Stderr, i18n.G("If this is your first time using LXD, you should also run: sudo lxd init")+"\n")
+		fmt.Fprintf(os.Stderr, i18n.G("To start your first container, try: lxc launch ubuntu:16.04")+"\n\n")
 	}
 
-	err = cmd.run(conf, gnuflag.Args())
-	if err == errArgs || err == errUsage {
-		out := os.Stdout
-		if err == errArgs {
-			/* If we got an error about invalid arguments, let's try to
-			 * expand this as an alias
-			 */
-			if !*noAlias {
-				execIfAliases(conf, origArgs)
-			}
-
-			out = os.Stderr
+	err = cmd.run(config, gnuflag.Args())
+	if err == errArgs {
+		/* If we got an error about invalid arguments, let's try to
+		 * expand this as an alias
+		 */
+		if !*noAlias {
+			execIfAliases(config, origArgs)
 		}
-		gnuflag.SetOut(out)
-
-		if err == errArgs {
-			fmt.Fprintf(out, i18n.G("error: %v"), err)
-			fmt.Fprintf(out, "\n\n")
-		}
-		fmt.Fprint(out, cmd.usage())
-		fmt.Fprintf(out, "\n\n%s\n", i18n.G("Options:"))
-
-		gnuflag.PrintDefaults()
-
-		if err == errArgs {
-			os.Exit(1)
-		}
-		os.Exit(0)
+		fmt.Fprintf(os.Stderr, "%s\n\n"+i18n.G("error: %v")+"\n", cmd.usage(), err)
+		os.Exit(1)
 	}
-
 	return err
 }
 
@@ -196,7 +155,7 @@ type command interface {
 	usage() string
 	flags()
 	showByDefault() bool
-	run(config *config.Config, args []string) error
+	run(config *lxd.Config, args []string) error
 }
 
 var commands = map[string]command{
@@ -212,52 +171,76 @@ var commands = map[string]command{
 	"init":    &initCmd{},
 	"launch":  &launchCmd{},
 	"list":    &listCmd{},
+	"manpage": &manpageCmd{},
 	"monitor": &monitorCmd{},
 	"move":    &moveCmd{},
+	"network": &networkCmd{},
 	"pause": &actionCmd{
-		action:      shared.Freeze,
-		description: i18n.G("Pause containers."),
-		name:        "pause",
+		action:         shared.Freeze,
+		name:           "pause",
+		additionalHelp: i18n.G("The opposite of `lxc pause` is `lxc start`."),
 	},
 	"profile": &profileCmd{},
 	"publish": &publishCmd{},
 	"remote":  &remoteCmd{},
 	"restart": &actionCmd{
-		action:      shared.Restart,
-		description: i18n.G("Restart containers."),
-		hasTimeout:  true,
-		visible:     true,
-		name:        "restart",
-		timeout:     -1,
+		action:     shared.Restart,
+		hasTimeout: true,
+		visible:    true,
+		name:       "restart",
+		timeout:    -1,
 	},
 	"restore":  &restoreCmd{},
 	"snapshot": &snapshotCmd{},
 	"start": &actionCmd{
-		action:      shared.Start,
-		description: i18n.G("Start containers."),
-		visible:     true,
-		name:        "start",
+		action:  shared.Start,
+		visible: true,
+		name:    "start",
 	},
 	"stop": &actionCmd{
-		action:      shared.Stop,
-		description: i18n.G("Stop containers."),
-		hasTimeout:  true,
-		visible:     true,
-		name:        "stop",
-		timeout:     -1,
+		action:     shared.Stop,
+		hasTimeout: true,
+		visible:    true,
+		name:       "stop",
+		timeout:    -1,
 	},
 	"version": &versionCmd{},
 }
 
-var errArgs = fmt.Errorf(i18n.G("wrong number of subcommand arguments"))
-var errUsage = fmt.Errorf("show usage")
+// defaultAliases contains LXC's built-in command line aliases.  The built-in
+// aliases are checked only if no user-defined alias was found.
+var defaultAliases = map[string]string{
+	"shell": "exec @ARGS@ -- login -f root",
 
-func expandAlias(config *config.Config, origArgs []string) ([]string, bool) {
+	"cp":     "copy",
+	"ls":     "list",
+	"mv":     "move",
+	"rename": "move",
+	"rm":     "delete",
+
+	"image cp": "image copy",
+	"image ls": "image list",
+	"image rm": "image delete",
+
+	"image alias ls": "image alias list",
+	"image alias rm": "image alias delete",
+
+	"remote ls": "remote list",
+	"remote mv": "remote rename",
+	"remote rm": "remote remove",
+
+	"config device ls": "config device list",
+	"config device rm": "config device remove",
+}
+
+var errArgs = fmt.Errorf(i18n.G("wrong number of subcommand arguments"))
+
+func findAlias(aliases map[string]string, origArgs []string) ([]string, []string, bool) {
 	foundAlias := false
 	aliasKey := []string{}
 	aliasValue := []string{}
 
-	for k, v := range config.Aliases {
+	for k, v := range aliases {
 		foundAlias = true
 		for i, key := range strings.Split(k, " ") {
 			if len(origArgs) <= i+1 || origArgs[i+1] != key {
@@ -273,8 +256,16 @@ func expandAlias(config *config.Config, origArgs []string) ([]string, bool) {
 		}
 	}
 
+	return aliasKey, aliasValue, foundAlias
+}
+
+func expandAlias(config *lxd.Config, origArgs []string) ([]string, bool) {
+	aliasKey, aliasValue, foundAlias := findAlias(config.Aliases, origArgs)
 	if !foundAlias {
-		return []string{}, false
+		aliasKey, aliasValue, foundAlias = findAlias(defaultAliases, origArgs)
+		if !foundAlias {
+			return []string{}, false
+		}
 	}
 
 	newArgs := []string{origArgs[0]}
@@ -302,7 +293,7 @@ func expandAlias(config *config.Config, origArgs []string) ([]string, bool) {
 	return newArgs, true
 }
 
-func execIfAliases(config *config.Config, origArgs []string) {
+func execIfAliases(config *lxd.Config, origArgs []string) {
 	newArgs, expanded := expandAlias(config, origArgs)
 	if !expanded {
 		return
@@ -317,3 +308,41 @@ func execIfAliases(config *config.Config, origArgs []string) {
 	fmt.Fprintf(os.Stderr, i18n.G("processing aliases failed %s\n"), ret)
 	os.Exit(5)
 }
+
+type ProgressRenderer struct {
+	Format string
+
+	maxLength int
+}
+
+func (p *ProgressRenderer) Done(msg string) {
+	if msg != "" {
+		msg += "\n"
+	}
+
+	if len(msg) > p.maxLength {
+		p.maxLength = len(msg)
+	} else {
+		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
+	}
+
+	fmt.Print("\r")
+	fmt.Print(msg)
+}
+
+func (p *ProgressRenderer) Update(status string) {
+	msg := "%s"
+	if p.Format != "" {
+		msg = p.Format
+	}
+
+	msg = fmt.Sprintf("\r"+msg, status)
+
+	if len(msg) > p.maxLength {
+		p.maxLength = len(msg)
+	} else {
+		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
+	}
+
+	fmt.Print(msg)
+}
