@@ -5,7 +5,7 @@ import (
 
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -39,25 +39,24 @@ func (c *monitorCmd) showByDefault() bool {
 
 func (c *monitorCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc monitor [<remote>:] [--type=TYPE...]
+		`Monitor activity on the LXD server.
 
-Monitor a local or remote LXD server.
+lxc monitor [<remote>:] [--type=TYPE...]
 
-By default the monitor will listen to all message types.
+Connects to the monitoring interface of the specified LXD server.
 
-Message types to listen for can be specified with --type.
+By default will listen to all message types.
+Specific types to listen to can be specified with --type.
 
-*Examples*
-lxc monitor --type=logging
-    Only show log message.`)
+Example:
+    lxc monitor --type=logging`)
 }
 
 func (c *monitorCmd) flags() {
 	gnuflag.Var(&c.typeArgs, "type", i18n.G("Event type to listen for"))
 }
 
-func (c *monitorCmd) run(conf *config.Config, args []string) error {
-	var err error
+func (c *monitorCmd) run(config *lxd.Config, args []string) error {
 	var remote string
 
 	if len(args) > 1 {
@@ -65,23 +64,12 @@ func (c *monitorCmd) run(conf *config.Config, args []string) error {
 	}
 
 	if len(args) == 0 {
-		remote, _, err = conf.ParseRemote("")
-		if err != nil {
-			return err
-		}
+		remote, _ = config.ParseRemoteAndContainer("")
 	} else {
-		remote, _, err = conf.ParseRemote(args[0])
-		if err != nil {
-			return err
-		}
+		remote, _ = config.ParseRemoteAndContainer(args[0])
 	}
 
-	d, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
-	}
-
-	listener, err := d.GetEvents()
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -96,10 +84,5 @@ func (c *monitorCmd) run(conf *config.Config, args []string) error {
 		fmt.Printf("%s\n\n", render)
 	}
 
-	_, err = listener.AddHandler(c.typeArgs, handler)
-	if err != nil {
-		return err
-	}
-
-	return listener.Wait()
+	return d.Monitor(c.typeArgs, handler)
 }
