@@ -4,9 +4,7 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 
@@ -14,9 +12,10 @@ import (
 )
 
 type publishCmd struct {
-	pAliases   aliasList // aliasList defined in lxc/image.go
-	makePublic bool
-	Force      bool
+	pAliases              aliasList // aliasList defined in lxc/image.go
+	compression_algorithm string
+	makePublic            bool
+	Force                 bool
 }
 
 func (c *publishCmd) showByDefault() bool {
@@ -25,9 +24,9 @@ func (c *publishCmd) showByDefault() bool {
 
 func (c *publishCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc publish [<remote>:]<container>[/<snapshot>] [<remote>:] [--alias=ALIAS...] [prop-key=prop-value...]
+		`Publish containers as images.
 
-Publish containers as images.`)
+lxc publish [<remote>:]<container>[/<snapshot>] [<remote>:] [--alias=ALIAS...] [prop-key=prop-value...]`)
 }
 
 func (c *publishCmd) flags() {
@@ -35,9 +34,12 @@ func (c *publishCmd) flags() {
 	gnuflag.Var(&c.pAliases, "alias", i18n.G("New alias to define at target"))
 	gnuflag.BoolVar(&c.Force, "force", false, i18n.G("Stop the container if currently running"))
 	gnuflag.BoolVar(&c.Force, "f", false, i18n.G("Stop the container if currently running"))
+	gnuflag.StringVar(&c.compression_algorithm, "compression", "", i18n.G("Define a compression algorithm: for image or none"))
 }
 
-func (c *publishCmd) run(conf *config.Config, args []string) error {
+func (c *publishCmd) run(config *lxd.Config, args []string) error {
+	var cRemote string
+	var cName string
 	iName := ""
 	iRemote := ""
 	properties := map[string]string{}
@@ -47,22 +49,12 @@ func (c *publishCmd) run(conf *config.Config, args []string) error {
 		return errArgs
 	}
 
-	cRemote, cName, err := conf.ParseRemote(args[0])
-	if err != nil {
-		return err
-	}
-
+	cRemote, cName = config.ParseRemoteAndContainer(args[0])
 	if len(args) >= 2 && !strings.Contains(args[1], "=") {
 		firstprop = 2
-		iRemote, iName, err = conf.ParseRemote(args[1])
-		if err != nil {
-			return err
-		}
+		iRemote, iName = config.ParseRemoteAndContainer(args[1])
 	} else {
-		iRemote, iName, err = conf.ParseRemote("")
-		if err != nil {
-			return err
-		}
+		iRemote, iName = config.ParseRemoteAndContainer("")
 	}
 
 	if cName == "" {
@@ -72,26 +64,26 @@ func (c *publishCmd) run(conf *config.Config, args []string) error {
 		return fmt.Errorf(i18n.G("There is no \"image name\".  Did you want an alias?"))
 	}
 
-	d, err := conf.GetContainerServer(iRemote)
+	d, err := lxd.NewClient(config, iRemote)
 	if err != nil {
 		return err
 	}
 
 	s := d
 	if cRemote != iRemote {
-		s, err = conf.GetContainerServer(cRemote)
+		s, err = lxd.NewClient(config, cRemote)
 		if err != nil {
 			return err
 		}
 	}
 
 	if !shared.IsSnapshot(cName) {
-		ct, etag, err := s.GetContainer(cName)
+		ct, err := s.ContainerInfo(cName)
 		if err != nil {
 			return err
 		}
 
-		wasRunning := ct.StatusCode != 0 && ct.StatusCode != api.Stopped
+		wasRunning := ct.StatusCode != 0 && ct.StatusCode != shared.Stopped
 		wasEphemeral := ct.Ephemeral
 
 		if wasRunning {
@@ -101,57 +93,30 @@ func (c *publishCmd) run(conf *config.Config, args []string) error {
 
 			if ct.Ephemeral {
 				ct.Ephemeral = false
-				op, err := s.UpdateContainer(cName, ct.Writable(), etag)
-				if err != nil {
-					return err
-				}
-
-				err = op.Wait()
-				if err != nil {
-					return err
-				}
-
-				// Refresh the ETag
-				_, etag, err = s.GetContainer(cName)
+				err := s.UpdateContainerConfig(cName, ct.Brief())
 				if err != nil {
 					return err
 				}
 			}
 
-			req := api.ContainerStatePut{
-				Action:  string(shared.Stop),
-				Timeout: -1,
-				Force:   true,
-			}
-
-			op, err := s.UpdateContainerState(cName, req, "")
+			resp, err := s.Action(cName, shared.Stop, -1, true, false)
 			if err != nil {
 				return err
 			}
 
-			err = op.Wait()
+			op, err := s.WaitFor(resp.Operation)
 			if err != nil {
-				return fmt.Errorf(i18n.G("Stopping container failed!"))
+				return err
 			}
 
-			defer func() {
-				req.Action = string(shared.Start)
-				op, err = s.UpdateContainerState(cName, req, "")
-				if err != nil {
-					return
-				}
-
-				op.Wait()
-			}()
+			if op.StatusCode == shared.Failure {
+				return fmt.Errorf(i18n.G("Stopping container failed!"))
+			}
+			defer s.Action(cName, shared.Start, -1, true, false)
 
 			if wasEphemeral {
 				ct.Ephemeral = true
-				op, err := s.UpdateContainer(cName, ct.Writable(), etag)
-				if err != nil {
-					return err
-				}
-
-				err = op.Wait()
+				err := s.UpdateContainerConfig(cName, ct.Brief())
 				if err != nil {
 					return err
 				}
@@ -167,6 +132,8 @@ func (c *publishCmd) run(conf *config.Config, args []string) error {
 		properties[entry[0]] = entry[1]
 	}
 
+	var fp string
+
 	// We should only set the properties field if there actually are any.
 	// Otherwise we will only delete any existing properties on publish.
 	// This is something which only direct callers of the API are allowed to
@@ -175,76 +142,28 @@ func (c *publishCmd) run(conf *config.Config, args []string) error {
 		properties = nil
 	}
 
-	// Reformat aliases
-	aliases := []api.ImageAlias{}
-	for _, entry := range c.pAliases {
-		alias := api.ImageAlias{}
-		alias.Name = entry
-		aliases = append(aliases, alias)
-	}
-
-	// Create the image
-	req := api.ImagesPost{
-		Source: &api.ImagesPostSource{
-			Type: "container",
-			Name: cName,
-		},
-	}
-	req.Properties = properties
-
-	if shared.IsSnapshot(cName) {
-		req.Source.Type = "snapshot"
-	}
-
+	// Optimized local publish
 	if cRemote == iRemote {
-		req.Public = c.makePublic
+		fp, err = d.ImageFromContainer(cName, c.makePublic, c.pAliases, properties, c.compression_algorithm)
+		if err != nil {
+			return err
+		}
+		fmt.Printf(i18n.G("Container published with fingerprint: %s")+"\n", fp)
+		return nil
 	}
 
-	op, err := s.CreateImage(req, nil)
+	fp, err = s.ImageFromContainer(cName, false, nil, properties, c.compression_algorithm)
+	if err != nil {
+		return err
+	}
+	defer s.DeleteImage(fp)
+
+	err = s.CopyImage(fp, d, false, c.pAliases, c.makePublic, false, nil)
 	if err != nil {
 		return err
 	}
 
-	err = op.Wait()
-	if err != nil {
-		return err
-	}
-
-	// Grab the fingerprint
-	fingerprint := op.Metadata["fingerprint"].(string)
-
-	// For remote publish, copy to target now
-	if cRemote != iRemote {
-		defer s.DeleteImage(fingerprint)
-
-		// Get the source image
-		image, _, err := s.GetImage(fingerprint)
-		if err != nil {
-			return err
-		}
-
-		// Image copy arguments
-		args := lxd.ImageCopyArgs{
-			Public: c.makePublic,
-		}
-
-		// Copy the image to the destination host
-		op, err := d.CopyImage(s, *image, &args)
-		if err != nil {
-			return err
-		}
-
-		err = op.Wait()
-		if err != nil {
-			return err
-		}
-	}
-
-	err = ensureImageAliases(d, aliases, fingerprint)
-	if err != nil {
-		return err
-	}
-	fmt.Printf(i18n.G("Container published with fingerprint: %s")+"\n", fingerprint)
+	fmt.Printf(i18n.G("Container published with fingerprint: %s")+"\n", fp)
 
 	return nil
 }
