@@ -1,7 +1,6 @@
 package main
 
 import (
-	"crypto/sha256"
 	"crypto/x509"
 	"encoding/pem"
 	"fmt"
@@ -38,13 +37,13 @@ func (c *remoteCmd) usage() string {
 	return i18n.G(
 		`Manage remote LXD servers.
 
-lxc remote add <name> <url> [--accept-certificate] [--password=PASSWORD]
-                            [--public] [--protocol=PROTOCOL]                Add the remote <name> at <url>.
-lxc remote remove <name>                                                    Remove the remote <name>.
+lxc remote add [<remote>] <IP|FQDN|URL> [--accept-certificate] [--password=PASSWORD]
+                                        [--public] [--protocol=PROTOCOL]    Add the remote <name> at <url>.
+lxc remote remove <remote>                                                  Remove the remote <name>.
 lxc remote list                                                             List all remotes.
-lxc remote rename <old> <new>                                               Rename remote <old> to <new>.
-lxc remote set-url <name> <url>                                             Update <name>'s url to <url>.
-lxc remote set-default <name>                                               Set the default remote.
+lxc remote rename <old name> <new name>                                     Rename remote <old> to <new>.
+lxc remote set-url <remote> <url>                                           Update <name>'s url to <url>.
+lxc remote set-default <remote>                                             Set the default remote.
 lxc remote get-default                                                      Print the default remote.`)
 }
 
@@ -112,7 +111,7 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 	/* Complex remote URL parsing */
 	remoteURL, err := url.Parse(addr)
 	if err != nil {
-		return err
+		remoteURL = &url.URL{Host: addr}
 	}
 
 	// Fast track simplestreams
@@ -213,7 +212,7 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 
 	if certificate != nil {
 		if !acceptCert {
-			digest := sha256.Sum256(certificate.Raw)
+			digest := shared.CertFingerprint(certificate)
 
 			fmt.Printf(i18n.G("Certificate fingerprint: %x")+"\n", digest)
 			fmt.Printf(i18n.G("ok (y/n)?") + " ")
@@ -306,18 +305,24 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 
 	switch args[0] {
 	case "add":
-		if len(args) < 3 {
+		if len(args) < 2 {
 			return errArgs
 		}
 
-		if rc, ok := config.Remotes[args[1]]; ok {
-			return fmt.Errorf(i18n.G("remote %s exists as <%s>"), args[1], rc.Addr)
+		remote := args[1]
+		fqdn := args[1]
+		if len(args) > 2 {
+			fqdn = args[2]
 		}
 
-		err := c.addServer(config, args[1], args[2], c.acceptCert, c.password, c.public, c.protocol)
+		if rc, ok := config.Remotes[remote]; ok {
+			return fmt.Errorf(i18n.G("remote %s exists as <%s>"), remote, rc.Addr)
+		}
+
+		err := c.addServer(config, remote, fqdn, c.acceptCert, c.password, c.public, c.protocol)
 		if err != nil {
-			delete(config.Remotes, args[1])
-			c.removeCertificate(config, args[1])
+			delete(config.Remotes, remote)
+			c.removeCertificate(config, remote)
 			return err
 		}
 
