@@ -4,36 +4,16 @@ import (
 	"fmt"
 	"io"
 	"os"
-	"os/exec"
-	"strconv"
 	"strings"
 	"time"
 
 	"gopkg.in/lxc/go-lxc.v2"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/osarch"
 )
 
 // Helper functions
-
-// Returns the parent container name, snapshot name, and whether it actually was
-// a snapshot name.
-func containerGetParentAndSnapshotName(name string) (string, string, bool) {
-	fields := strings.SplitN(name, shared.SnapshotDelimiter, 2)
-	if len(fields) == 1 {
-		return name, "", false
-	}
-
-	return fields[0], fields[1], true
-}
-
 func containerPath(name string, isSnapshot bool) string {
 	if isSnapshot {
 		return shared.VarPath("snapshots", name)
@@ -56,191 +36,30 @@ func containerValidName(name string) error {
 	return nil
 }
 
-func containerValidConfigKey(key string, value string) error {
-	isInt64 := func(key string, value string) error {
-		if value == "" {
-			return nil
-		}
-
-		_, err := strconv.ParseInt(value, 10, 64)
-		if err != nil {
-			return fmt.Errorf("Invalid value for an integer: %s", value)
-		}
-
-		return nil
+func containerValidConfigKey(d *Daemon, key string, value string) error {
+	f, err := shared.ConfigKeyChecker(key)
+	if err != nil {
+		return err
 	}
-
-	isBool := func(key string, value string) error {
-		if value == "" {
-			return nil
-		}
-
-		if !shared.StringInSlice(strings.ToLower(value), []string{"true", "false", "yes", "no", "1", "0", "on", "off"}) {
-			return fmt.Errorf("Invalid value for a boolean: %s", value)
-		}
-
-		return nil
+	if err = f(value); err != nil {
+		return err
 	}
-
-	isOneOf := func(key string, value string, valid []string) error {
-		if value == "" {
-			return nil
-		}
-
-		if !shared.StringInSlice(value, valid) {
-			return fmt.Errorf("Invalid value: %s (not one of %s)", value, valid)
-		}
-
-		return nil
-	}
-
-	isUint32 := func(key string, value string) error {
-		if value == "" {
-			return nil
-		}
-
-		_, err := strconv.ParseInt(value, 10, 32)
-		if err != nil {
-			return fmt.Errorf("Invalid value for uint32: %s: %v", value, err)
-		}
-
-		return nil
-	}
-
-	switch key {
-	case "boot.autostart":
-		return isBool(key, value)
-	case "boot.autostart.delay":
-		return isInt64(key, value)
-	case "boot.autostart.priority":
-		return isInt64(key, value)
-	case "limits.cpu":
-		return nil
-	case "limits.cpu.allowance":
-		if value == "" {
-			return nil
-		}
-
-		if strings.HasSuffix(value, "%") {
-			// Percentage based allocation
-			_, err := strconv.Atoi(strings.TrimSuffix(value, "%"))
-			if err != nil {
-				return err
-			}
-
-			return nil
-		}
-
-		// Time based allocation
-		fields := strings.SplitN(value, "/", 2)
-		if len(fields) != 2 {
-			return fmt.Errorf("Invalid allowance: %s", value)
-		}
-
-		_, err := strconv.Atoi(strings.TrimSuffix(fields[0], "ms"))
-		if err != nil {
-			return err
-		}
-
-		_, err = strconv.Atoi(strings.TrimSuffix(fields[1], "ms"))
-		if err != nil {
-			return err
-		}
-
-		return nil
-	case "limits.cpu.priority":
-		return isInt64(key, value)
-	case "limits.disk.priority":
-		return isInt64(key, value)
-	case "limits.memory":
-		if value == "" {
-			return nil
-		}
-
-		if strings.HasSuffix(value, "%") {
-			_, err := strconv.ParseInt(strings.TrimSuffix(value, "%"), 10, 64)
-			if err != nil {
-				return err
-			}
-
-			return nil
-		}
-
-		_, err := shared.ParseByteSizeString(value)
-		if err != nil {
-			return err
-		}
-
-		return nil
-	case "limits.memory.enforce":
-		return isOneOf(key, value, []string{"soft", "hard"})
-	case "limits.memory.swap":
-		return isBool(key, value)
-	case "limits.memory.swap.priority":
-		return isInt64(key, value)
-	case "limits.network.priority":
-		return isInt64(key, value)
-	case "limits.processes":
-		return isInt64(key, value)
-	case "linux.kernel_modules":
-		return nil
-	case "security.privileged":
-		return isBool(key, value)
-	case "security.nesting":
-		return isBool(key, value)
-	case "security.idmap.base":
-		return isUint32(key, value)
-	case "security.idmap.size":
-		return isUint32(key, value)
-	case "security.idmap.isolated":
-		return isBool(key, value)
-	case "raw.apparmor":
-		return nil
-	case "raw.idmap":
-		return nil
-	case "raw.lxc":
+	if key == "raw.lxc" {
 		return lxcValidConfig(value)
-	case "volatile.apply_template":
-		return nil
-	case "volatile.base_image":
-		return nil
-	case "volatile.last_state.idmap":
-		return nil
-	case "volatile.last_state.power":
-		return nil
-	case "volatile.idmap.next":
-		return nil
-	case "volatile.idmap.base":
-		return nil
 	}
-
-	if strings.HasPrefix(key, "volatile.") {
-		if strings.HasSuffix(key, ".hwaddr") {
-			return nil
-		}
-
-		if strings.HasSuffix(key, ".name") {
-			return nil
-		}
-
-		if strings.HasSuffix(key, ".host_name") {
-			return nil
+	if key == "security.syscalls.blacklist_compat" {
+		for _, arch := range d.architectures {
+			if arch == osarch.ARCH_64BIT_INTEL_X86 ||
+				arch == osarch.ARCH_64BIT_ARMV8_LITTLE_ENDIAN ||
+				arch == osarch.ARCH_64BIT_POWERPC_BIG_ENDIAN {
+				return nil
+			}
 		}
+		return fmt.Errorf("security.syscalls.blacklist_compat isn't supported on this architecture")
 	}
-
-	if strings.HasPrefix(key, "environment.") {
-		return nil
-	}
-
-	if strings.HasPrefix(key, "user.") {
-		return nil
-	}
-
-	return fmt.Errorf("Unknown configuration key: %s", key)
+	return nil
 }
 
-var containerNetworkLimitKeys = []string{"limits.max", "limits.ingress", "limits.egress"}
-
 func containerValidDeviceConfigKey(t, k string) bool {
 	if k == "type" {
 		return true
@@ -284,6 +103,12 @@ func containerValidDeviceConfigKey(t, k string) bool {
 			return true
 		case "parent":
 			return true
+		case "ipv4.address":
+			return true
+		case "ipv6.address":
+			return true
+		case "security.mac_filtering":
+			return true
 		default:
 			return false
 		}
@@ -310,6 +135,42 @@ func containerValidDeviceConfigKey(t, k string) bool {
 		default:
 			return false
 		}
+	case "usb":
+		switch k {
+		case "vendorid":
+			return true
+		case "productid":
+			return true
+		case "mode":
+			return true
+		case "gid":
+			return true
+		case "uid":
+			return true
+		case "required":
+			return true
+		default:
+			return false
+		}
+	case "gpu":
+		switch k {
+		case "vendorid":
+			return true
+		case "productid":
+			return true
+		case "id":
+			return true
+		case "pci":
+			return true
+		case "mode":
+			return true
+		case "gid":
+			return true
+		case "uid":
+			return true
+		default:
+			return false
+		}
 	case "none":
 		return false
 	default:
@@ -317,7 +178,7 @@ func containerValidDeviceConfigKey(t, k string) bool {
 	}
 }
 
-func containerValidConfig(os *sys.OS, config map[string]string, profile bool, expanded bool) error {
+func containerValidConfig(d *Daemon, config map[string]string, profile bool, expanded bool) error {
 	if config == nil {
 		return nil
 	}
@@ -327,67 +188,50 @@ func containerValidConfig(os *sys.OS, config map[string]string, profile bool, ex
 			return fmt.Errorf("Volatile keys can only be set on containers.")
 		}
 
-		err := containerValidConfigKey(k, v)
+		if profile && strings.HasPrefix(k, "image.") {
+			return fmt.Errorf("Image keys can only be set on containers.")
+		}
+
+		err := containerValidConfigKey(d, k, v)
 		if err != nil {
 			return err
 		}
 	}
 
-	if expanded && (config["security.privileged"] == "" || !shared.IsTrue(config["security.privileged"])) && os.IdmapSet == nil {
-		return fmt.Errorf("LXD doesn't have a uid/gid allocation. In this mode, only privileged containers are supported.")
+	_, rawSeccomp := config["raw.seccomp"]
+	_, whitelist := config["security.syscalls.whitelist"]
+	_, blacklist := config["securtiy.syscalls.blacklist"]
+	blacklistDefault := shared.IsTrue(config["security.syscalls.blacklist_default"])
+	blacklistCompat := shared.IsTrue(config["security.syscalls.blacklist_compat"])
+
+	if rawSeccomp && (whitelist || blacklist || blacklistDefault || blacklistCompat) {
+		return fmt.Errorf("raw.seccomp is mutually exclusive with security.syscalls*")
+	}
+
+	if whitelist && (blacklist || blacklistDefault || blacklistCompat) {
+		return fmt.Errorf("security.syscalls.whitelist is mutually exclusive with security.syscalls.blacklist*")
 	}
 
 	return nil
 }
 
-func isRootDiskDevice(device types.Device) bool {
-	if device["type"] == "disk" && device["path"] == "/" && device["source"] == "" {
-		return true
-	}
-
-	return false
-}
-
-func containerGetRootDiskDevice(devices types.Devices) (string, types.Device, error) {
-	var devName string
-	var dev types.Device
-
-	for n, d := range devices {
-		if isRootDiskDevice(d) {
-			if devName != "" {
-				return "", types.Device{}, fmt.Errorf("More than one root device found.")
-			}
-
-			devName = n
-			dev = d
-		}
-	}
-
-	if devName != "" {
-		return devName, dev, nil
-	}
-
-	return "", types.Device{}, fmt.Errorf("No root device could be found.")
-}
-
-func containerValidDevices(devices types.Devices, profile bool, expanded bool) error {
+func containerValidDevices(devices shared.Devices, profile bool, expanded bool) error {
 	// Empty device list
 	if devices == nil {
 		return nil
 	}
 
-	var diskDevicePaths []string
 	// Check each device individually
 	for name, m := range devices {
 		if m["type"] == "" {
 			return fmt.Errorf("Missing device type for device '%s'", name)
 		}
 
-		if !shared.StringInSlice(m["type"], []string{"none", "nic", "disk", "unix-char", "unix-block"}) {
+		if !shared.StringInSlice(m["type"], []string{"none", "nic", "disk", "unix-char", "unix-block", "usb", "gpu"}) {
 			return fmt.Errorf("Invalid device type for device '%s'", name)
 		}
 
-		for k := range m {
+		for k, _ := range m {
 			if !containerValidDeviceConfigKey(m["type"], k) {
 				return fmt.Errorf("Invalid device configuration key for %s: %s", m["type"], k)
 			}
@@ -406,12 +250,6 @@ func containerValidDevices(devices types.Devices, profile bool, expanded bool) e
 				return fmt.Errorf("Missing parent for %s type nic.", m["nictype"])
 			}
 		} else if m["type"] == "disk" {
-			if !expanded && !shared.StringInSlice(m["path"], diskDevicePaths) {
-				diskDevicePaths = append(diskDevicePaths, m["path"])
-			} else if !expanded {
-				return fmt.Errorf("More than one disk device uses the same path: %s.", m["path"])
-			}
-
 			if m["path"] == "" {
 				return fmt.Errorf("Disk entry is missing the required \"path\" property.")
 			}
@@ -435,25 +273,13 @@ func containerValidDevices(devices types.Devices, profile bool, expanded bool) e
 			if m["path"] == "" {
 				return fmt.Errorf("Unix device entry is missing the required \"path\" property.")
 			}
-
-			if m["major"] == "" || m["minor"] == "" {
-				if !shared.PathExists(m["path"]) {
-					return fmt.Errorf("The device path doesn't exist on the host and major/minor wasn't specified.")
-				}
-
-				dType, _, _, err := deviceGetAttributes(m["path"])
-				if err != nil {
-					return err
-				}
-
-				if m["type"] == "unix-char" && dType != "c" {
-					return fmt.Errorf("Path specified for unix-char device is a block device.")
-				}
-
-				if m["type"] == "unix-block" && dType != "b" {
-					return fmt.Errorf("Path specified for unix-block device is a character device.")
-				}
+		} else if m["type"] == "usb" {
+			if m["vendorid"] == "" {
+				return fmt.Errorf("Missing vendorid for USB device.")
 			}
+		} else if m["type"] == "gpu" {
+			// Probably no checks needed, since we allow users to
+			// pass in all GPUs.
 		} else if m["type"] == "none" {
 			continue
 		} else {
@@ -463,15 +289,39 @@ func containerValidDevices(devices types.Devices, profile bool, expanded bool) e
 
 	// Checks on the expanded config
 	if expanded {
-		_, _, err := containerGetRootDiskDevice(devices)
-		if err != nil {
-			return err
+		foundRootfs := false
+		for _, m := range devices {
+			if m["type"] == "disk" && m["path"] == "/" {
+				foundRootfs = true
+			}
+		}
+
+		if !foundRootfs {
+			return fmt.Errorf("Container is lacking rootfs entry")
 		}
 	}
 
 	return nil
 }
 
+// The container arguments
+type containerArgs struct {
+	// Don't set manually
+	Id int
+
+	Architecture int
+	BaseImage    string
+	Config       map[string]string
+	CreationDate time.Time
+	LastUsedDate time.Time
+	Ctype        containerType
+	Devices      shared.Devices
+	Ephemeral    bool
+	Name         string
+	Profiles     []string
+	Stateful     bool
+}
+
 // The container interface
 type container interface {
 	// Container actions
@@ -491,7 +341,7 @@ type container interface {
 
 	// Config handling
 	Rename(newName string) error
-	Update(newConfig db.ContainerArgs, userRequested bool) error
+	Update(newConfig containerArgs, userRequested bool) error
 
 	Delete() error
 	Export(w io.Writer, properties map[string]string) error
@@ -502,9 +352,9 @@ type container interface {
 	ConfigKeySet(key string, value string) error
 
 	// File handling
-	FilePull(srcpath string, dstpath string) (int64, int64, os.FileMode, error)
 	FileExists(path string) error
-	FilePush(srcpath string, dstpath string, uid int64, gid int64, mode int) error
+	FilePull(srcpath string, dstpath string) (int, int, os.FileMode, string, []string, error)
+	FilePush(srcpath string, dstpath string, uid int, gid int, mode int) error
 	FileRemove(path string) error
 
 	/* Command execution:
@@ -519,11 +369,11 @@ type container interface {
 	         *      (the PID returned in the first return argument). It can however
 	         *      be used to e.g. forward signals.)
 	*/
-	Exec(command []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File, wait bool) (*exec.Cmd, int, int, error)
+	Exec(command []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File, wait bool) (int, int, error)
 
 	// Status
-	Render() (interface{}, error)
-	RenderState() (*api.ContainerState, error)
+	Render() (interface{}, interface{}, error)
+	RenderState() (*shared.ContainerState, error)
 	IsPrivileged() bool
 	IsRunning() bool
 	IsFrozen() bool
@@ -541,10 +391,11 @@ type container interface {
 	Name() string
 	Architecture() int
 	CreationDate() time.Time
+	LastUsedDate() time.Time
 	ExpandedConfig() map[string]string
-	ExpandedDevices() types.Devices
+	ExpandedDevices() shared.Devices
 	LocalConfig() map[string]string
-	LocalDevices() types.Devices
+	LocalDevices() shared.Devices
 	Profiles() []string
 	InitPID() int
 	State() string
@@ -561,24 +412,23 @@ type container interface {
 	StorageStart() error
 	StorageStop() error
 	Storage() storage
-	IdmapSet() (*idmap.IdmapSet, error)
-	LastIdmapSet() (*idmap.IdmapSet, error)
+	IdmapSet() *shared.IdmapSet
+	LastIdmapSet() (*shared.IdmapSet, error)
 	TemplateApply(trigger string) error
-	StateObject() *state.State
+	Daemon() *Daemon
 }
 
 // Loader functions
-func containerCreateAsEmpty(d *Daemon, args db.ContainerArgs) (container, error) {
+func containerCreateAsEmpty(d *Daemon, args containerArgs) (container, error) {
 	// Create the container
-	c, err := containerCreateInternal(d.State(), d.Storage, args)
+	c, err := containerCreateInternal(d, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Now create the empty storage
-	err = c.Storage().ContainerCreate(c)
-	if err != nil {
-		db.ContainerRemove(d.db, args.Name)
+	if err := c.Storage().ContainerCreate(c); err != nil {
+		c.Delete()
 		return nil, err
 	}
 
@@ -592,43 +442,49 @@ func containerCreateAsEmpty(d *Daemon, args db.ContainerArgs) (container, error)
 	return c, nil
 }
 
-func containerCreateEmptySnapshot(s *state.State, storage storage, args db.ContainerArgs) (container, error) {
+func containerCreateEmptySnapshot(d *Daemon, args containerArgs) (container, error) {
 	// Create the snapshot
-	c, err := containerCreateInternal(s, storage, args)
+	c, err := containerCreateInternal(d, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Now create the empty snapshot
-	err = c.Storage().ContainerSnapshotCreateEmpty(c)
-	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+	if err := c.Storage().ContainerSnapshotCreateEmpty(c); err != nil {
+		c.Delete()
 		return nil, err
 	}
 
 	return c, nil
 }
 
-func containerCreateFromImage(s *state.State, storage storage, args db.ContainerArgs, hash string) (container, error) {
-	// Set the BaseImage field (regardless of previous value)
-	args.BaseImage = hash
-
-	// Create the container
-	c, err := containerCreateInternal(s, storage, args)
+func containerCreateFromImage(d *Daemon, args containerArgs, hash string) (container, error) {
+	// Get the image properties
+	_, img, err := dbImageGet(d.db, hash, false, false)
 	if err != nil {
 		return nil, err
 	}
 
-	err = db.ImageLastAccessUpdate(s.DB, hash, time.Now().UTC())
+	// Set the "image.*" keys
+	if img.Properties != nil {
+		for k, v := range img.Properties {
+			args.Config[fmt.Sprintf("image.%s", k)] = v
+		}
+	}
+
+	// Create the container
+	c, err := containerCreateInternal(d, args)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		return nil, err
+	}
+
+	if err := dbImageLastAccessUpdate(d.db, hash, time.Now().UTC()); err != nil {
 		return nil, fmt.Errorf("Error updating image last use date: %s", err)
 	}
 
 	// Now create the storage from an image
-	err = c.Storage().ContainerCreateFromImage(c, hash)
-	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+	if err := c.Storage().ContainerCreateFromImage(c, hash); err != nil {
+		c.Delete()
 		return nil, err
 	}
 
@@ -642,9 +498,9 @@ func containerCreateFromImage(s *state.State, storage storage, args db.Container
 	return c, nil
 }
 
-func containerCreateAsCopy(s *state.State, storage storage, args db.ContainerArgs, sourceContainer container) (container, error) {
-	// Create the container.
-	c, err := containerCreateInternal(s, storage, args)
+func containerCreateAsCopy(d *Daemon, args containerArgs, sourceContainer container) (container, error) {
+	// Create the container
+	c, err := containerCreateInternal(d, args)
 	if err != nil {
 		return nil, err
 	}
@@ -665,20 +521,19 @@ func containerCreateAsCopy(s *state.State, storage storage, args db.ContainerArg
 	return c, nil
 }
 
-func containerCreateAsSnapshot(s *state.State, storage storage, args db.ContainerArgs, sourceContainer container) (container, error) {
+func containerCreateAsSnapshot(d *Daemon, args containerArgs, sourceContainer container) (container, error) {
 	// Deal with state
 	if args.Stateful {
 		if !sourceContainer.IsRunning() {
-			return nil, fmt.Errorf("Unable to create a stateful snapshot. The container isn't running.")
+			return nil, fmt.Errorf("Container not running, cannot do stateful snapshot")
 		}
 
-		_, err := exec.LookPath("criu")
-		if err != nil {
-			return nil, fmt.Errorf("Unable to create a stateful snapshot. CRIU isn't installed.")
+		if err := findCriu("snapshot"); err != nil {
+			return nil, err
 		}
 
 		stateDir := sourceContainer.StatePath()
-		err = os.MkdirAll(stateDir, 0700)
+		err := os.MkdirAll(stateDir, 0700)
 		if err != nil {
 			return nil, err
 		}
@@ -701,15 +556,20 @@ func containerCreateAsSnapshot(s *state.State, storage storage, args db.Containe
 	}
 
 	// Create the snapshot
-	c, err := containerCreateInternal(s, storage, args)
+	c, err := containerCreateInternal(d, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Clone the container
-	err = sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer)
+	if err := sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer); err != nil {
+		c.Delete()
+		return nil, err
+	}
+
+	err = writeBackupFile(sourceContainer)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
+		c.Delete()
 		return nil, err
 	}
 
@@ -721,7 +581,7 @@ func containerCreateAsSnapshot(s *state.State, storage storage, args db.Containe
 	return c, nil
 }
 
-func containerCreateInternal(s *state.State, storage storage, args db.ContainerArgs) (container, error) {
+func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
 	// Set default values
 	if args.Profiles == nil {
 		args.Profiles = []string{"default"}
@@ -736,15 +596,15 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 	}
 
 	if args.Devices == nil {
-		args.Devices = types.Devices{}
+		args.Devices = shared.Devices{}
 	}
 
 	if args.Architecture == 0 {
-		args.Architecture = s.OS.Architectures[0]
+		args.Architecture = d.architectures[0]
 	}
 
 	// Validate container name
-	if args.Ctype == db.CTypeRegular {
+	if args.Ctype == cTypeRegular {
 		err := containerValidName(args.Name)
 		if err != nil {
 			return nil, err
@@ -752,7 +612,7 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 	}
 
 	// Validate container config
-	err := containerValidConfig(s.OS, args.Config, false, false)
+	err := containerValidConfig(d, args.Config, false, false)
 	if err != nil {
 		return nil, err
 	}
@@ -769,12 +629,8 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 		return nil, err
 	}
 
-	if !shared.IntInSlice(args.Architecture, s.OS.Architectures) {
-		return nil, fmt.Errorf("Requested architecture isn't supported by this host")
-	}
-
 	// Validate profiles
-	profiles, err := db.Profiles(s.DB)
+	profiles, err := dbProfiles(d.db)
 	if err != nil {
 		return nil, err
 	}
@@ -786,9 +642,9 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 	}
 
 	// Create the container entry
-	id, err := db.ContainerCreate(s.DB, args)
+	id, err := dbContainerCreate(d.db, args)
 	if err != nil {
-		if err == db.DbErrAlreadyDefined {
+		if err == DbErrAlreadyDefined {
 			thing := "Container"
 			if shared.IsSnapshot(args.Name) {
 				thing = "Snapshot"
@@ -804,17 +660,15 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 	args.Id = id
 
 	// Read the timestamp from the database
-	dbArgs, err := db.ContainerGet(s.DB, args.Name)
+	dbArgs, err := dbContainerGet(d.db, args.Name)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 	args.CreationDate = dbArgs.CreationDate
+	args.LastUsedDate = dbArgs.LastUsedDate
 
-	// Setup the container struct and finish creation (storage and idmap)
-	c, err := containerLXCCreate(s, storage, args)
+	c, err := containerLXCCreate(d, args)
 	if err != nil {
-		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 
@@ -823,43 +677,48 @@ func containerCreateInternal(s *state.State, storage storage, args db.ContainerA
 
 func containerConfigureInternal(c container) error {
 	// Find the root device
-	_, rootDiskDevice, err := containerGetRootDiskDevice(c.ExpandedDevices())
-	if err != nil {
-		return err
-	}
+	for _, m := range c.ExpandedDevices() {
+		if m["type"] != "disk" || m["path"] != "/" || m["size"] == "" {
+			continue
+		}
 
-	if rootDiskDevice["size"] != "" {
-		size, err := shared.ParseByteSizeString(rootDiskDevice["size"])
+		size, err := shared.ParseByteSizeString(m["size"])
 		if err != nil {
 			return err
 		}
 
-		// Storage is guaranteed to be ready.
 		err = c.Storage().ContainerSetQuota(c, size)
 		if err != nil {
 			return err
 		}
+
+		break
+	}
+
+	err := writeBackupFile(c)
+	if err != nil {
+		return err
 	}
 
 	return nil
 }
 
-func containerLoadById(s *state.State, storage storage, id int) (container, error) {
+func containerLoadById(d *Daemon, id int) (container, error) {
 	// Get the DB record
-	name, err := db.ContainerName(s.DB, id)
+	name, err := dbContainerName(d.db, id)
 	if err != nil {
 		return nil, err
 	}
 
-	return containerLoadByName(s, storage, name)
+	return containerLoadByName(d, name)
 }
 
-func containerLoadByName(s *state.State, storage storage, name string) (container, error) {
+func containerLoadByName(d *Daemon, name string) (container, error) {
 	// Get the DB record
-	args, err := db.ContainerGet(s.DB, name)
+	args, err := dbContainerGet(d.db, name)
 	if err != nil {
 		return nil, err
 	}
 
-	return containerLXCLoad(s, storage, args)
+	return containerLXCLoad(d, args)
 }
