@@ -6,7 +6,7 @@ import (
 	"io/ioutil"
 	"net/http"
 	"os"
-	"os/exec"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"sync"
@@ -16,19 +16,28 @@ import (
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
+type commandPostContent struct {
+	Command      []string          `json:"command"`
+	WaitForWS    bool              `json:"wait-for-websocket"`
+	RecordOutput bool              `json:"record-output"`
+	Interactive  bool              `json:"interactive"`
+	Environment  map[string]string `json:"environment"`
+	Width        int               `json:"width"`
+	Height       int               `json:"height"`
+}
+
 type execWs struct {
 	command   []string
 	container container
 	env       map[string]string
 
-	rootUid          int64
-	rootGid          int64
+	rootUid          int
+	rootGid          int
 	conns            map[int]*websocket.Conn
 	connsLock        sync.Mutex
 	allConnected     chan bool
@@ -149,84 +158,62 @@ func (s *execWs) Do(op *operation) error {
 			}
 
 			for {
-				s.connsLock.Lock()
-				conn := s.conns[-1]
-				s.connsLock.Unlock()
-
-				mt, r, err := conn.NextReader()
+				mt, r, err := s.conns[-1].NextReader()
 				if mt == websocket.CloseMessage {
 					break
 				}
 
 				if err != nil {
-					logger.Debugf("Got error getting next reader %s", err)
-					er, ok := err.(*websocket.CloseError)
-					if !ok {
-						break
-					}
-
-					if er.Code != websocket.CloseAbnormalClosure {
-						break
-					}
-
-					// If an abnormal closure occurred, kill the attached process.
-					err := syscall.Kill(attachedChildPid, syscall.SIGKILL)
-					if err != nil {
-						logger.Debugf("Failed to send SIGKILL to pid %d.", attachedChildPid)
-					} else {
-						logger.Debugf("Sent SIGKILL to pid %d.", attachedChildPid)
-					}
-					return
+					shared.LogDebugf("Got error getting next reader %s", err)
+					break
 				}
 
 				buf, err := ioutil.ReadAll(r)
 				if err != nil {
-					logger.Debugf("Failed to read message %s", err)
+					shared.LogDebugf("Failed to read message %s", err)
 					break
 				}
 
-				command := api.ContainerExecControl{}
+				command := shared.ContainerExecControl{}
 
 				if err := json.Unmarshal(buf, &command); err != nil {
-					logger.Debugf("Failed to unmarshal control socket command: %s", err)
+					shared.LogDebugf("Failed to unmarshal control socket command: %s", err)
 					continue
 				}
 
 				if command.Command == "window-resize" {
 					winchWidth, err := strconv.Atoi(command.Args["width"])
 					if err != nil {
-						logger.Debugf("Unable to extract window width: %s", err)
+						shared.LogDebugf("Unable to extract window width: %s", err)
 						continue
 					}
 
 					winchHeight, err := strconv.Atoi(command.Args["height"])
 					if err != nil {
-						logger.Debugf("Unable to extract window height: %s", err)
+						shared.LogDebugf("Unable to extract window height: %s", err)
 						continue
 					}
 
 					err = shared.SetSize(int(ptys[0].Fd()), winchWidth, winchHeight)
 					if err != nil {
-						logger.Debugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
+						shared.LogDebugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
 						continue
 					}
+				} else if command.Command == "signal" {
+					if err := syscall.Kill(attachedChildPid, command.Signal); err != nil {
+						shared.LogDebugf("Failed forwarding signal '%s' to PID %d.", command.Signal, attachedChildPid)
+						continue
+					}
+					shared.LogDebugf("Forwarded signal '%s' to PID %d.", command.Signal, attachedChildPid)
 				}
 			}
 		}()
 
 		go func() {
-			s.connsLock.Lock()
-			conn := s.conns[0]
-			s.connsLock.Unlock()
-
-			logger.Debugf("Starting to mirror websocket")
-			readDone, writeDone := shared.WebsocketExecMirror(conn, ptys[0], ptys[0], attachedChildIsDead, int(ptys[0].Fd()))
-
+			readDone, writeDone := shared.WebsocketExecMirror(s.conns[0], ptys[0], ptys[0], attachedChildIsDead, int(ptys[0].Fd()))
 			<-readDone
 			<-writeDone
-			logger.Debugf("Finished to mirror websocket")
-
-			conn.Close()
+			s.conns[0].Close()
 			wgEOF.Done()
 		}()
 
@@ -235,18 +222,10 @@ func (s *execWs) Do(op *operation) error {
 		for i := 0; i < len(ttys); i++ {
 			go func(i int) {
 				if i == 0 {
-					s.connsLock.Lock()
-					conn := s.conns[i]
-					s.connsLock.Unlock()
-
-					<-shared.WebsocketRecvStream(ttys[i], conn)
+					<-shared.WebsocketRecvStream(ttys[i], s.conns[i])
 					ttys[i].Close()
 				} else {
-					s.connsLock.Lock()
-					conn := s.conns[i]
-					s.connsLock.Unlock()
-
-					<-shared.WebsocketSendStream(conn, ptys[i], -1)
+					<-shared.WebsocketSendStream(s.conns[i], ptys[i], -1)
 					ptys[i].Close()
 					wgEOF.Done()
 				}
@@ -259,16 +238,12 @@ func (s *execWs) Do(op *operation) error {
 			tty.Close()
 		}
 
-		s.connsLock.Lock()
-		conn := s.conns[-1]
-		s.connsLock.Unlock()
-
-		if conn == nil {
+		if s.conns[-1] == nil {
 			if s.interactive {
 				controlExit <- true
 			}
 		} else {
-			conn.Close()
+			s.conns[-1].Close()
 		}
 
 		attachedChildIsDead <- true
@@ -288,7 +263,7 @@ func (s *execWs) Do(op *operation) error {
 		return cmdErr
 	}
 
-	cmd, _, attachedPid, err := s.container.Exec(s.command, s.env, stdin, stdout, stderr, false)
+	pid, attachedPid, err := s.container.Exec(s.command, s.env, stdin, stdout, stderr, false)
 	if err != nil {
 		return err
 	}
@@ -297,21 +272,31 @@ func (s *execWs) Do(op *operation) error {
 		attachedChildIsBorn <- attachedPid
 	}
 
-	err = cmd.Wait()
-	if err == nil {
+	proc, err := os.FindProcess(pid)
+	if err != nil {
+		return finisher(-1, fmt.Errorf("Failed finding process: %q", err))
+	}
+
+	procState, err := proc.Wait()
+	if err != nil {
+		return finisher(-1, fmt.Errorf("Failed waiting on process %d: %q", pid, err))
+	}
+
+	if procState.Success() {
 		return finisher(0, nil)
 	}
 
-	exitErr, ok := err.(*exec.ExitError)
+	status, ok := procState.Sys().(syscall.WaitStatus)
 	if ok {
-		status, ok := exitErr.Sys().(syscall.WaitStatus)
-		if ok {
+		if status.Exited() {
 			return finisher(status.ExitStatus(), nil)
 		}
-
+		// Backwards compatible behavior. Report success when we exited
+		// due to a signal. Otherwise this may break Jenkins, e.g. when
+		// lxc exec foo reboot receives SIGTERM and status.Exitstats()
+		// would report -1.
 		if status.Signaled() {
-			// COMMENT(brauner): 128 + n == Fatal error signal "n"
-			return finisher(128+int(status.Signal()), nil)
+			return finisher(0, nil)
 		}
 	}
 
@@ -320,7 +305,7 @@ func (s *execWs) Do(op *operation) error {
 
 func containerExecPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d.State(), d.Storage, name)
+	c, err := containerLoadByName(d, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -333,7 +318,7 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Container is frozen."))
 	}
 
-	post := api.ContainerExecPost{}
+	post := commandPostContent{}
 	buf, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return BadRequest(err)
@@ -357,7 +342,6 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	// Set default value for PATH
 	_, ok := env["PATH"]
 	if !ok {
 		env["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
@@ -366,37 +350,13 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	// Set default value for HOME
-	_, ok = env["HOME"]
-	if !ok {
-		env["HOME"] = "/root"
-	}
-
-	// Set default value for USER
-	_, ok = env["USER"]
-	if !ok {
-		env["USER"] = "root"
-	}
-
-	// Set default value for USER
-	_, ok = env["LANG"]
-	if !ok {
-		env["LANG"] = "C.UTF-8"
-	}
-
 	if post.WaitForWS {
 		ws := &execWs{}
 		ws.fds = map[int]string{}
-
-		idmapset, err := c.IdmapSet()
-		if err != nil {
-			return InternalError(err)
-		}
-
+		idmapset := c.IdmapSet()
 		if idmapset != nil {
 			ws.rootUid, ws.rootGid = idmapset.ShiftIntoNs(0, 0)
 		}
-
 		ws.conns = map[int]*websocket.Conn{}
 		ws.conns[-1] = nil
 		ws.conns[0] = nil
@@ -433,13 +393,43 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 	}
 
 	run := func(op *operation) error {
-		_, cmdResult, _, cmdErr := c.Exec(post.Command, env, nil, nil, nil, true)
-		metadata := shared.Jmap{"return": cmdResult}
+		var cmdErr error
+		var cmdResult int
+		metadata := shared.Jmap{}
+
+		if post.RecordOutput {
+			// Prepare stdout and stderr recording
+			stdout, err := os.OpenFile(filepath.Join(c.LogPath(), fmt.Sprintf("exec_%s.stdout", op.id)), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
+			if err != nil {
+				return err
+			}
+			defer stdout.Close()
+
+			stderr, err := os.OpenFile(filepath.Join(c.LogPath(), fmt.Sprintf("exec_%s.stderr", op.id)), os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
+			if err != nil {
+				return err
+			}
+			defer stderr.Close()
+
+			// Run the command
+			cmdResult, _, cmdErr = c.Exec(post.Command, env, nil, stdout, stderr, true)
+
+			// Update metadata with the right URLs
+			metadata["return"] = cmdResult
+			metadata["output"] = shared.Jmap{
+				"1": fmt.Sprintf("/%s/containers/%s/logs/%s", version.APIVersion, c.Name(), filepath.Base(stdout.Name())),
+				"2": fmt.Sprintf("/%s/containers/%s/logs/%s", version.APIVersion, c.Name(), filepath.Base(stderr.Name())),
+			}
+		} else {
+			cmdResult, _, cmdErr = c.Exec(post.Command, env, nil, nil, nil, true)
+			metadata["return"] = cmdResult
+		}
 
 		err = op.UpdateMetadata(metadata)
 		if err != nil {
-			logger.Error("error updating metadata for cmd", log.Ctx{"err": err, "cmd": post.Command})
+			shared.LogError("error updating metadata for cmd", log.Ctx{"err": err, "cmd": post.Command})
 		}
+
 		return cmdErr
 	}
 
