@@ -5,15 +5,19 @@ import (
 	"fmt"
 	"net/http"
 	"strconv"
+	"strings"
 
 	"github.com/gorilla/mux"
 
-	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 )
 
+type containerSnapshotPostReq struct {
+	Name     string `json:"name"`
+	Stateful bool   `json:"stateful"`
+}
+
 func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 	recursionStr := r.FormValue("recursion")
 	recursion, err := strconv.Atoi(recursionStr)
@@ -22,7 +26,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 	}
 
 	cname := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d.State(), d.Storage, cname)
+	c, err := containerLoadByName(d, cname)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -33,20 +37,20 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 	}
 
 	resultString := []string{}
-	resultMap := []*api.ContainerSnapshot{}
+	resultMap := []*shared.SnapshotInfo{}
 
 	for _, snap := range snaps {
-		_, snapName, _ := containerGetParentAndSnapshotName(snap.Name())
+		snapName := strings.SplitN(snap.Name(), shared.SnapshotDelimiter, 2)[1]
 		if recursion == 0 {
 			url := fmt.Sprintf("/%s/containers/%s/snapshots/%s", version.APIVersion, cname, snapName)
 			resultString = append(resultString, url)
 		} else {
-			render, err := snap.Render()
+			render, _, err := snap.Render()
 			if err != nil {
 				continue
 			}
 
-			resultMap = append(resultMap, render.(*api.ContainerSnapshot))
+			resultMap = append(resultMap, render.(*shared.SnapshotInfo))
 		}
 	}
 
@@ -64,11 +68,11 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 func nextSnapshot(d *Daemon, name string) int {
 	base := name + shared.SnapshotDelimiter + "snap"
 	length := len(base)
-	q := fmt.Sprintf("SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
+	q := fmt.Sprintf("SELECT MAX(name) FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
 	var numstr string
-	inargs := []interface{}{db.CTypeSnapshot, length, base}
+	inargs := []interface{}{cTypeSnapshot, length, base}
 	outfmt := []interface{}{numstr}
-	results, err := db.QueryScan(d.db, q, inargs, outfmt)
+	results, err := dbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return 0
 	}
@@ -102,12 +106,12 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 	 * 2. copy the database info over
 	 * 3. copy over the rootfs
 	 */
-	c, err := containerLoadByName(d.State(), d.Storage, name)
+	c, err := containerLoadByName(d, name)
 	if err != nil {
 		return SmartError(err)
 	}
 
-	req := api.ContainerSnapshotsPost{}
+	req := containerSnapshotPostReq{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -123,9 +127,9 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 		req.Name
 
 	snapshot := func(op *operation) error {
-		args := db.ContainerArgs{
+		args := containerArgs{
 			Name:         fullName,
-			Ctype:        db.CTypeSnapshot,
+			Ctype:        cTypeSnapshot,
 			Config:       c.LocalConfig(),
 			Profiles:     c.Profiles(),
 			Ephemeral:    c.IsEphemeral(),
@@ -135,7 +139,7 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 			Stateful:     req.Stateful,
 		}
 
-		_, err := containerCreateAsSnapshot(d.State(), d.Storage, args, c)
+		_, err := containerCreateAsSnapshot(d, args, c)
 		if err != nil {
 			return err
 		}
@@ -159,8 +163,7 @@ func snapshotHandler(d *Daemon, r *http.Request) Response {
 	snapshotName := mux.Vars(r)["snapshotName"]
 
 	sc, err := containerLoadByName(
-		d.State(),
-		d.Storage,
+		d,
 		containerName+
 			shared.SnapshotDelimiter+
 			snapshotName)
@@ -181,12 +184,12 @@ func snapshotHandler(d *Daemon, r *http.Request) Response {
 }
 
 func snapshotGet(sc container, name string) Response {
-	render, err := sc.Render()
+	render, _, err := sc.Render()
 	if err != nil {
 		return SmartError(err)
 	}
 
-	return SyncResponse(true, render.(*api.ContainerSnapshot))
+	return SyncResponse(true, render.(*shared.SnapshotInfo))
 }
 
 func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string) Response {
@@ -221,7 +224,7 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 	fullName := containerName + shared.SnapshotDelimiter + newName
 
 	// Check that the name isn't already in use
-	id, _ := db.ContainerId(d.db, fullName)
+	id, _ := dbContainerId(d.db, fullName)
 	if id > 0 {
 		return Conflict
 	}
