@@ -6,10 +6,7 @@ import (
 	"sync"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -26,6 +23,7 @@ var containerCmd = Command{
 	put:    containerPut,
 	delete: containerDelete,
 	post:   containerPost,
+	patch:  containerPatch,
 }
 
 var containerStateCmd = Command{
@@ -81,9 +79,9 @@ func (slice containerAutostartList) Swap(i, j int) {
 	slice[i], slice[j] = slice[j], slice[i]
 }
 
-func containersRestart(s *state.State, storage storage) error {
+func containersRestart(d *Daemon) error {
 	// Get all the containers
-	result, err := db.ContainersList(s.DB, db.CTypeRegular)
+	result, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
 	}
@@ -91,7 +89,7 @@ func containersRestart(s *state.State, storage storage) error {
 	containers := []container{}
 
 	for _, name := range result {
-		c, err := containerLoadByName(s, storage, name)
+		c, err := containerLoadByName(d, name)
 		if err != nil {
 			return err
 		}
@@ -126,24 +124,24 @@ func containersRestart(s *state.State, storage storage) error {
 	return nil
 }
 
-func containersShutdown(s *state.State, storage storage) error {
+func containersShutdown(d *Daemon) error {
 	var wg sync.WaitGroup
 
 	// Get all the containers
-	results, err := db.ContainersList(s.DB, db.CTypeRegular)
+	results, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
 	}
 
 	// Reset all container states
-	_, err = db.Exec(s.DB, "DELETE FROM containers_config WHERE key='volatile.last_state.power'")
+	_, err = dbExec(d.db, "DELETE FROM containers_config WHERE key='volatile.last_state.power'")
 	if err != nil {
 		return err
 	}
 
 	for _, r := range results {
 		// Load the container
-		c, err := containerLoadByName(s, storage, r)
+		c, err := containerLoadByName(d, r)
 		if err != nil {
 			return err
 		}
@@ -153,9 +151,19 @@ func containersShutdown(s *state.State, storage storage) error {
 
 		// Stop the container
 		if c.IsRunning() {
+			// Determinate how long to wait for the container to shutdown cleanly
+			var timeoutSeconds int
+			value, ok := c.ExpandedConfig()["boot.host_shutdown_timeout"]
+			if ok {
+				timeoutSeconds, _ = strconv.Atoi(value)
+			} else {
+				timeoutSeconds = 30
+			}
+
+			// Stop the container
 			wg.Add(1)
 			go func() {
-				c.Shutdown(time.Second * 30)
+				c.Shutdown(time.Second * time.Duration(timeoutSeconds))
 				c.Stop(false)
 				c.ConfigKeySet("volatile.last_state.power", lastState)
 
@@ -170,19 +178,19 @@ func containersShutdown(s *state.State, storage storage) error {
 	return nil
 }
 
-func containerDeleteSnapshots(s *state.State, storage storage, cname string) error {
-	logger.Debug("containerDeleteSnapshots",
+func containerDeleteSnapshots(d *Daemon, cname string) error {
+	shared.LogDebug("containerDeleteSnapshots",
 		log.Ctx{"container": cname})
 
-	results, err := db.ContainerGetSnapshots(s.DB, cname)
+	results, err := dbContainerGetSnapshots(d.db, cname)
 	if err != nil {
 		return err
 	}
 
 	for _, sname := range results {
-		sc, err := containerLoadByName(s, storage, sname)
+		sc, err := containerLoadByName(d, sname)
 		if err != nil {
-			logger.Error(
+			shared.LogError(
 				"containerDeleteSnapshots: Failed to load the snapshotcontainer",
 				log.Ctx{"container": cname, "snapshot": sname})
 
@@ -190,7 +198,7 @@ func containerDeleteSnapshots(s *state.State, storage storage, cname string) err
 		}
 
 		if err := sc.Delete(); err != nil {
-			logger.Error(
+			shared.LogError(
 				"containerDeleteSnapshots: Failed to delete a snapshotcontainer",
 				log.Ctx{"container": cname, "snapshot": sname, "err": err})
 		}
