@@ -5,42 +5,38 @@ import (
 	"net/http"
 	"time"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/version"
 )
 
 func containersGet(d *Daemon, r *http.Request) Response {
 	for i := 0; i < 100; i++ {
-		result, err := doContainersGet(d.State(), d.Storage, util.IsRecursionRequest(r))
+		result, err := doContainersGet(d, d.isRecursionRequest(r))
 		if err == nil {
 			return SyncResponse(true, result)
 		}
-		if !db.IsDbLockedError(err) {
-			logger.Debugf("DBERR: containersGet: error %q", err)
-			return SmartError(err)
+		if !isDbLockedError(err) {
+			shared.LogDebugf("DBERR: containersGet: error %q", err)
+			return InternalError(err)
 		}
 		// 1 s may seem drastic, but we really don't want to thrash
 		// perhaps we should use a random amount
 		time.Sleep(100 * time.Millisecond)
 	}
 
-	logger.Debugf("DBERR: containersGet, db is locked")
-	logger.Debugf(logger.GetStack())
+	shared.LogDebugf("DBERR: containersGet, db is locked")
+	shared.PrintStack()
 	return InternalError(fmt.Errorf("DB is locked"))
 }
 
-func doContainersGet(s *state.State, storage storage, recursion bool) (interface{}, error) {
-	result, err := db.ContainersList(s.DB, db.CTypeRegular)
+func doContainersGet(d *Daemon, recursion bool) (interface{}, error) {
+	result, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return nil, err
 	}
 
 	resultString := []string{}
-	resultList := []*api.Container{}
+	resultList := []*shared.ContainerInfo{}
 	if err != nil {
 		return []string{}, err
 	}
@@ -50,12 +46,12 @@ func doContainersGet(s *state.State, storage storage, recursion bool) (interface
 			url := fmt.Sprintf("/%s/containers/%s", version.APIVersion, container)
 			resultString = append(resultString, url)
 		} else {
-			c, err := doContainerGet(s, storage, container)
+			c, err := doContainerGet(d, container)
 			if err != nil {
-				c = &api.Container{
+				c = &shared.ContainerInfo{
 					Name:       container,
-					Status:     api.Error.String(),
-					StatusCode: api.Error}
+					Status:     shared.Error.String(),
+					StatusCode: shared.Error}
 			}
 			resultList = append(resultList, c)
 		}
@@ -68,16 +64,16 @@ func doContainersGet(s *state.State, storage storage, recursion bool) (interface
 	return resultList, nil
 }
 
-func doContainerGet(s *state.State, storage storage, cname string) (*api.Container, error) {
-	c, err := containerLoadByName(s, storage, cname)
+func doContainerGet(d *Daemon, cname string) (*shared.ContainerInfo, error) {
+	c, err := containerLoadByName(d, cname)
 	if err != nil {
 		return nil, err
 	}
 
-	cts, err := c.Render()
+	cts, _, err := c.Render()
 	if err != nil {
 		return nil, err
 	}
 
-	return cts.(*api.Container), nil
+	return cts.(*shared.ContainerInfo), nil
 }
