@@ -11,17 +11,55 @@ import (
 	"github.com/dustinkirkland/golang-petname"
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/osarch"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-func createFromImage(d *Daemon, req *api.ContainersPost) Response {
+type containerImageSource struct {
+	Type        string `json:"type"`
+	Certificate string `json:"certificate"`
+
+	/* for "image" type */
+	Alias       string            `json:"alias"`
+	Fingerprint string            `json:"fingerprint"`
+	Properties  map[string]string `json:"properties"`
+	Server      string            `json:"server"`
+	Secret      string            `json:"secret"`
+	Protocol    string            `json:"protocol"`
+
+	/*
+	 * for "migration" and "copy" types, as an optimization users can
+	 * provide an image hash to extract before the filesystem is rsync'd,
+	 * potentially cutting down filesystem transfer time. LXD will not go
+	 * and fetch this image, it will simply use it if it exists in the
+	 * image store.
+	 */
+	BaseImage string `json:"base-image"`
+
+	/* for "migration" type */
+	Mode       string            `json:"mode"`
+	Operation  string            `json:"operation"`
+	Websockets map[string]string `json:"secrets"`
+
+	/* for "copy" type */
+	Source string `json:"source"`
+	/* for "migration" type. Whether the migration is live. */
+	Live bool `json:"live"`
+}
+
+type containerPostReq struct {
+	Architecture string               `json:"architecture"`
+	Config       map[string]string    `json:"config"`
+	Devices      shared.Devices       `json:"devices"`
+	Ephemeral    bool                 `json:"ephemeral"`
+	Name         string               `json:"name"`
+	Profiles     []string             `json:"profiles"`
+	Source       containerImageSource `json:"source"`
+}
+
+func createFromImage(d *Daemon, req *containerPostReq) Response {
 	var hash string
 	var err error
 
@@ -31,32 +69,34 @@ func createFromImage(d *Daemon, req *api.ContainersPost) Response {
 		if req.Source.Server != "" {
 			hash = req.Source.Alias
 		} else {
-			_, alias, err := db.ImageAliasGet(d.db, req.Source.Alias, true)
+			_, alias, err := dbImageAliasGet(d.db, req.Source.Alias, true)
 			if err != nil {
-				return SmartError(err)
+				return InternalError(err)
 			}
 
 			hash = alias.Target
 		}
+	} else if req.Source.Fingerprint != "" {
+		hash = req.Source.Fingerprint
 	} else if req.Source.Properties != nil {
 		if req.Source.Server != "" {
 			return BadRequest(fmt.Errorf("Property match is only supported for local images"))
 		}
 
-		hashes, err := db.ImagesGet(d.db, false)
+		hashes, err := dbImagesGet(d.db, false)
 		if err != nil {
-			return SmartError(err)
+			return InternalError(err)
 		}
 
-		var image *api.Image
+		var image *shared.ImageInfo
 
-		for _, imageHash := range hashes {
-			_, img, err := db.ImageGet(d.db, imageHash, false, true)
+		for _, hash := range hashes {
+			_, img, err := dbImageGet(d.db, hash, false, true)
 			if err != nil {
 				continue
 			}
 
-			if image != nil && img.CreatedAt.Before(image.CreatedAt) {
+			if image != nil && img.CreationDate.Before(image.CreationDate) {
 				continue
 			}
 
@@ -85,36 +125,39 @@ func createFromImage(d *Daemon, req *api.ContainersPost) Response {
 	}
 
 	run := func(op *operation) error {
-		args := db.ContainerArgs{
-			Config:    req.Config,
-			Ctype:     db.CTypeRegular,
-			Devices:   req.Devices,
-			Ephemeral: req.Ephemeral,
-			Name:      req.Name,
-			Profiles:  req.Profiles,
-		}
-
-		var info *api.Image
 		if req.Source.Server != "" {
-			info, err = d.ImageDownload(
+			hash, err = d.ImageDownload(
 				op, req.Source.Server, req.Source.Protocol, req.Source.Certificate, req.Source.Secret,
 				hash, true, daemonConfig["images.auto_update_cached"].GetBool())
 			if err != nil {
 				return err
 			}
-		} else {
-			_, info, err = db.ImageGet(d.db, hash, false, false)
-			if err != nil {
-				return err
-			}
 		}
 
-		args.Architecture, err = osarch.ArchitectureId(info.Architecture)
+		_, imgInfo, err := dbImageGet(d.db, hash, false, false)
 		if err != nil {
 			return err
 		}
 
-		_, err = containerCreateFromImage(d.State(), d.Storage, args, info.Fingerprint)
+		hash = imgInfo.Fingerprint
+
+		architecture, err := osarch.ArchitectureId(imgInfo.Architecture)
+		if err != nil {
+			architecture = 0
+		}
+
+		args := containerArgs{
+			Architecture: architecture,
+			BaseImage:    hash,
+			Config:       req.Config,
+			Ctype:        cTypeRegular,
+			Devices:      req.Devices,
+			Ephemeral:    req.Ephemeral,
+			Name:         req.Name,
+			Profiles:     req.Profiles,
+		}
+
+		_, err = containerCreateFromImage(d, args, hash)
 		return err
 	}
 
@@ -129,22 +172,20 @@ func createFromImage(d *Daemon, req *api.ContainersPost) Response {
 	return OperationResponse(op)
 }
 
-func createFromNone(d *Daemon, req *api.ContainersPost) Response {
-	args := db.ContainerArgs{
-		Config:    req.Config,
-		Ctype:     db.CTypeRegular,
-		Devices:   req.Devices,
-		Ephemeral: req.Ephemeral,
-		Name:      req.Name,
-		Profiles:  req.Profiles,
+func createFromNone(d *Daemon, req *containerPostReq) Response {
+	architecture, err := osarch.ArchitectureId(req.Architecture)
+	if err != nil {
+		architecture = 0
 	}
 
-	if req.Architecture != "" {
-		architecture, err := osarch.ArchitectureId(req.Architecture)
-		if err != nil {
-			return InternalError(err)
-		}
-		args.Architecture = architecture
+	args := containerArgs{
+		Architecture: architecture,
+		Config:       req.Config,
+		Ctype:        cTypeRegular,
+		Devices:      req.Devices,
+		Ephemeral:    req.Ephemeral,
+		Name:         req.Name,
+		Profiles:     req.Profiles,
 	}
 
 	run := func(op *operation) error {
@@ -163,32 +204,30 @@ func createFromNone(d *Daemon, req *api.ContainersPost) Response {
 	return OperationResponse(op)
 }
 
-func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
-	// Validate migration mode
-	if req.Source.Mode != "pull" {
+func createFromMigration(d *Daemon, req *containerPostReq) Response {
+	if req.Source.Mode != "pull" && req.Source.Mode != "push" {
 		return NotImplemented
 	}
 
-	var c container
-
-	// Parse the architecture name
 	architecture, err := osarch.ArchitectureId(req.Architecture)
 	if err != nil {
-		return BadRequest(err)
+		architecture = 0
 	}
 
-	// Prepare the container creation request
-	args := db.ContainerArgs{
+	args := containerArgs{
 		Architecture: architecture,
 		BaseImage:    req.Source.BaseImage,
 		Config:       req.Config,
-		Ctype:        db.CTypeRegular,
+		Ctype:        cTypeRegular,
 		Devices:      req.Devices,
 		Ephemeral:    req.Ephemeral,
 		Name:         req.Name,
 		Profiles:     req.Profiles,
 	}
 
+	var c container
+	_, _, err = dbImageGet(d.db, req.Source.BaseImage, false, true)
+
 	/* Only create a container from an image if we're going to
 	 * rsync over the top of it. In the case of a better file
 	 * transfer mechanism, let's just use that.
@@ -202,9 +241,8 @@ func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
 	 * point and just negotiate it over the migration control
 	 * socket. Anyway, it'll happen later :)
 	 */
-	_, _, err = db.ImageGet(d.db, req.Source.BaseImage, false, true)
 	if err == nil && d.Storage.MigrationType() == MigrationFSType_RSYNC {
-		c, err = containerCreateFromImage(d.State(), d.Storage, args, req.Source.BaseImage)
+		c, err = containerCreateFromImage(d, args, req.Source.BaseImage)
 		if err != nil {
 			return InternalError(err)
 		}
@@ -236,6 +274,11 @@ func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
 		return InternalError(err)
 	}
 
+	push := false
+	if req.Source.Mode == "push" {
+		push = true
+	}
+
 	migrationArgs := MigrationSinkArgs{
 		Url: req.Source.Operation,
 		Dialer: websocket.Dialer{
@@ -243,6 +286,8 @@ func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
 			NetDial:         shared.RFC3493Dialer},
 		Container: c,
 		Secrets:   req.Source.Websockets,
+		Push:      push,
+		Live:      req.Source.Live,
 	}
 
 	sink, err := NewMigrationSink(&migrationArgs)
@@ -251,23 +296,49 @@ func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
 		return InternalError(err)
 	}
 
+	run := func(op *operation) error {
+		// And finaly run the migration.
+		err = sink.Do(op)
+		if err != nil {
+			shared.LogError("Error during migration sink", log.Ctx{"err": err})
+			c.Delete()
+			return fmt.Errorf("Error transferring container data: %s", err)
+		}
+
+		err = c.TemplateApply("copy")
+		if err != nil {
+			c.Delete()
+			return err
+		}
+
+		return nil
+	}
+
 	resources := map[string][]string{}
 	resources["containers"] = []string{req.Name}
 
-	op, err := operationCreate(operationClassTask, resources, nil, sink.Do, nil, nil)
-	if err != nil {
-		return InternalError(err)
+	var op *operation
+	if push {
+		op, err = operationCreate(operationClassWebsocket, resources, sink.Metadata(), run, nil, sink.Connect)
+		if err != nil {
+			return InternalError(err)
+		}
+	} else {
+		op, err = operationCreate(operationClassTask, resources, nil, run, nil, nil)
+		if err != nil {
+			return InternalError(err)
+		}
 	}
 
 	return OperationResponse(op)
 }
 
-func createFromCopy(d *Daemon, req *api.ContainersPost) Response {
+func createFromCopy(d *Daemon, req *containerPostReq) Response {
 	if req.Source.Source == "" {
 		return BadRequest(fmt.Errorf("must specify a source container"))
 	}
 
-	source, err := containerLoadByName(d.State(), d.Storage, req.Source.Source)
+	source, err := containerLoadByName(d, req.Source.Source)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -281,7 +352,7 @@ func createFromCopy(d *Daemon, req *api.ContainersPost) Response {
 
 	for key, value := range sourceConfig {
 		if len(key) > 8 && key[0:8] == "volatile" && !shared.StringInSlice(key[9:], []string{"base_image", "last_state.idmap"}) {
-			logger.Debug("Skipping volatile key from copy source",
+			shared.LogDebug("Skipping volatile key from copy source",
 				log.Ctx{"key": key})
 			continue
 		}
@@ -294,40 +365,24 @@ func createFromCopy(d *Daemon, req *api.ContainersPost) Response {
 		req.Config[key] = value
 	}
 
-	// Devices override
-	sourceDevices := source.LocalDevices()
-
-	if req.Devices == nil {
-		req.Devices = make(map[string]map[string]string)
-	}
-
-	for key, value := range sourceDevices {
-		_, exists := req.Devices[key]
-		if exists {
-			continue
-		}
-
-		req.Devices[key] = value
-	}
-
 	// Profiles override
 	if req.Profiles == nil {
 		req.Profiles = source.Profiles()
 	}
 
-	args := db.ContainerArgs{
+	args := containerArgs{
 		Architecture: source.Architecture(),
 		BaseImage:    req.Source.BaseImage,
 		Config:       req.Config,
-		Ctype:        db.CTypeRegular,
-		Devices:      req.Devices,
+		Ctype:        cTypeRegular,
+		Devices:      source.LocalDevices(),
 		Ephemeral:    req.Ephemeral,
 		Name:         req.Name,
 		Profiles:     req.Profiles,
 	}
 
 	run := func(op *operation) error {
-		_, err := containerCreateAsCopy(d.State(), d.Storage, args, source)
+		_, err := containerCreateAsCopy(d, args, source)
 		if err != nil {
 			return err
 		}
@@ -347,17 +402,17 @@ func createFromCopy(d *Daemon, req *api.ContainersPost) Response {
 }
 
 func containersPost(d *Daemon, r *http.Request) Response {
-	logger.Debugf("Responding to container create")
+	shared.LogDebugf("Responding to container create")
 
-	req := api.ContainersPost{}
+	req := containerPostReq{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
 
 	if req.Name == "" {
-		cs, err := db.ContainersList(d.db, db.CTypeRegular)
+		cs, err := dbContainersList(d.db, cTypeRegular)
 		if err != nil {
-			return SmartError(err)
+			return InternalError(err)
 		}
 
 		i := 0
@@ -372,30 +427,17 @@ func containersPost(d *Daemon, r *http.Request) Response {
 				return InternalError(fmt.Errorf("couldn't generate a new unique name after 100 tries"))
 			}
 		}
-		logger.Debugf("No name provided, creating %s", req.Name)
+		shared.LogDebugf("No name provided, creating %s", req.Name)
 	}
 
 	if req.Devices == nil {
-		req.Devices = types.Devices{}
+		req.Devices = shared.Devices{}
 	}
 
 	if req.Config == nil {
 		req.Config = map[string]string{}
 	}
 
-	if req.InstanceType != "" {
-		conf, err := instanceParseType(req.InstanceType)
-		if err != nil {
-			return BadRequest(err)
-		}
-
-		for k, v := range conf {
-			if req.Config[k] == "" {
-				req.Config[k] = v
-			}
-		}
-	}
-
 	if strings.Contains(req.Name, shared.SnapshotDelimiter) {
 		return BadRequest(fmt.Errorf("Invalid container name: '%s' is reserved for snapshots", shared.SnapshotDelimiter))
 	}
