@@ -5,6 +5,8 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"database/sql"
+	"encoding/hex"
+	"encoding/pem"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -20,34 +22,32 @@ import (
 	"syscall"
 	"time"
 
+	"golang.org/x/crypto/scrypt"
+
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/syndtr/gocapability/capability"
 	"gopkg.in/tomb.v2"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/sys"
-	"github.com/lxc/lxd/lxd/util"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 // AppArmor
-var aaAvailable = false
-var aaAdmin = false
+var aaAdmin = true
+var aaAvailable = true
 var aaConfined = false
 var aaStacking = false
-var aaStacked = false
 
 // CGroup
 var cgBlkioController = false
 var cgCpuController = false
+var cgCpuacctController = false
 var cgCpusetController = false
 var cgDevicesController = false
 var cgMemoryController = false
@@ -65,10 +65,13 @@ type Socket struct {
 
 // A Daemon can respond to requests from a shared client.
 type Daemon struct {
+	architectures       []int
+	BackingFs           string
 	clientCerts         []x509.Certificate
-	os                  *sys.OS
 	db                  *sql.DB
 	group               string
+	IdmapSet            *shared.IdmapSet
+	lxcpath             string
 	mux                 *mux.Router
 	tomb                tomb.Tomb
 	readyChan           chan bool
@@ -83,20 +86,17 @@ type Daemon struct {
 
 	devlxd *net.UnixListener
 
+	MockMode  bool
 	SetupMode bool
 
+	imagesDownloading     map[string]chan bool
+	imagesDownloadingLock sync.RWMutex
+
 	tlsConfig *tls.Config
 
 	proxy func(req *http.Request) (*url.URL, error)
 }
 
-// NewDaemon returns a new, un-initialized Daemon object.
-func NewDaemon() *Daemon {
-	return &Daemon{
-		os: sys.NewOS(),
-	}
-}
-
 // Command is the basic structure for every API call.
 type Command struct {
 	name          string
@@ -106,17 +106,136 @@ type Command struct {
 	put           func(d *Daemon, r *http.Request) Response
 	post          func(d *Daemon, r *http.Request) Response
 	delete        func(d *Daemon, r *http.Request) Response
+	patch         func(d *Daemon, r *http.Request) Response
+}
+
+func (d *Daemon) httpClient(certificate string) (*http.Client, error) {
+	var err error
+	var cert *x509.Certificate
+
+	if certificate != "" {
+		certBlock, _ := pem.Decode([]byte(certificate))
+		if certBlock == nil {
+			return nil, fmt.Errorf("Invalid certificate")
+		}
+
+		cert, err = x509.ParseCertificate(certBlock.Bytes)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	tlsConfig, err := shared.GetTLSConfig("", "", "", cert)
+	if err != nil {
+		return nil, err
+	}
+
+	tr := &http.Transport{
+		TLSClientConfig:   tlsConfig,
+		Dial:              shared.RFC3493Dialer,
+		Proxy:             d.proxy,
+		DisableKeepAlives: true,
+	}
+
+	myhttp := http.Client{
+		Transport: tr,
+	}
+
+	return &myhttp, nil
+}
+
+func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, error) {
+	var err error
+
+	req, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	req.Header.Set("User-Agent", version.UserAgent)
+
+	myhttp, err := d.httpClient(certificate)
+	if err != nil {
+		return nil, err
+	}
+
+	r, err := myhttp.Do(req)
+	if err != nil {
+		return nil, err
+	}
+
+	resp, err := lxd.ParseResponse(r)
+	if err != nil {
+		return nil, err
+	}
+
+	if resp.Type != lxd.Sync {
+		return nil, fmt.Errorf("unexpected non-sync response")
+	}
+
+	return resp, nil
+}
+
+func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, error) {
+	var err error
+
+	myhttp, err := d.httpClient(certificate)
+	if err != nil {
+		return nil, err
+	}
+
+	req, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	req.Header.Set("User-Agent", version.UserAgent)
+
+	raw, err := myhttp.Do(req)
+	if err != nil {
+		return nil, err
+	}
+
+	if raw.StatusCode != 200 {
+		_, err := lxd.HoistResponse(raw, lxd.Error)
+		if err != nil {
+			return nil, err
+		}
+
+		return nil, fmt.Errorf("non-200 status with no error response?")
+	}
+
+	return raw, nil
 }
 
 func readMyCert() (string, string, error) {
 	certf := shared.VarPath("server.crt")
 	keyf := shared.VarPath("server.key")
-	logger.Debug("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
+	shared.LogDebug("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
 	err := shared.FindOrGenCert(certf, keyf, false)
 
 	return certf, keyf, err
 }
 
+func (d *Daemon) isTrustedClient(r *http.Request) bool {
+	if r.RemoteAddr == "@" {
+		// Unix socket
+		return true
+	}
+
+	if r.TLS == nil {
+		return false
+	}
+
+	for i := range r.TLS.PeerCertificates {
+		if d.CheckTrustState(*r.TLS.PeerCertificates[i]) {
+			return true
+		}
+	}
+
+	return false
+}
+
 func isJSONRequest(r *http.Request) bool {
 	for k, vs := range r.Header {
 		if strings.ToLower(k) == "content-type" &&
@@ -128,9 +247,15 @@ func isJSONRequest(r *http.Request) bool {
 	return false
 }
 
-// State creates a new State instance liked to our internal db and os.
-func (d *Daemon) State() *state.State {
-	return state.NewState(d.db, d.os)
+func (d *Daemon) isRecursionRequest(r *http.Request) bool {
+	recursionStr := r.FormValue("recursion")
+
+	recursion, err := strconv.Atoi(recursionStr)
+	if err != nil {
+		return false
+	}
+
+	return recursion == 1
 }
 
 func (d *Daemon) createCmd(version string, c Command) {
@@ -144,20 +269,20 @@ func (d *Daemon) createCmd(version string, c Command) {
 	d.mux.HandleFunc(uri, func(w http.ResponseWriter, r *http.Request) {
 		w.Header().Set("Content-Type", "application/json")
 
-		if util.IsTrustedClient(r, d.clientCerts) {
-			logger.Debug(
+		if d.isTrustedClient(r) {
+			shared.LogDebug(
 				"handling",
 				log.Ctx{"method": r.Method, "url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "GET" && c.untrustedGet {
-			logger.Debug(
+			shared.LogDebug(
 				"allowing untrusted GET",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "POST" && c.untrustedPost {
-			logger.Debug(
+			shared.LogDebug(
 				"allowing untrusted POST",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else {
-			logger.Warn(
+			shared.LogWarn(
 				"rejecting request from untrusted client",
 				log.Ctx{"ip": r.RemoteAddr})
 			Forbidden.Render(w)
@@ -197,6 +322,10 @@ func (d *Daemon) createCmd(version string, c Command) {
 			if c.delete != nil {
 				resp = c.delete(d, r)
 			}
+		case "PATCH":
+			if c.patch != nil {
+				resp = c.patch(d, r)
+			}
 		default:
 			resp = NotFound
 		}
@@ -204,7 +333,7 @@ func (d *Daemon) createCmd(version string, c Command) {
 		if err := resp.Render(w); err != nil {
 			err := InternalError(err).Render(w)
 			if err != nil {
-				logger.Errorf("Failed writing error for error, giving up")
+				shared.LogErrorf("Failed writing error for error, giving up")
 			}
 		}
 
@@ -222,33 +351,74 @@ func (d *Daemon) createCmd(version string, c Command) {
 	})
 }
 
+func (d *Daemon) SetupStorageDriver() error {
+	var err error
+
+	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
+	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
+
+	if lvmVgName != "" {
+		d.Storage, err = newStorage(d, storageTypeLvm)
+		if err != nil {
+			shared.LogErrorf("Could not initialize storage type LVM: %s - falling back to dir", err)
+		} else {
+			return nil
+		}
+	} else if zfsPoolName != "" {
+		d.Storage, err = newStorage(d, storageTypeZfs)
+		if err != nil {
+			shared.LogErrorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
+		} else {
+			return nil
+		}
+	} else if d.BackingFs == "btrfs" {
+		d.Storage, err = newStorage(d, storageTypeBtrfs)
+		if err != nil {
+			shared.LogErrorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
+		} else {
+			return nil
+		}
+	}
+
+	d.Storage, err = newStorage(d, storageTypeDir)
+
+	return err
+}
+
 // have we setup shared mounts?
 var sharedMounted bool
 var sharedMountsLock sync.Mutex
 
 func setupSharedMounts() error {
-	// Check if we already went through this
 	if sharedMounted {
 		return nil
 	}
 
-	// Get a lock to prevent races
 	sharedMountsLock.Lock()
 	defer sharedMountsLock.Unlock()
 
-	// Check if already setup
+	if sharedMounted {
+		return nil
+	}
+
 	path := shared.VarPath("shmounts")
-	if shared.IsMountPoint(path) {
+
+	isShared, err := shared.IsOnSharedMount(path)
+	if err != nil {
+		return err
+	}
+
+	if isShared {
+		// / may already be ms-shared, or shmounts may have
+		// been mounted by a previous lxd run
 		sharedMounted = true
 		return nil
 	}
 
-	// Mount a new tmpfs
-	if err := syscall.Mount("tmpfs", path, "tmpfs", 0, "size=100k,mode=0711"); err != nil {
+	if err := syscall.Mount(path, path, "none", syscall.MS_BIND, ""); err != nil {
 		return err
 	}
 
-	// Mark as MS_SHARED and MS_REC
 	var flags uintptr = syscall.MS_SHARED | syscall.MS_REC
 	if err := syscall.Mount(path, path, "none", flags, ""); err != nil {
 		return err
@@ -258,6 +428,66 @@ func setupSharedMounts() error {
 	return nil
 }
 
+func (d *Daemon) ListenAddresses() ([]string, error) {
+	addresses := make([]string, 0)
+
+	value := daemonConfig["core.https_address"].Get()
+	if value == "" {
+		return addresses, nil
+	}
+
+	localHost, localPort, err := net.SplitHostPort(value)
+	if err != nil {
+		localHost = value
+		localPort = shared.DefaultPort
+	}
+
+	if localHost == "0.0.0.0" || localHost == "::" || localHost == "[::]" {
+		ifaces, err := net.Interfaces()
+		if err != nil {
+			return addresses, err
+		}
+
+		for _, i := range ifaces {
+			addrs, err := i.Addrs()
+			if err != nil {
+				continue
+			}
+
+			for _, addr := range addrs {
+				var ip net.IP
+				switch v := addr.(type) {
+				case *net.IPNet:
+					ip = v.IP
+				case *net.IPAddr:
+					ip = v.IP
+				}
+
+				if !ip.IsGlobalUnicast() {
+					continue
+				}
+
+				if ip.To4() == nil {
+					if localHost == "0.0.0.0" {
+						continue
+					}
+					addresses = append(addresses, fmt.Sprintf("[%s]:%s", ip, localPort))
+				} else {
+					addresses = append(addresses, fmt.Sprintf("%s:%s", ip, localPort))
+				}
+			}
+		}
+	} else {
+		if strings.Contains(localHost, ":") {
+			addresses = append(addresses, fmt.Sprintf("[%s]:%s", localHost, localPort))
+		} else {
+			addresses = append(addresses, fmt.Sprintf("%s:%s", localHost, localPort))
+		}
+	}
+
+	return addresses, nil
+}
+
 func (d *Daemon) UpdateHTTPsPort(newAddress string) error {
 	oldAddress := daemonConfig["core.https_address"].Get()
 
@@ -314,6 +544,8 @@ func haveMacAdmin() bool {
 
 func (d *Daemon) Init() error {
 	/* Initialize some variables */
+	d.imagesDownloading = map[string]chan bool{}
+
 	d.readyChan = make(chan bool)
 	d.shutdownChan = make(chan bool)
 
@@ -325,156 +557,186 @@ func (d *Daemon) Init() error {
 	}
 
 	/* Setup logging if that wasn't done before */
-	if logger.Log == nil {
-		logger.Log, err = logging.GetLogger("", "", true, true, nil)
+	if shared.Log == nil {
+		shared.Log, err = logging.GetLogger("", "", true, true, nil)
 		if err != nil {
 			return err
 		}
 	}
 
 	/* Print welcome message */
-	if d.os.MockMode {
-		logger.Info(fmt.Sprintf("LXD %s is starting in mock mode", version.Version),
+	if d.MockMode {
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in mock mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else if d.SetupMode {
-		logger.Info(fmt.Sprintf("LXD %s is starting in setup mode", version.Version),
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in setup mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else {
-		logger.Info(fmt.Sprintf("LXD %s is starting in normal mode", version.Version),
+		shared.LogInfo(fmt.Sprintf("LXD %s is starting in normal mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	}
 
 	/* Detect user namespaces */
 	runningInUserns = shared.RunningInUserNS()
 
-	/* Detect AppArmor availability */
+	/* Detect AppArmor support */
+	if aaAvailable && os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
+		aaAvailable = false
+		aaAdmin = false
+		shared.LogWarnf("AppArmor support has been manually disabled")
+	}
+
+	if aaAvailable && !shared.IsDir("/sys/kernel/security/apparmor") {
+		aaAvailable = false
+		aaAdmin = false
+		shared.LogWarnf("AppArmor support has been disabled because of lack of kernel support")
+	}
+
 	_, err = exec.LookPath("apparmor_parser")
-	if os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
-		logger.Warnf("AppArmor support has been manually disabled")
-	} else if !shared.IsDir("/sys/kernel/security/apparmor") {
-		logger.Warnf("AppArmor support has been disabled because of lack of kernel support")
-	} else if err != nil {
-		logger.Warnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
-	} else {
-		aaAvailable = true
-	}
-
-	/* Detect AppArmor stacking support */
-	aaCanStack := func() bool {
-		contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
-		if err != nil {
-			return false
-		}
-
-		if string(contentBytes) != "yes\n" {
-			return false
-		}
-
-		contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
-		if err != nil {
-			return false
-		}
-
-		content := string(contentBytes)
-
-		parts := strings.Split(strings.TrimSpace(content), ".")
-
-		if len(parts) == 0 {
-			logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
-			return false
-		}
-
-		major, err := strconv.Atoi(parts[0])
-		if err != nil {
-			logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
-			return false
-		}
-
-		minor := 0
-		if len(parts) == 2 {
-			minor, err = strconv.Atoi(parts[1])
-			if err != nil {
-				logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
-				return false
-			}
-		}
-
-		return major >= 1 && minor >= 2
-	}
-
-	aaStacking = aaCanStack()
-
-	/* Detect existing AppArmor stack */
-	if shared.PathExists("/sys/kernel/security/apparmor/.ns_stacked") {
-		contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/.ns_stacked")
-		if err == nil && string(contentBytes) == "yes\n" {
-			aaStacked = true
-		}
+	if aaAvailable && err != nil {
+		aaAvailable = false
+		aaAdmin = false
+		shared.LogWarnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
 	}
 
 	/* Detect AppArmor admin support */
-	if !haveMacAdmin() {
-		if aaAvailable {
-			logger.Warnf("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
-		}
-	} else if runningInUserns && !aaStacked {
-		if aaAvailable {
-			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container without stacking.")
-		}
-	} else {
-		aaAdmin = true
+	if aaAdmin && !haveMacAdmin() {
+		aaAdmin = false
+		shared.LogWarnf("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
+	}
+
+	if aaAdmin && runningInUserns {
+		aaAdmin = false
+		shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container.")
 	}
 
 	/* Detect AppArmor confinment */
-	profile := aaProfile()
-	if profile != "unconfined" && profile != "" {
-		if aaAvailable {
-			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
+	if !aaConfined {
+		profile := aaProfile()
+		if profile != "unconfined" && profile != "" {
+			aaConfined = true
+			shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
 		}
-		aaConfined = true
+	}
+
+	if aaAvailable {
+		canStack := func() bool {
+			contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
+			if err != nil {
+				return false
+			}
+
+			if string(contentBytes) != "yes\n" {
+				return false
+			}
+
+			contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
+			if err != nil {
+				return false
+			}
+
+			content := string(contentBytes)
+
+			parts := strings.Split(strings.TrimSpace(content), ".")
+
+			if len(parts) == 0 {
+				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+				return false
+			}
+
+			major, err := strconv.Atoi(parts[0])
+			if err != nil {
+				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+				return false
+			}
+
+			minor := 0
+			if len(parts) == 2 {
+				minor, err = strconv.Atoi(parts[1])
+				if err != nil {
+					shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
+					return false
+				}
+			}
+
+			return major >= 1 && minor >= 2
+		}
+
+		aaStacking = canStack()
 	}
 
 	/* Detect CGroup support */
 	cgBlkioController = shared.PathExists("/sys/fs/cgroup/blkio/")
 	if !cgBlkioController {
-		logger.Warnf("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
 	}
 
 	cgCpuController = shared.PathExists("/sys/fs/cgroup/cpu/")
 	if !cgCpuController {
-		logger.Warnf("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
+	}
+
+	cgCpuacctController = shared.PathExists("/sys/fs/cgroup/cpuacct/")
+	if !cgCpuacctController {
+		shared.LogWarnf("Couldn't find the CGroup CPUacct controller, CPU accounting will not be available.")
 	}
 
 	cgCpusetController = shared.PathExists("/sys/fs/cgroup/cpuset/")
 	if !cgCpusetController {
-		logger.Warnf("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
 	}
 
 	cgDevicesController = shared.PathExists("/sys/fs/cgroup/devices/")
 	if !cgDevicesController {
-		logger.Warnf("Couldn't find the CGroup devices controller, device access control won't work.")
+		shared.LogWarnf("Couldn't find the CGroup devices controller, device access control won't work.")
 	}
 
 	cgMemoryController = shared.PathExists("/sys/fs/cgroup/memory/")
 	if !cgMemoryController {
-		logger.Warnf("Couldn't find the CGroup memory controller, memory limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup memory controller, memory limits will be ignored.")
 	}
 
 	cgNetPrioController = shared.PathExists("/sys/fs/cgroup/net_prio/")
 	if !cgNetPrioController {
-		logger.Warnf("Couldn't find the CGroup network class controller, network limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup network class controller, network limits will be ignored.")
 	}
 
 	cgPidsController = shared.PathExists("/sys/fs/cgroup/pids/")
 	if !cgPidsController {
-		logger.Warnf("Couldn't find the CGroup pids controller, process limits will be ignored.")
+		shared.LogWarnf("Couldn't find the CGroup pids controller, process limits will be ignored.")
 	}
 
 	cgSwapAccounting = shared.PathExists("/sys/fs/cgroup/memory/memory.memsw.limit_in_bytes")
 	if !cgSwapAccounting {
-		logger.Warnf("CGroup memory swap accounting is disabled, swap limits will be ignored.")
+		shared.LogWarnf("CGroup memory swap accounting is disabled, swap limits will be ignored.")
 	}
 
+	/* Get the list of supported architectures */
+	var architectures = []int{}
+
+	architectureName, err := osarch.ArchitectureGetLocal()
+	if err != nil {
+		return err
+	}
+
+	architecture, err := osarch.ArchitectureId(architectureName)
+	if err != nil {
+		return err
+	}
+	architectures = append(architectures, architecture)
+
+	personalities, err := osarch.ArchitecturePersonalities(architecture)
+	if err != nil {
+		return err
+	}
+	for _, personality := range personalities {
+		architectures = append(architectures, personality)
+	}
+	d.architectures = architectures
+
+	/* Set container path */
+	d.lxcpath = shared.VarPath("containers")
+
 	/* Make sure all our directories are available */
 	if err := os.MkdirAll(shared.VarPath(), 0711); err != nil {
 		return err
@@ -507,10 +769,22 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	/* Initialize the operating system facade */
-	err = d.os.Init()
+	/* Detect the filesystem */
+	d.BackingFs, err = filesystemDetect(d.lxcpath)
 	if err != nil {
-		return err
+		shared.LogError("Error detecting backing fs", log.Ctx{"err": err})
+	}
+
+	/* Read the uid/gid allocation */
+	d.IdmapSet, err = shared.DefaultIdmapSet()
+	if err != nil {
+		shared.LogWarn("Error reading idmap", log.Ctx{"err": err.Error()})
+		shared.LogWarnf("Only privileged containers will be able to run")
+	} else {
+		shared.LogInfof("Default uid/gid map:")
+		for _, lxcmap := range d.IdmapSet.ToLxcString() {
+			shared.LogInfof(strings.TrimRight(" - "+lxcmap, "\n"))
+		}
 	}
 
 	/* Initialize the database */
@@ -525,9 +799,9 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	if !d.os.MockMode {
+	if !d.MockMode {
 		/* Setup the storage driver */
-		err = SetupStorageDriver(d)
+		err = d.SetupStorageDriver()
 		if err != nil {
 			return fmt.Errorf("Failed to setup storage: %s", err)
 		}
@@ -538,6 +812,12 @@ func (d *Daemon) Init() error {
 			return err
 		}
 
+		/* Setup the networks */
+		err = networkStartup(d)
+		if err != nil {
+			return err
+		}
+
 		/* Restore simplestreams cache */
 		err = imageLoadStreamCache(d)
 		if err != nil {
@@ -549,14 +829,14 @@ func (d *Daemon) Init() error {
 	go func() {
 		t := time.NewTicker(24 * time.Hour)
 		for {
-			logger.Infof("Expiring log files")
+			shared.LogInfof("Expiring log files")
 
-			err := ExpireLogs(d.db)
+			err := d.ExpireLogs()
 			if err != nil {
-				logger.Error("Failed to expire logs", log.Ctx{"err": err})
+				shared.LogError("Failed to expire logs", log.Ctx{"err": err})
 			}
 
-			logger.Infof("Done expiring log files")
+			shared.LogInfof("Done expiring log files")
 			<-t.C
 		}
 	}()
@@ -568,32 +848,16 @@ func (d *Daemon) Init() error {
 		daemonConfig["core.proxy_ignore_hosts"].Get(),
 	)
 
-	/* Setup some mounts (nice to have) */
-	if !d.os.MockMode {
-		// Attempt to mount the shmounts tmpfs
-		setupSharedMounts()
-
-		// Attempt to Mount the devlxd tmpfs
-		if !shared.IsMountPoint(shared.VarPath("devlxd")) {
-			syscall.Mount("tmpfs", shared.VarPath("devlxd"), "tmpfs", 0, "size=100k,mode=0755")
-		}
-	}
-
 	/* Setup /dev/lxd */
-	logger.Infof("Starting /dev/lxd handler")
+	shared.LogInfof("Starting /dev/lxd handler")
 	d.devlxd, err = createAndBindDevLxd()
 	if err != nil {
 		return err
 	}
 
-	d.tomb.Go(func() error {
-		server := devLxdServer(d)
-		return server.Serve(d.devlxd)
-	})
-
-	if !d.os.MockMode {
+	if !d.MockMode {
 		/* Start the scheduler */
-		go deviceEventListener(d.State(), d.Storage)
+		go deviceEventListener(d)
 
 		/* Setup the TLS authentication */
 		certf, keyf, err := readMyCert()
@@ -612,16 +876,25 @@ func (d *Daemon) Init() error {
 			MinVersion:   tls.VersionTLS12,
 			MaxVersion:   tls.VersionTLS12,
 			CipherSuites: []uint16{
-				tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
-				tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
-				tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
-				tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
-				tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
-				tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
 				tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
 				tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA},
 			PreferServerCipherSuites: true,
 		}
+
+		if shared.PathExists(shared.VarPath("server.ca")) {
+			ca, err := shared.ReadCert(shared.VarPath("server.ca"))
+			if err != nil {
+				return err
+			}
+
+			caPool := x509.NewCertPool()
+			caPool.AddCert(ca)
+			tlsConfig.RootCAs = caPool
+			tlsConfig.ClientCAs = caPool
+
+			shared.LogInfof("LXD is in CA mode, only CA-signed certificates will be allowed")
+		}
+
 		tlsConfig.BuildNameToCertificate()
 
 		d.tlsConfig = tlsConfig
@@ -647,15 +920,14 @@ func (d *Daemon) Init() error {
 	}
 
 	d.mux.NotFoundHandler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		logger.Info("Sending top level 404", log.Ctx{"url": r.URL})
+		shared.LogInfo("Sending top level 404", log.Ctx{"url": r.URL})
 		w.Header().Set("Content-Type", "application/json")
 		NotFound.Render(w)
 	})
 
-	// Prepare the list of listeners
-	listeners := util.GetListeners()
+	listeners := d.GetListeners()
 	if len(listeners) > 0 {
-		logger.Infof("LXD is socket activated")
+		shared.LogInfof("LXD is socket activated")
 
 		for _, listener := range listeners {
 			if shared.PathExists(listener.Addr().String()) {
@@ -666,16 +938,16 @@ func (d *Daemon) Init() error {
 			}
 		}
 	} else {
-		logger.Infof("LXD isn't socket activated")
+		shared.LogInfof("LXD isn't socket activated")
 
 		localSocketPath := shared.VarPath("unix.socket")
 
 		// If the socket exists, let's try to connect to it and see if there's
 		// a lxd running.
 		if shared.PathExists(localSocketPath) {
-			_, err := lxd.ConnectLXDUnix("", nil)
+			_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
 			if err != nil {
-				logger.Debugf("Detected stale unix socket, deleting")
+				shared.LogDebugf("Detected stale unix socket, deleting")
 				// Connecting failed, so let's delete the socket and
 				// listen on it ourselves.
 				err = os.Remove(localSocketPath)
@@ -727,30 +999,36 @@ func (d *Daemon) Init() error {
 
 		tcpl, err := tls.Listen("tcp", listenAddr, d.tlsConfig)
 		if err != nil {
-			logger.Error("cannot listen on https socket, skipping...", log.Ctx{"err": err})
+			shared.LogError("cannot listen on https socket, skipping...", log.Ctx{"err": err})
 		} else {
 			if d.TCPSocket != nil {
-				logger.Infof("Replacing inherited TCP socket with configured one")
+				shared.LogInfof("Replacing inherited TCP socket with configured one")
 				d.TCPSocket.Socket.Close()
 			}
 			d.TCPSocket = &Socket{Socket: tcpl, CloseOnExit: true}
 		}
 	}
 
-	// Bind the REST API
-	logger.Infof("REST API daemon:")
-	if d.UnixSocket != nil {
-		logger.Info(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
-		d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
-	}
+	d.tomb.Go(func() error {
+		shared.LogInfof("REST API daemon:")
+		if d.UnixSocket != nil {
+			shared.LogInfo(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
+			d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
+		}
 
-	if d.TCPSocket != nil {
-		logger.Info(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
-		d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
-	}
+		if d.TCPSocket != nil {
+			shared.LogInfo(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
+			d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
+		}
 
-	// Run the post initialization actions
-	if !d.os.MockMode && !d.SetupMode {
+		d.tomb.Go(func() error {
+			server := devLxdServer(d)
+			return server.Serve(d.devlxd)
+		})
+		return nil
+	})
+
+	if !d.MockMode && !d.SetupMode {
 		err := d.Ready()
 		if err != nil {
 			return err
@@ -764,10 +1042,12 @@ func (d *Daemon) Ready() error {
 	/* Prune images */
 	d.pruneChan = make(chan bool)
 	go func() {
+		pruneExpiredImages(d)
 		for {
 			timer := time.NewTimer(24 * time.Hour)
+			timeChan := timer.C
 			select {
-			case <-timer.C:
+			case <-timeChan:
 				/* run once per day */
 				pruneExpiredImages(d)
 			case <-d.pruneChan:
@@ -778,9 +1058,6 @@ func (d *Daemon) Ready() error {
 		}
 	}()
 
-	// Do an initial pruning run before we start updating images
-	pruneExpiredImages(d)
-
 	/* Auto-update images */
 	d.resetAutoUpdateChan = make(chan bool)
 	go func() {
@@ -795,9 +1072,10 @@ func (d *Daemon) Ready() error {
 			interval := daemonConfig["images.auto_update_interval"].GetInt64()
 			if interval > 0 {
 				timer := time.NewTimer(time.Duration(interval) * time.Hour)
+				timeChan := timer.C
 
 				select {
-				case <-timer.C:
+				case <-timeChan:
 					autoUpdateImages(d)
 				case <-d.resetAutoUpdateChan:
 					timer.Stop()
@@ -811,37 +1089,38 @@ func (d *Daemon) Ready() error {
 		}
 	}()
 
-	/* Auto-update instance types */
-	go func() {
-		// Background update
-		for {
-			instanceRefreshTypes(d)
-			time.Sleep(24 * time.Hour)
-		}
-	}()
-
-	s := d.State()
-
 	/* Restore containers */
-	containersRestart(s, d.Storage)
+	containersRestart(d)
 
 	/* Re-balance in case things changed while LXD was down */
-	deviceTaskBalance(s, d.Storage)
+	deviceTaskBalance(d)
 
 	close(d.readyChan)
 
 	return nil
 }
 
+// CheckTrustState returns True if the client is trusted else false.
+func (d *Daemon) CheckTrustState(cert x509.Certificate) bool {
+	for k, v := range d.clientCerts {
+		if bytes.Compare(cert.Raw, v.Raw) == 0 {
+			shared.LogDebug("Found cert", log.Ctx{"k": k})
+			return true
+		}
+	}
+
+	return false
+}
+
 func (d *Daemon) numRunningContainers() (int, error) {
-	results, err := db.ContainersList(d.db, db.CTypeRegular)
+	results, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return 0, err
 	}
 
 	count := 0
 	for _, r := range results {
-		container, err := containerLoadByName(d.State(), d.Storage, r)
+		container, err := containerLoadByName(d, r)
 		if err != nil {
 			continue
 		}
@@ -861,44 +1140,43 @@ func (d *Daemon) Stop() error {
 	forceStop := false
 
 	d.tomb.Kill(errStop)
-	logger.Infof("Stopping REST API handler:")
+	shared.LogInfof("Stopping REST API handler:")
 	for _, socket := range []*Socket{d.TCPSocket, d.UnixSocket} {
 		if socket == nil {
 			continue
 		}
 
 		if socket.CloseOnExit {
-			logger.Info(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
+			shared.LogInfo(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
 			socket.Socket.Close()
 		} else {
-			logger.Info(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
+			shared.LogInfo(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
 			forceStop = true
 		}
 	}
 
-	logger.Infof("Stopping /dev/lxd handler")
-	d.devlxd.Close()
-	logger.Infof("Stopped /dev/lxd handler")
-
 	if n, err := d.numRunningContainers(); err != nil || n == 0 {
-		logger.Infof("Unmounting temporary filesystems")
+		shared.LogInfof("Unmounting shmounts")
 
-		syscall.Unmount(shared.VarPath("devlxd"), syscall.MNT_DETACH)
 		syscall.Unmount(shared.VarPath("shmounts"), syscall.MNT_DETACH)
 
-		logger.Infof("Done unmounting temporary filesystems")
+		shared.LogInfof("Done unmounting shmounts")
 	} else {
-		logger.Debugf("Not unmounting temporary filesystems (containers are still running)")
+		shared.LogDebugf("Not unmounting shmounts (containers are still running)")
 	}
 
-	logger.Infof("Closing the database")
+	shared.LogInfof("Closing the database")
 	d.db.Close()
 
-	logger.Infof("Saving simplestreams cache")
-	imageSaveStreamCache()
-	logger.Infof("Saved simplestreams cache")
+	shared.LogInfof("Stopping /dev/lxd handler")
+	d.devlxd.Close()
+	shared.LogInfof("Stopped /dev/lxd handler")
 
-	if d.os.MockMode || forceStop {
+	shared.LogInfof("Saving simplestreams cache")
+	imageSaveStreamCache()
+	shared.LogInfof("Saved simplestreams cache")
+
+	if d.MockMode || forceStop {
 		return nil
 	}
 
@@ -910,6 +1188,152 @@ func (d *Daemon) Stop() error {
 	return err
 }
 
+func (d *Daemon) PasswordCheck(password string) error {
+	value := daemonConfig["core.trust_password"].Get()
+
+	// No password set
+	if value == "" {
+		return fmt.Errorf("No password is set")
+	}
+
+	// Compare the password
+	buff, err := hex.DecodeString(value)
+	if err != nil {
+		return err
+	}
+
+	salt := buff[0:32]
+	hash, err := scrypt.Key([]byte(password), salt, 1<<14, 8, 1, 64)
+	if err != nil {
+		return err
+	}
+
+	if !bytes.Equal(hash, buff[32:]) {
+		return fmt.Errorf("Bad password provided")
+	}
+
+	return nil
+}
+
+func (d *Daemon) ExpireLogs() error {
+	entries, err := ioutil.ReadDir(shared.LogPath())
+	if err != nil {
+		return err
+	}
+
+	result, err := dbContainersList(d.db, cTypeRegular)
+	if err != nil {
+		return err
+	}
+
+	newestFile := func(path string, dir os.FileInfo) time.Time {
+		newest := dir.ModTime()
+
+		entries, err := ioutil.ReadDir(path)
+		if err != nil {
+			return newest
+		}
+
+		for _, entry := range entries {
+			if entry.ModTime().After(newest) {
+				newest = entry.ModTime()
+			}
+		}
+
+		return newest
+	}
+
+	for _, entry := range entries {
+		// Check if the container still exists
+		if shared.StringInSlice(entry.Name(), result) {
+			// Remove any log file which wasn't modified in the past 48 hours
+			logs, err := ioutil.ReadDir(shared.LogPath(entry.Name()))
+			if err != nil {
+				return err
+			}
+
+			for _, logfile := range logs {
+				path := shared.LogPath(entry.Name(), logfile.Name())
+
+				// Always keep the LXC config
+				if logfile.Name() == "lxc.conf" {
+					continue
+				}
+
+				// Deal with directories (snapshots)
+				if logfile.IsDir() {
+					newest := newestFile(path, logfile)
+					if time.Since(newest).Hours() >= 48 {
+						os.RemoveAll(path)
+						if err != nil {
+							return err
+						}
+					}
+
+					continue
+				}
+
+				// Individual files
+				if time.Since(logfile.ModTime()).Hours() >= 48 {
+					err := os.Remove(path)
+					if err != nil {
+						return err
+					}
+				}
+			}
+		} else {
+			// Empty directory if unchanged in the past 24 hours
+			path := shared.LogPath(entry.Name())
+			newest := newestFile(path, entry)
+			if time.Since(newest).Hours() >= 24 {
+				err := os.RemoveAll(path)
+				if err != nil {
+					return err
+				}
+			}
+		}
+	}
+
+	return nil
+}
+
+func (d *Daemon) GetListeners() []net.Listener {
+	defer func() {
+		os.Unsetenv("LISTEN_PID")
+		os.Unsetenv("LISTEN_FDS")
+	}()
+
+	pid, err := strconv.Atoi(os.Getenv("LISTEN_PID"))
+	if err != nil {
+		return nil
+	}
+
+	if pid != os.Getpid() {
+		return nil
+	}
+
+	fds, err := strconv.Atoi(os.Getenv("LISTEN_FDS"))
+	if err != nil {
+		return nil
+	}
+
+	listeners := []net.Listener{}
+
+	for i := 3; i < 3+fds; i++ {
+		syscall.CloseOnExec(i)
+
+		file := os.NewFile(uintptr(i), fmt.Sprintf("inherited-fd%d", i))
+		listener, err := net.FileListener(file)
+		if err != nil {
+			continue
+		}
+
+		listeners = append(listeners, listener)
+	}
+
+	return listeners
+}
+
 type lxdHttpServer struct {
 	r *mux.Router
 	d *Daemon
@@ -932,6 +1356,11 @@ func (s *lxdHttpServer) ServeHTTP(rw http.ResponseWriter, req *http.Request) {
 		rw.Header().Set("Access-Control-Allow-Headers", allowedHeaders)
 	}
 
+	allowedCredentials := daemonConfig["core.https_allowed_credentials"].GetBool()
+	if allowedCredentials {
+		rw.Header().Set("Access-Control-Allow-Credentials", "true")
+	}
+
 	// OPTIONS request don't need any further processing
 	if req.Method == "OPTIONS" {
 		return
@@ -940,50 +1369,3 @@ func (s *lxdHttpServer) ServeHTTP(rw http.ResponseWriter, req *http.Request) {
 	// Call the original server
 	s.r.ServeHTTP(rw, req)
 }
-
-// Create a database connection and perform any updates needed.
-func initializeDbObject(d *Daemon, path string) error {
-	var err error
-
-	// Open the database. If the file doesn't exist it is created.
-	d.db, err = db.OpenDb(path)
-	if err != nil {
-		return err
-	}
-
-	// Create the DB if it doesn't exist.
-	err = db.CreateDb(d.db, patchesGetNames())
-	if err != nil {
-		return fmt.Errorf("Error creating database: %s", err)
-	}
-
-	// Detect LXD downgrades
-	if db.GetSchema(d.db) > db.GetLatestSchema() {
-		return fmt.Errorf("The database schema is more recent than LXD's schema.")
-	}
-
-	// Apply any database update.
-	//
-	// NOTE: we use the legacyPatches parameter to run a few
-	// legacy non-db updates that were in place before the
-	// patches mechanism was introduced in lxd/patches.go. The
-	// rest of non-db patches will be applied separately via
-	// patchesApplyAll. See PR #3322 for more details.
-	legacy := map[int]*db.LegacyPatch{}
-	for i, patch := range legacyPatches {
-		legacy[i] = &db.LegacyPatch{
-			Hook: func() error {
-				return patch(d)
-			},
-		}
-	}
-	for _, i := range legacyPatchesNeedingDB {
-		legacy[i].NeedsDB = true
-	}
-	err = db.UpdatesApplyAll(d.db, true, legacy)
-	if err != nil {
-		return err
-	}
-
-	return nil
-}
