@@ -9,8 +9,10 @@ import (
 	"io/ioutil"
 	"math/big"
 	"os"
+	"os/exec"
 	"path"
 	"path/filepath"
+	"regexp"
 	"sort"
 	"strconv"
 	"strings"
@@ -18,11 +20,7 @@ import (
 
 	_ "github.com/mattn/go-sqlite3"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -47,7 +45,263 @@ func (c deviceTaskCPUs) Len() int           { return len(c) }
 func (c deviceTaskCPUs) Less(i, j int) bool { return *c[i].count < *c[j].count }
 func (c deviceTaskCPUs) Swap(i, j int)      { c[i], c[j] = c[j], c[i] }
 
-func deviceNetlinkListener() (chan []string, chan []string, error) {
+type usbDevice struct {
+	action string
+
+	vendor  string
+	product string
+
+	path  string
+	major int
+	minor int
+}
+
+// /dev/nvidia[0-9]+
+type nvidiaGpuCards struct {
+	path  string
+	major int
+	minor int
+	id    string
+}
+
+// {/dev/nvidiactl, /dev/nvidia-uvm, ...}
+type nvidiaGpuDevices struct {
+	path  string
+	major int
+	minor int
+}
+
+// /dev/dri/card0. If we detect that vendor == nvidia, then nvidia will contain
+// the corresponding nvidia car, e.g. {/dev/dri/card1 --> /dev/nvidia1}.
+type gpuDevice struct {
+	vendorid  string
+	productid string
+	id        string // card id e.g. 0
+	// If related devices have the same PCI address as the GPU we should
+	// mount them all. Meaning if we detect /dev/dri/card0,
+	// /dev/dri/controlD64, and /dev/dri/renderD128 with the same PCI
+	// address, then they should all be made available in the container.
+	pci    string
+	nvidia nvidiaGpuCards
+
+	path  string
+	major int
+	minor int
+}
+
+func (g *gpuDevice) isNvidiaGpu() bool {
+	return strings.EqualFold(g.vendorid, "10de")
+}
+
+type cardIds struct {
+	id  string
+	pci string
+}
+
+func deviceLoadGpu() ([]gpuDevice, []nvidiaGpuDevices, error) {
+	const DRI_PATH = "/sys/bus/pci/devices"
+	var gpus []gpuDevice
+	var nvidiaDevices []nvidiaGpuDevices
+	var cards []cardIds
+
+	ents, err := ioutil.ReadDir(DRI_PATH)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil, nil, nil
+		}
+		return nil, nil, err
+	}
+
+	isNvidia := false
+	for _, ent := range ents {
+		// The pci address == the name of the directory. So let's use
+		// this cheap way of retrieving it.
+		pciAddr := ent.Name()
+
+		// Make sure that we are dealing with a GPU by looking whether
+		// the "drm" subfolder exists.
+		drm := filepath.Join(DRI_PATH, pciAddr, "drm")
+		drmEnts, err := ioutil.ReadDir(drm)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Retrieve vendor ID.
+		vendorIdPath := filepath.Join(DRI_PATH, pciAddr, "vendor")
+		vendorId, err := ioutil.ReadFile(vendorIdPath)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Retrieve device ID.
+		productIdPath := filepath.Join(DRI_PATH, pciAddr, "device")
+		productId, err := ioutil.ReadFile(productIdPath)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+		}
+
+		// Store all associated subdevices, e.g. controlD64, renderD128.
+		// The name of the directory == the last part of the
+		// /dev/dri/controlD64 path. So ent.Name() will give us
+		// controlD64.
+		for _, drmEnt := range drmEnts {
+			vendorTmp := strings.TrimSpace(string(vendorId))
+			productTmp := strings.TrimSpace(string(productId))
+			vendorTmp = strings.TrimPrefix(vendorTmp, "0x")
+			productTmp = strings.TrimPrefix(productTmp, "0x")
+			tmpGpu := gpuDevice{
+				pci:       pciAddr,
+				vendorid:  vendorTmp,
+				productid: productTmp,
+				path:      filepath.Join("/dev/dri", drmEnt.Name()),
+			}
+
+			majMinPath := filepath.Join(drm, drmEnt.Name(), "dev")
+			majMinByte, err := ioutil.ReadFile(majMinPath)
+			if err != nil {
+				if os.IsNotExist(err) {
+					continue
+				}
+			}
+			majMin := strings.TrimSpace(string(majMinByte))
+			majMinSlice := strings.Split(string(majMin), ":")
+			if len(majMinSlice) != 2 {
+				continue
+			}
+			majorInt, err := strconv.Atoi(majMinSlice[0])
+			if err != nil {
+				continue
+			}
+			minorInt, err := strconv.Atoi(majMinSlice[1])
+			if err != nil {
+				continue
+			}
+
+			tmpGpu.major = majorInt
+			tmpGpu.minor = minorInt
+
+			isCard, err := regexp.MatchString("^card[0-9]+", drmEnt.Name())
+			if isCard {
+				// If it is a card it's minor number will be its id.
+				tmpGpu.id = strconv.Itoa(minorInt)
+				tmp := cardIds{
+					id:  tmpGpu.id,
+					pci: tmpGpu.pci,
+				}
+				cards = append(cards, tmp)
+			}
+			// Find matching /dev/nvidia* entry for /dev/dri/card*
+			if tmpGpu.isNvidiaGpu() && isCard {
+				if !isNvidia {
+					isNvidia = true
+				}
+				nvidiaPath := "/dev/nvidia" + strconv.Itoa(tmpGpu.minor)
+				stat := syscall.Stat_t{}
+				err := syscall.Stat(nvidiaPath, &stat)
+				if err != nil {
+					continue
+				}
+				tmpGpu.nvidia.path = nvidiaPath
+				tmpGpu.nvidia.major = int(stat.Rdev / 256)
+				tmpGpu.nvidia.minor = int(stat.Rdev % 256)
+				tmpGpu.nvidia.id = strconv.Itoa(tmpGpu.nvidia.minor)
+			}
+			gpus = append(gpus, tmpGpu)
+		}
+	}
+
+	// We detected a Nvidia card, so let's collect all other nvidia devices
+	// that are not /dev/nvidia[0-9]+.
+	if isNvidia {
+		nvidiaEnts, err := ioutil.ReadDir("/dev")
+		if err != nil {
+			if os.IsNotExist(err) {
+				return nil, nil, err
+			}
+		}
+		validNvidia, err := regexp.Compile(`^nvidia[^0-9]+`)
+		if err != nil {
+			return nil, nil, err
+		}
+		for _, nvidiaEnt := range nvidiaEnts {
+			if !validNvidia.MatchString(nvidiaEnt.Name()) {
+				continue
+			}
+			nvidiaPath := filepath.Join("/dev", nvidiaEnt.Name())
+			stat := syscall.Stat_t{}
+			err = syscall.Stat(nvidiaPath, &stat)
+			if err != nil {
+				continue
+			}
+			tmpNividiaGpu := nvidiaGpuDevices{
+				path:  nvidiaPath,
+				major: int(stat.Rdev / 256),
+				minor: int(stat.Rdev % 256),
+			}
+			nvidiaDevices = append(nvidiaDevices, tmpNividiaGpu)
+		}
+
+	}
+
+	// Since we'll give users to ability to specify and id we need to group
+	// devices on the same PCI that belong to the same card by id.
+	for _, card := range cards {
+		for i := 0; i < len(gpus); i++ {
+			if gpus[i].pci == card.pci {
+				gpus[i].id = card.id
+			}
+		}
+	}
+
+	return gpus, nvidiaDevices, nil
+}
+
+func createUSBDevice(action string, vendor string, product string, major string, minor string, busnum string, devnum string, devname string) (usbDevice, error) {
+	majorInt, err := strconv.Atoi(major)
+	if err != nil {
+		return usbDevice{}, err
+	}
+
+	minorInt, err := strconv.Atoi(minor)
+	if err != nil {
+		return usbDevice{}, err
+	}
+
+	path := devname
+	if devname == "" {
+		busnumInt, err := strconv.Atoi(busnum)
+		if err != nil {
+			return usbDevice{}, err
+		}
+
+		devnumInt, err := strconv.Atoi(devnum)
+		if err != nil {
+			return usbDevice{}, err
+		}
+		path = fmt.Sprintf("/dev/bus/usb/%03d/%03d", busnumInt, devnumInt)
+	} else {
+		if !filepath.IsAbs(devname) {
+			path = fmt.Sprintf("/dev/%s", devname)
+		}
+	}
+
+	return usbDevice{
+		action,
+		vendor,
+		product,
+		path,
+		majorInt,
+		minorInt,
+	}, nil
+}
+
+func deviceNetlinkListener() (chan []string, chan []string, chan usbDevice, error) {
 	NETLINK_KOBJECT_UEVENT := 15
 	UEVENT_BUFFER_SIZE := 2048
 
@@ -57,7 +311,7 @@ func deviceNetlinkListener() (chan []string, chan []string, error) {
 	)
 
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, nil, err
 	}
 
 	nl := syscall.SockaddrNetlink{
@@ -68,13 +322,14 @@ func deviceNetlinkListener() (chan []string, chan []string, error) {
 
 	err = syscall.Bind(fd, &nl)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, nil, err
 	}
 
 	chCPU := make(chan []string, 1)
 	chNetwork := make(chan []string, 0)
+	chUSB := make(chan usbDevice)
 
-	go func(chCPU chan []string, chNetwork chan []string) {
+	go func(chCPU chan []string, chNetwork chan []string, chUSB chan usbDevice) {
 		b := make([]byte, UEVENT_BUFFER_SIZE*2)
 		for {
 			_, err := syscall.Read(fd, b)
@@ -130,10 +385,69 @@ func deviceNetlinkListener() (chan []string, chan []string, error) {
 				// Network balancing is interface specific, so queue everything
 				chNetwork <- []string{props["INTERFACE"], props["ACTION"]}
 			}
-		}
-	}(chCPU, chNetwork)
 
-	return chCPU, chNetwork, nil
+			if props["SUBSYSTEM"] == "usb" {
+				if props["ACTION"] != "add" && props["ACTION"] != "remove" {
+					continue
+				}
+
+				parts := strings.Split(props["PRODUCT"], "/")
+				if len(parts) < 2 {
+					continue
+				}
+
+				major, ok := props["MAJOR"]
+				if !ok {
+					continue
+				}
+
+				minor, ok := props["MINOR"]
+				if !ok {
+					continue
+				}
+
+				devname, ok := props["DEVNAME"]
+
+				busnum, ok := props["BUSNUM"]
+				if !ok {
+					continue
+				}
+
+				devnum, ok := props["DEVNUM"]
+				if !ok {
+					continue
+				}
+
+				zeroPad := func(s string, l int) string {
+					return strings.Repeat("0", l-len(s)) + s
+				}
+
+				usb, err := createUSBDevice(
+					props["ACTION"],
+					/* udev doesn't zero pad these, while
+					 * everything else does, so let's zero pad them
+					 * for consistency
+					 */
+					zeroPad(parts[0], 4),
+					zeroPad(parts[1], 4),
+					major,
+					minor,
+					busnum,
+					devnum,
+					devname,
+				)
+				if err != nil {
+					shared.LogError("error reading usb device", log.Ctx{"err": err, "path": props["PHYSDEVPATH"]})
+					continue
+				}
+
+				chUSB <- usb
+			}
+
+		}
+	}(chCPU, chNetwork, chUSB)
+
+	return chCPU, chNetwork, chUSB, nil
 }
 
 func parseCpuset(cpu string) ([]int, error) {
@@ -172,7 +486,7 @@ func parseCpuset(cpu string) ([]int, error) {
 	return cpus, nil
 }
 
-func deviceTaskBalance(s *state.State, storage storage) {
+func deviceTaskBalance(d *Daemon) {
 	min := func(x, y int) int {
 		if x < y {
 			return x
@@ -191,67 +505,30 @@ func deviceTaskBalance(s *state.State, storage storage) {
 		// Older kernel - use cpuset.cpus
 		effectiveCpus, err = cGroupGet("cpuset", "/", "cpuset.cpus")
 		if err != nil {
-			logger.Errorf("Error reading host's cpuset.cpus")
+			shared.LogErrorf("Error reading host's cpuset.cpus")
 			return
 		}
 	}
-
-	effectiveCpusInt, err := parseCpuset(effectiveCpus)
-	if err != nil {
-		logger.Errorf("Error parsing effective CPU set")
-		return
-	}
-
-	isolatedCpusInt := []int{}
-	if shared.PathExists("/sys/devices/system/cpu/isolated") {
-		buf, err := ioutil.ReadFile("/sys/devices/system/cpu/isolated")
-		if err != nil {
-			logger.Errorf("Error reading host's isolated cpu")
-			return
-		}
-
-		// File might exist even though there are no isolated cpus.
-		isolatedCpus := strings.TrimSpace(string(buf))
-		if isolatedCpus != "" {
-			isolatedCpusInt, err = parseCpuset(isolatedCpus)
-			if err != nil {
-				logger.Errorf("Error parsing isolated CPU set: %s", string(isolatedCpus))
-				return
-			}
-		}
-	}
-
-	effectiveCpusSlice := []string{}
-	for _, id := range effectiveCpusInt {
-		if shared.IntInSlice(id, isolatedCpusInt) {
-			continue
-		}
-
-		effectiveCpusSlice = append(effectiveCpusSlice, fmt.Sprintf("%d", id))
-	}
-
-	effectiveCpus = strings.Join(effectiveCpusSlice, ",")
-
 	err = cGroupSet("cpuset", "/lxc", "cpuset.cpus", effectiveCpus)
 	if err != nil && shared.PathExists("/sys/fs/cgroup/cpuset/lxc") {
-		logger.Warn("Error setting lxd's cpuset.cpus", log.Ctx{"err": err})
+		shared.LogWarn("Error setting lxd's cpuset.cpus", log.Ctx{"err": err})
 	}
 	cpus, err := parseCpuset(effectiveCpus)
 	if err != nil {
-		logger.Error("Error parsing host's cpu set", log.Ctx{"cpuset": effectiveCpus, "err": err})
+		shared.LogError("Error parsing host's cpu set", log.Ctx{"cpuset": effectiveCpus, "err": err})
 		return
 	}
 
 	// Iterate through the containers
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
-		logger.Error("problem loading containers list", log.Ctx{"err": err})
+		shared.LogError("problem loading containers list", log.Ctx{"err": err})
 		return
 	}
 	fixedContainers := map[int][]container{}
 	balancedContainers := map[container]int{}
 	for _, name := range containers {
-		c, err := containerLoadByName(s, storage, name)
+		c, err := containerLoadByName(d, name)
 		if err != nil {
 			continue
 		}
@@ -309,7 +586,7 @@ func deviceTaskBalance(s *state.State, storage storage) {
 	for cpu, ctns := range fixedContainers {
 		c, ok := usage[cpu]
 		if !ok {
-			logger.Errorf("Internal error: container using unavailable cpu")
+			shared.LogErrorf("Internal error: container using unavailable cpu")
 			continue
 		}
 		id := c.strId
@@ -358,25 +635,25 @@ func deviceTaskBalance(s *state.State, storage storage) {
 		sort.Strings(set)
 		err := ctn.CGroupSet("cpuset.cpus", strings.Join(set, ","))
 		if err != nil {
-			logger.Error("balance: Unable to set cpuset", log.Ctx{"name": ctn.Name(), "err": err, "value": strings.Join(set, ",")})
+			shared.LogError("balance: Unable to set cpuset", log.Ctx{"name": ctn.Name(), "err": err, "value": strings.Join(set, ",")})
 		}
 	}
 }
 
-func deviceNetworkPriority(s *state.State, storage storage, netif string) {
+func deviceNetworkPriority(d *Daemon, netif string) {
 	// Don't bother running when CGroup support isn't there
 	if !cgNetPrioController {
 		return
 	}
 
-	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
+	containers, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return
 	}
 
 	for _, name := range containers {
 		// Get the container struct
-		c, err := containerLoadByName(s, storage, name)
+		c, err := containerLoadByName(d, name)
 		if err != nil {
 			continue
 		}
@@ -399,10 +676,64 @@ func deviceNetworkPriority(s *state.State, storage storage, netif string) {
 	return
 }
 
-func deviceEventListener(s *state.State, storage storage) {
-	chNetlinkCPU, chNetlinkNetwork, err := deviceNetlinkListener()
+func deviceUSBEvent(d *Daemon, usb usbDevice) {
+	containers, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
-		logger.Errorf("scheduler: couldn't setup netlink listener")
+		shared.LogError("problem loading containers list", log.Ctx{"err": err})
+		return
+	}
+
+	for _, name := range containers {
+		containerIf, err := containerLoadByName(d, name)
+		if err != nil {
+			continue
+		}
+
+		c, ok := containerIf.(*containerLXC)
+		if !ok {
+			shared.LogErrorf("got device event on non-LXC container?")
+			return
+		}
+
+		if !c.IsRunning() {
+			continue
+		}
+
+		devices := c.ExpandedDevices()
+		for _, name := range devices.DeviceNames() {
+			m := devices[name]
+			if m["type"] != "usb" {
+				continue
+			}
+
+			if m["vendorid"] != usb.vendor || (m["productid"] != "" && m["productid"] != usb.product) {
+				continue
+			}
+
+			if usb.action == "add" {
+				err := c.insertUnixDeviceNum(m, usb.major, usb.minor, usb.path)
+				if err != nil {
+					shared.LogError("failed to create usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
+					return
+				}
+			} else if usb.action == "remove" {
+				err := c.removeUnixDeviceNum(m, usb.major, usb.minor, usb.path)
+				if err != nil {
+					shared.LogError("failed to remove usb device", log.Ctx{"err": err, "usb": usb, "container": c.Name()})
+					return
+				}
+			} else {
+				shared.LogError("unknown action for usb device", log.Ctx{"usb": usb})
+				continue
+			}
+		}
+	}
+}
+
+func deviceEventListener(d *Daemon) {
+	chNetlinkCPU, chNetlinkNetwork, chUSB, err := deviceNetlinkListener()
+	if err != nil {
+		shared.LogErrorf("scheduler: couldn't setup netlink listener")
 		return
 	}
 
@@ -410,7 +741,7 @@ func deviceEventListener(s *state.State, storage storage) {
 		select {
 		case e := <-chNetlinkCPU:
 			if len(e) != 2 {
-				logger.Errorf("Scheduler: received an invalid cpu hotplug event")
+				shared.LogErrorf("Scheduler: received an invalid cpu hotplug event")
 				continue
 			}
 
@@ -418,11 +749,11 @@ func deviceEventListener(s *state.State, storage storage) {
 				continue
 			}
 
-			logger.Debugf("Scheduler: cpu: %s is now %s: re-balancing", e[0], e[1])
-			deviceTaskBalance(s, storage)
+			shared.LogDebugf("Scheduler: cpu: %s is now %s: re-balancing", e[0], e[1])
+			deviceTaskBalance(d)
 		case e := <-chNetlinkNetwork:
 			if len(e) != 2 {
-				logger.Errorf("Scheduler: received an invalid network hotplug event")
+				shared.LogErrorf("Scheduler: received an invalid network hotplug event")
 				continue
 			}
 
@@ -430,11 +761,14 @@ func deviceEventListener(s *state.State, storage storage) {
 				continue
 			}
 
-			logger.Debugf("Scheduler: network: %s has been added: updating network priorities", e[0])
-			deviceNetworkPriority(s, storage, e[0])
+			shared.LogDebugf("Scheduler: network: %s has been added: updating network priorities", e[0])
+			deviceNetworkPriority(d, e[0])
+			networkAutoAttach(d, e[0])
+		case e := <-chUSB:
+			deviceUSBEvent(d, e)
 		case e := <-deviceSchedRebalance:
 			if len(e) != 3 {
-				logger.Errorf("Scheduler: received an invalid rebalance event")
+				shared.LogErrorf("Scheduler: received an invalid rebalance event")
 				continue
 			}
 
@@ -442,8 +776,8 @@ func deviceEventListener(s *state.State, storage storage) {
 				continue
 			}
 
-			logger.Debugf("Scheduler: %s %s %s: re-balancing", e[0], e[1], e[2])
-			deviceTaskBalance(s, storage)
+			shared.LogDebugf("Scheduler: %s %s %s: re-balancing", e[0], e[1], e[2])
+			deviceTaskBalance(d)
 		}
 	}
 }
@@ -508,8 +842,8 @@ func deviceGetAttributes(path string) (string, int, int, error) {
 	}
 
 	// Return the device information
-	major := shared.Major(stat.Rdev)
-	minor := shared.Minor(stat.Rdev)
+	major := int(stat.Rdev / 256)
+	minor := int(stat.Rdev % 256)
 	return dType, major, minor, nil
 }
 
@@ -539,8 +873,7 @@ func deviceNextVeth() string {
 }
 
 func deviceRemoveInterface(nic string) error {
-	_, err := shared.RunCommand("ip", "link", "del", "dev", nic)
-	return err
+	return exec.Command("ip", "link", "del", nic).Run()
 }
 
 func deviceMountDisk(srcPath string, dstPath string, readonly bool, recursive bool) error {
@@ -571,14 +904,6 @@ func deviceMountDisk(srcPath string, dstPath string, readonly bool, recursive bo
 		return fmt.Errorf("Unable to mount %s at %s: %s", srcPath, dstPath, err)
 	}
 
-	// Remount bind mounts in readonly mode if requested
-	if readonly == true && flags&syscall.MS_BIND == syscall.MS_BIND {
-		flags = syscall.MS_RDONLY | syscall.MS_BIND | syscall.MS_REMOUNT
-		if err = syscall.Mount("", dstPath, fstype, uintptr(flags), ""); err != nil {
-			return fmt.Errorf("Unable to mount %s in readonly mode: %s", dstPath, err)
-		}
-	}
-
 	flags = syscall.MS_REC | syscall.MS_SLAVE
 	if err = syscall.Mount("", dstPath, "", uintptr(flags), ""); err != nil {
 		return fmt.Errorf("unable to make mount %s private: %s", dstPath, err)
@@ -736,19 +1061,19 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 
 	// Deal with per-filesystem oddities. We don't care about failures here
 	// because any non-special filesystem => directory backend.
-	fs, _ := util.FilesystemDetect(expPath)
+	fs, _ := filesystemDetect(expPath)
 
 	if fs == "zfs" && shared.PathExists("/dev/zfs") {
 		// Accessible zfs filesystems
 		poolName := strings.Split(device[1], "/")[0]
 
-		output, err := shared.RunCommand("zpool", "status", "-P", "-L", poolName)
+		output, err := exec.Command("zpool", "status", poolName).CombinedOutput()
 		if err != nil {
 			return nil, fmt.Errorf("Failed to query zfs filesystem information for %s: %s", device[1], output)
 		}
 
 		header := true
-		for _, line := range strings.Split(output, "\n") {
+		for _, line := range strings.Split(string(output), "\n") {
 			fields := strings.Fields(line)
 			if len(fields) < 5 {
 				continue
@@ -777,6 +1102,12 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 						devices = append(devices, dev)
 					}
 				}
+			} else if deviceIsBlockdev(fmt.Sprintf("/dev/%s", fields[0])) {
+				path = fmt.Sprintf("/dev/%s", fields[0])
+			} else if deviceIsBlockdev(fmt.Sprintf("/dev/disk/by-id/%s", fields[0])) {
+				path = fmt.Sprintf("/dev/disk/by-id/%s", fields[0])
+			} else if deviceIsBlockdev(fmt.Sprintf("/dev/mapper/%s", fields[0])) {
+				path = fmt.Sprintf("/dev/mapper/%s", fields[0])
 			} else {
 				continue
 			}
@@ -796,12 +1127,12 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 		}
 	} else if fs == "btrfs" && shared.PathExists(device[1]) {
 		// Accessible btrfs filesystems
-		output, err := shared.RunCommand("btrfs", "filesystem", "show", device[1])
+		output, err := exec.Command("btrfs", "filesystem", "show", device[1]).CombinedOutput()
 		if err != nil {
 			return nil, fmt.Errorf("Failed to query btrfs filesystem information for %s: %s", device[1], output)
 		}
 
-		for _, line := range strings.Split(output, "\n") {
+		for _, line := range strings.Split(string(output), "\n") {
 			fields := strings.Fields(line)
 			if len(fields) == 0 || fields[0] != "devid" {
 				continue
@@ -867,3 +1198,77 @@ func deviceParseDiskLimit(readSpeed string, writeSpeed string) (int64, int64, in
 
 	return readBps, readIops, writeBps, writeIops, nil
 }
+
+const USB_PATH = "/sys/bus/usb/devices"
+
+func loadRawValues(p string) (map[string]string, error) {
+	values := map[string]string{
+		"idVendor":  "",
+		"idProduct": "",
+		"dev":       "",
+		"busnum":    "",
+		"devnum":    "",
+	}
+
+	for k, _ := range values {
+		v, err := ioutil.ReadFile(path.Join(p, k))
+		if err != nil {
+			return nil, err
+		}
+
+		values[k] = strings.TrimSpace(string(v))
+	}
+
+	return values, nil
+}
+
+func deviceLoadUsb() ([]usbDevice, error) {
+	result := []usbDevice{}
+
+	ents, err := ioutil.ReadDir(USB_PATH)
+	if err != nil {
+		/* if there are no USB devices, let's render an empty list,
+		 * i.e. no usb devices */
+		if os.IsNotExist(err) {
+			return result, nil
+		}
+		return nil, err
+	}
+
+	for _, ent := range ents {
+		values, err := loadRawValues(path.Join(USB_PATH, ent.Name()))
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+
+			return []usbDevice{}, err
+		}
+
+		parts := strings.Split(values["dev"], ":")
+		if len(parts) != 2 {
+			return []usbDevice{}, fmt.Errorf("invalid device value %s", values["dev"])
+		}
+
+		usb, err := createUSBDevice(
+			"add",
+			values["idVendor"],
+			values["idProduct"],
+			parts[0],
+			parts[1],
+			values["busnum"],
+			values["devnum"],
+			values["devname"],
+		)
+		if err != nil {
+			if os.IsNotExist(err) {
+				continue
+			}
+			return nil, err
+		}
+
+		result = append(result, usb)
+	}
+
+	return result, nil
+}
