@@ -16,6 +16,7 @@ import (
 	"strconv"
 	"strings"
 	"sync"
+	"syscall"
 	"time"
 
 	"github.com/gorilla/mux"
@@ -23,6 +24,8 @@ import (
 
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/osarch"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -76,7 +79,7 @@ func detectCompression(fname string) ([]string, string, error) {
 
 }
 
-func unpack(file string, path string) error {
+func unpack(d *Daemon, file string, path string) error {
 	extractArgs, extension, err := detectCompression(file)
 	if err != nil {
 		return err
@@ -115,6 +118,23 @@ func unpack(file string, path string) error {
 
 	output, err := exec.Command(command, args...).CombinedOutput()
 	if err != nil {
+		// Check if we ran out of space
+		fs := syscall.Statfs_t{}
+
+		err1 := syscall.Statfs(path, &fs)
+		if err1 != nil {
+			return err1
+		}
+
+		// Check if we're running out of space
+		if int64(fs.Bfree) < int64(2*fs.Bsize) {
+			if d.Storage.GetStorageType() == storageTypeLvm {
+				return fmt.Errorf("Unable to unpack image, run out of disk space (consider increasing storage.lvm_volume_size).")
+			} else {
+				return fmt.Errorf("Unable to unpack image, run out of disk space.")
+			}
+		}
+
 		co := string(output)
 		shared.LogDebugf("Unpacking failed")
 		shared.LogDebugf(co)
@@ -128,8 +148,8 @@ func unpack(file string, path string) error {
 	return nil
 }
 
-func unpackImage(imagefname string, destpath string) error {
-	err := unpack(imagefname, destpath)
+func unpackImage(d *Daemon, imagefname string, destpath string) error {
+	err := unpack(d, imagefname, destpath)
 	if err != nil {
 		return err
 	}
@@ -141,7 +161,7 @@ func unpackImage(imagefname string, destpath string) error {
 			return fmt.Errorf("Error creating rootfs directory")
 		}
 
-		err = unpack(imagefname+".rootfs", rootfsPath)
+		err = unpack(d, imagefname+".rootfs", rootfsPath)
 		if err != nil {
 			return err
 		}
@@ -252,7 +272,7 @@ func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq,
 	}
 	defer os.Remove(tarfile.Name())
 
-	if err := c.Export(tarfile); err != nil {
+	if err := c.Export(tarfile, req.Properties); err != nil {
 		tarfile.Close()
 		return info, err
 	}
@@ -301,7 +321,7 @@ func imgPostContInfo(d *Daemon, r *http.Request, req imagePostReq,
 		return info, err
 	}
 
-	info.Architecture, _ = shared.ArchitectureName(c.Architecture())
+	info.Architecture, _ = osarch.ArchitectureName(c.Architecture())
 	info.Properties = req.Properties
 
 	return info, nil
@@ -357,22 +377,12 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 		return fmt.Errorf("Missing URL")
 	}
 
-	// Resolve the image URL
-	tlsConfig, err := shared.GetTLSConfig("", "", "", nil)
+	myhttp, err := d.httpClient("")
 	if err != nil {
 		return err
 	}
 
-	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           d.proxy,
-	}
-
-	myhttp := http.Client{
-		Transport: tr,
-	}
-
+	// Resolve the image URL
 	head, err := http.NewRequest("HEAD", req.Source["url"], nil)
 	if err != nil {
 		return err
@@ -383,9 +393,9 @@ func imgPostURLInfo(d *Daemon, req imagePostReq, op *operation) error {
 		architecturesStr = append(architecturesStr, fmt.Sprintf("%d", arch))
 	}
 
-	head.Header.Set("User-Agent", shared.UserAgent)
+	head.Header.Set("User-Agent", version.UserAgent)
 	head.Header.Set("LXD-Server-Architectures", strings.Join(architecturesStr, ", "))
-	head.Header.Set("LXD-Server-Version", shared.Version)
+	head.Header.Set("LXD-Server-Version", version.Version)
 
 	raw, err := myhttp.Do(head)
 	if err != nil {
@@ -450,14 +460,14 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 	sha256 := sha256.New()
 	var size int64
 
-	// Create a temporary file for the image tarball
-	imageTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
-	if err != nil {
-		return info, err
-	}
-	defer os.Remove(imageTarf.Name())
-
 	if ctype == "multipart/form-data" {
+		// Create a temporary file for the image tarball
+		imageTarf, err := ioutil.TempFile(builddir, "lxd_tar_")
+		if err != nil {
+			return info, err
+		}
+		defer os.Remove(imageTarf.Name())
+
 		// Parse the POST data
 		post.Seek(0, 0)
 		mr := multipart.NewReader(post, ctypeParams["boundary"])
@@ -559,9 +569,8 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 		}
 	} else {
 		post.Seek(0, 0)
-		size, err = io.Copy(io.MultiWriter(imageTarf, sha256), post)
+		size, err = io.Copy(sha256, post)
 		info.Size = size
-		imageTarf.Close()
 		logger.Debug("Tar size", log.Ctx{"size": size})
 		if err != nil {
 			logger.Error(
@@ -587,7 +596,7 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 			return info, err
 		}
 
-		imageMeta, err = getImageMetadata(imageTarf.Name())
+		imageMeta, err = getImageMetadata(post.Name())
 		if err != nil {
 			logger.Error(
 				"Failed to get image metadata",
@@ -596,13 +605,13 @@ func getImgPostInfo(d *Daemon, r *http.Request,
 		}
 
 		imgfname := shared.VarPath("images", info.Fingerprint)
-		err = shared.FileMove(imageTarf.Name(), imgfname)
+		err = shared.FileMove(post.Name(), imgfname)
 		if err != nil {
 			logger.Error(
 				"Failed to move the tarfile",
 				log.Ctx{
 					"err":    err,
-					"source": imageTarf.Name(),
+					"source": post.Name(),
 					"dest":   imgfname})
 			return info, err
 		}
@@ -797,7 +806,7 @@ func getImageMetadata(fname string) (*imageMetadata, error) {
 		return nil, fmt.Errorf("Could not parse %s: %v", metadataName, err)
 	}
 
-	_, err = shared.ArchitectureId(metadata.Architecture)
+	_, err = osarch.ArchitectureId(metadata.Architecture)
 	if err != nil {
 		return nil, err
 	}
@@ -820,7 +829,7 @@ func doImagesGet(d *Daemon, recursion bool, public bool) (interface{}, error) {
 	i := 0
 	for _, name := range results {
 		if !recursion {
-			url := fmt.Sprintf("/%s/images/%s", shared.APIVersion, name)
+			url := fmt.Sprintf("/%s/images/%s", version.APIVersion, name)
 			resultString[i] = url
 		} else {
 			image, response := doImageGet(d, name, public)
@@ -1194,7 +1203,7 @@ func aliasesPost(d *Daemon, r *http.Request) Response {
 		return InternalError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, req.Name))
 }
 
 func aliasesGet(d *Daemon, r *http.Request) Response {
@@ -1213,7 +1222,7 @@ func aliasesGet(d *Daemon, r *http.Request) Response {
 	for _, res := range results {
 		name = res[0].(string)
 		if !recursion {
-			url := fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, name)
+			url := fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, name)
 			responseStr = append(responseStr, url)
 
 		} else {
@@ -1370,7 +1379,7 @@ func aliasPost(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/images/aliases/%s", version.APIVersion, req.Name))
 }
 
 func imageExport(d *Daemon, r *http.Request) Response {
