@@ -1,28 +1,22 @@
 package main
 
 import (
-	"fmt"
-	"os"
+	"sync"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
 )
 
 func cmdActivateIfNeeded() error {
-	// Only root should run this
-	if os.Geteuid() != 0 {
-		return fmt.Errorf("This must be run as root")
+	// Don't start a full daemon, we just need DB access
+	d := &Daemon{
+		imagesDownloading:     map[string]chan bool{},
+		imagesDownloadingLock: sync.RWMutex{},
+		lxcpath:               shared.VarPath("containers"),
 	}
 
-	// Don't start a full daemon, we just need DB access
-	d := NewDaemon()
-	d.os.LxcPath = shared.VarPath("containers")
-
 	if !shared.PathExists(shared.VarPath("lxd.db")) {
-		logger.Debugf("No DB, so no need to start the daemon now.")
+		shared.LogDebugf("No DB, so no need to start the daemon now.")
 		return nil
 	}
 
@@ -40,25 +34,24 @@ func cmdActivateIfNeeded() error {
 	// Look for network socket
 	value := daemonConfig["core.https_address"].Get()
 	if value != "" {
-		logger.Debugf("Daemon has core.https_address set, activating...")
-		_, err := lxd.ConnectLXDUnix("", nil)
-		return err
-	}
-
-	// Load the idmap for unprivileged containers
-	d.os.IdmapSet, err = idmap.DefaultIdmapSet()
-	if err != nil {
+		shared.LogDebugf("Daemon has core.https_address set, activating...")
+		_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
 		return err
 	}
 
 	// Look for auto-started or previously started containers
-	result, err := db.ContainersList(d.db, db.CTypeRegular)
+	d.IdmapSet, err = shared.DefaultIdmapSet()
+	if err != nil {
+		return err
+	}
+
+	result, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
 	}
 
 	for _, name := range result {
-		c, err := containerLoadByName(d.State(), d.Storage, name)
+		c, err := containerLoadByName(d, name)
 		if err != nil {
 			return err
 		}
@@ -68,18 +61,18 @@ func cmdActivateIfNeeded() error {
 		autoStart := config["boot.autostart"]
 
 		if c.IsRunning() {
-			logger.Debugf("Daemon has running containers, activating...")
-			_, err := lxd.ConnectLXDUnix("", nil)
+			shared.LogDebugf("Daemon has running containers, activating...")
+			_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
 			return err
 		}
 
 		if lastState == "RUNNING" || lastState == "Running" || shared.IsTrue(autoStart) {
-			logger.Debugf("Daemon has auto-started containers, activating...")
-			_, err := lxd.ConnectLXDUnix("", nil)
+			shared.LogDebugf("Daemon has auto-started containers, activating...")
+			_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
 			return err
 		}
 	}
 
-	logger.Debugf("No need to start the daemon now.")
+	shared.LogDebugf("No need to start the daemon now.")
 	return nil
 }
