@@ -2,31 +2,35 @@ package main
 
 import (
 	"fmt"
+	"net/http"
 	"os"
 	"time"
 
-	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd"
 )
 
-func cmdCallHook(args *Args) error {
-	// Parse the arguments
-	if len(args.Params) < 3 {
+func cmdCallHook(args []string) error {
+	if len(args) < 4 {
 		return fmt.Errorf("Invalid arguments")
 	}
 
-	path := args.Params[0]
-	id := args.Params[1]
-	state := args.Params[2]
+	path := args[1]
+	id := args[2]
+	state := args[3]
 	target := ""
 
-	// Connect to LXD
-	c, err := lxd.ConnectLXDUnix(fmt.Sprintf("%s/unix.socket", path), nil)
+	err := os.Setenv("LXD_DIR", path)
 	if err != nil {
 		return err
 	}
 
-	// Prepare the request URL
-	url := fmt.Sprintf("/internal/containers/%s/on%s", id, state)
+	c, err := lxd.NewClient(&lxd.DefaultConfig, "local")
+	if err != nil {
+		return err
+	}
+
+	url := fmt.Sprintf("%s/internal/containers/%s/on%s", c.BaseURL, id, state)
+
 	if state == "stop" {
 		target = os.Getenv("LXC_TARGET")
 		if target == "" {
@@ -35,10 +39,20 @@ func cmdCallHook(args *Args) error {
 		url = fmt.Sprintf("%s?target=%s", url, target)
 	}
 
-	// Setup the request
+	req, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return err
+	}
+
 	hook := make(chan error, 1)
 	go func() {
-		_, _, err := c.RawQuery("GET", url, nil, "")
+		raw, err := c.Http.Do(req)
+		if err != nil {
+			hook <- err
+			return
+		}
+
+		_, err = lxd.HoistResponse(raw, lxd.Sync)
 		if err != nil {
 			hook <- err
 			return
@@ -47,7 +61,6 @@ func cmdCallHook(args *Args) error {
 		hook <- nil
 	}()
 
-	// Handle the timeout
 	select {
 	case err := <-hook:
 		if err != nil {
