@@ -15,17 +15,14 @@ import (
 /*
  * This is called by lxd when called as "lxd forkexec <container>"
  */
-func cmdForkExec(args *Args) (int, error) {
-	if len(args.Params) < 3 {
-		return -1, fmt.Errorf("Bad params: %q", args.Params)
-	}
-	if len(args.Extra) < 1 {
-		return -1, fmt.Errorf("Bad extra: %q", args.Extra)
+func cmdForkExec(args []string) (int, error) {
+	if len(args) < 6 {
+		return -1, fmt.Errorf("Bad arguments: %q", args)
 	}
 
-	name := args.Params[0]
-	lxcpath := args.Params[1]
-	configPath := args.Params[2]
+	name := args[1]
+	lxcpath := args[2]
+	configPath := args[3]
 
 	c, err := lxc.NewContainer(name, lxcpath)
 	if err != nil {
@@ -66,7 +63,7 @@ func cmdForkExec(args *Args) (int, error) {
 	cmd := []string{}
 
 	section := ""
-	for _, arg := range args.Extra {
+	for _, arg := range args[5:len(args)] {
 		// The "cmd" section must come last as it may contain a --
 		if arg == "--" && section != "cmd" {
 			section = ""
@@ -106,19 +103,32 @@ func cmdForkExec(args *Args) (int, error) {
 		return -1, fmt.Errorf("Failed sending PID of executing command: %q", err)
 	}
 
-	var ws syscall.WaitStatus
-	wpid, err := syscall.Wait4(status, &ws, 0, nil)
-	if err != nil || wpid != status {
+	proc, err := os.FindProcess(status)
+	if err != nil {
 		return -1, fmt.Errorf("Failed finding process: %q", err)
 	}
 
-	if ws.Exited() {
-		return ws.ExitStatus(), nil
+	procState, err := proc.Wait()
+	if err != nil {
+		return -1, fmt.Errorf("Failed waiting on process %d: %q", status, err)
 	}
 
-	if ws.Signaled() {
-		// 128 + n == Fatal error signal "n"
-		return 128 + int(ws.Signal()), nil
+	if procState.Success() {
+		return 0, nil
+	}
+
+	exCode, ok := procState.Sys().(syscall.WaitStatus)
+	if ok {
+		if exCode.Exited() {
+			return exCode.ExitStatus(), nil
+		}
+		// Backwards compatible behavior. Report success when we exited
+		// due to a signal. Otherwise this may break Jenkins, e.g. when
+		// lxc exec foo reboot receives SIGTERM and exCode.Exitstats()
+		// would report -1.
+		if exCode.Signaled() {
+			return 0, nil
+		}
 	}
 
 	return -1, fmt.Errorf("Command failed")
