@@ -11,7 +11,6 @@ import (
 	"net/http"
 	"net/url"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"strings"
 	"sync"
@@ -20,9 +19,8 @@ import (
 	"github.com/gorilla/websocket"
 	"gopkg.in/lxc/go-lxc.v2"
 
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
 )
 
 type migrationFields struct {
@@ -137,7 +135,7 @@ func (c *migrationFields) controlChannel() <-chan MigrationControl {
 		msg := MigrationControl{}
 		err := c.recv(&msg)
 		if err != nil {
-			logger.Debugf("Got error reading migration control socket %s", err)
+			shared.LogDebugf("Got error reading migration control socket %s", err)
 			close(ch)
 			return
 		}
@@ -168,9 +166,8 @@ func NewMigrationSource(c container) (*migrationSourceWs, error) {
 	}
 
 	if c.IsRunning() {
-		_, err := exec.LookPath("criu")
-		if err != nil {
-			return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the source server.")
+		if err := findCriu("source"); err != nil {
+			return nil, err
 		}
 
 		ret.live = true
@@ -212,9 +209,8 @@ func (s *migrationSourceWs) Connect(op *operation, r *http.Request, w http.Respo
 	case s.fsSecret:
 		conn = &s.fsConn
 	default:
-		// If we didn't find the right secret, the user provided a bad
-		// one, which 403, not 404, since this operation actually
-		// exists.
+		/* If we didn't find the right secret, the user provided a bad one,
+		 * which 403, not 404, since this operation actually exists */
 		return os.ErrPermission
 	}
 
@@ -245,8 +241,7 @@ fi
 	}
 	defer f.Close()
 
-	err = f.Chmod(0500)
-	if err != nil {
+	if err := f.Chmod(0500); err != nil {
 		return err
 	}
 
@@ -278,7 +273,6 @@ func snapshotToProtobuf(c container) *Snapshot {
 	isEphemeral := c.IsEphemeral()
 	arch := int32(c.Architecture())
 	stateful := c.IsStateful()
-
 	return &Snapshot{
 		Name:         &parts[len(parts)-1],
 		LocalConfig:  config,
@@ -307,19 +301,15 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 
 	idmaps := make([]*IDMapType, 0)
 
-	idmapset, err := s.container.IdmapSet()
-	if err != nil {
-		return err
-	}
-
+	idmapset := s.container.IdmapSet()
 	if idmapset != nil {
 		for _, ctnIdmap := range idmapset.Idmap {
 			idmap := IDMapType{
 				Isuid:    proto.Bool(ctnIdmap.Isuid),
 				Isgid:    proto.Bool(ctnIdmap.Isgid),
-				Hostid:   proto.Int32(int32(ctnIdmap.Hostid)),
-				Nsid:     proto.Int32(int32(ctnIdmap.Nsid)),
-				Maprange: proto.Int32(int32(ctnIdmap.Maprange)),
+				Hostid:   proto.Int(ctnIdmap.Hostid),
+				Nsid:     proto.Int(ctnIdmap.Nsid),
+				Maprange: proto.Int(ctnIdmap.Maprange),
 			}
 
 			idmaps = append(idmaps, &idmap)
@@ -340,8 +330,6 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		}
 	}
 
-	// The protocol says we have to send a header no matter what, so let's
-	// do that, but then immediately send an error.
 	myType := s.container.Storage().MigrationType()
 	header := MigrationHeader{
 		Fs:            &myType,
@@ -351,8 +339,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		Snapshots:     snapshots,
 	}
 
-	err = s.send(&header)
-	if err != nil {
+	if err := s.send(&header); err != nil {
 		s.sendControl(err)
 		return err
 	}
@@ -362,8 +349,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return fsErr
 	}
 
-	err = s.recv(&header)
-	if err != nil {
+	if err := s.recv(&header); err != nil {
 		s.sendControl(err)
 		return err
 	}
@@ -388,8 +374,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return err
 	}
 
-	err = driver.SendWhileRunning(s.fsConn)
-	if err != nil {
+	if err := driver.SendWhileRunning(s.fsConn, migrateOp); err != nil {
 		return abort(err)
 	}
 
@@ -469,8 +454,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				return abort(err)
 			}
 
-			err = writeActionScript(checkpointDir, actionScriptOp.url, actionScriptOpSecret)
-			if err != nil {
+			if err := writeActionScript(checkpointDir, actionScriptOp.url, actionScriptOpSecret); err != nil {
 				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
@@ -492,12 +476,11 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				return abort(err)
 			/* the dump finished, let's continue on to the restore */
 			case <-dumpDone:
-				logger.Debugf("Dump finished, continuing with restore...")
+				shared.LogDebugf("Dump finished, continuing with restore...")
 			}
 		} else {
 			defer os.RemoveAll(checkpointDir)
-			err = s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, false)
-			if err != nil {
+			if err := s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, false); err != nil {
 				return abort(err)
 			}
 		}
@@ -509,14 +492,11 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		 * no reason to do these in parallel. In the future when we're using
 		 * p.haul's protocol, it will make sense to do these in parallel.
 		 */
-		ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
-		err = RsyncSend(ctName, shared.AddSlash(checkpointDir), s.criuConn)
-		if err != nil {
+		if err := RsyncSend(shared.AddSlash(checkpointDir), s.criuConn, nil); err != nil {
 			return abort(err)
 		}
 
-		err = driver.SendAfterCheckpoint(s.fsConn)
-		if err != nil {
+		if err := driver.SendAfterCheckpoint(s.fsConn); err != nil {
 			return abort(err)
 		}
 	}
@@ -524,8 +504,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 	driver.Cleanup()
 
 	msg := MigrationControl{}
-	err = s.recv(&msg)
-	if err != nil {
+	if err := s.recv(&msg); err != nil {
 		s.disconnect()
 		return err
 	}
@@ -534,7 +513,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		restoreSuccess <- *msg.Success
 		err := <-dumpSuccess
 		if err != nil {
-			logger.Errorf("dump failed after successful restore?: %q", err)
+			shared.LogErrorf("dump failed after successful restore?: %q", err)
 		}
 	}
 
@@ -548,9 +527,15 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 type migrationSink struct {
 	// We are pulling the container from src in pull mode.
 	src migrationFields
+	// The container is pushed from src to dest in push mode. Note that
+	// websocket connections are not set in push mode. Only the secret
+	// fields are used since the client will connect to the sockets.
+	dest migrationFields
 
-	url    string
-	dialer websocket.Dialer
+	url          string
+	dialer       websocket.Dialer
+	allConnected chan bool
+	push         bool
 }
 
 type MigrationSinkArgs struct {
@@ -558,6 +543,8 @@ type MigrationSinkArgs struct {
 	Dialer    websocket.Dialer
 	Container container
 	Secrets   map[string]string
+	Push      bool
+	Live      bool
 }
 
 func NewMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
@@ -565,25 +552,53 @@ func NewMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
 		src:    migrationFields{container: args.Container},
 		url:    args.Url,
 		dialer: args.Dialer,
+		push:   args.Push,
+	}
+
+	if sink.push {
+		sink.allConnected = make(chan bool, 1)
 	}
 
 	var ok bool
-	sink.src.controlSecret, ok = args.Secrets["control"]
-	if !ok {
-		return nil, fmt.Errorf("Missing control secret")
+	var err error
+	if sink.push {
+		sink.dest.controlSecret, err = shared.RandomCryptoString()
+		if err != nil {
+			return nil, err
+		}
+
+		sink.dest.fsSecret, err = shared.RandomCryptoString()
+		if err != nil {
+			return nil, err
+		}
+
+		sink.dest.live = args.Live
+		if sink.dest.live {
+			sink.dest.criuSecret, err = shared.RandomCryptoString()
+			if err != nil {
+				return nil, err
+			}
+		}
+	} else {
+		sink.src.controlSecret, ok = args.Secrets["control"]
+		if !ok {
+			return nil, fmt.Errorf("Missing control secret")
+		}
+
+		sink.src.fsSecret, ok = args.Secrets["fs"]
+		if !ok {
+			return nil, fmt.Errorf("Missing fs secret")
+		}
+
+		sink.src.criuSecret, ok = args.Secrets["criu"]
+		sink.src.live = ok
 	}
 
-	sink.src.fsSecret, ok = args.Secrets["fs"]
-	if !ok {
-		return nil, fmt.Errorf("Missing fs secret")
-	}
-
-	sink.src.criuSecret, ok = args.Secrets["criu"]
-	sink.src.live = ok
-
-	_, err := exec.LookPath("criu")
-	if sink.src.live && err != nil {
-		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
+	err = findCriu("destination")
+	if sink.push && sink.dest.live && err != nil {
+		return nil, err
+	} else if sink.src.live && err != nil {
+		return nil, err
 	}
 
 	return &sink, nil
@@ -595,49 +610,121 @@ func (c *migrationSink) connectWithSecret(secret string) (*websocket.Conn, error
 	// The URL is a https URL to the operation, mangle to be a wss URL to the secret
 	wsUrl := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(c.url, "https://"), query.Encode())
 
-	conn, _, err := c.dialer.Dial(wsUrl, http.Header{})
-	if err != nil {
-		return nil, err
+	return lxd.WebsocketDial(c.dialer, wsUrl)
+}
+
+func (s *migrationSink) Metadata() interface{} {
+	secrets := shared.Jmap{
+		"control": s.dest.controlSecret,
+		"fs":      s.dest.fsSecret,
 	}
 
-	return conn, err
+	if s.dest.criuSecret != "" {
+		secrets["criu"] = s.dest.criuSecret
+	}
+
+	return secrets
+}
+
+func (s *migrationSink) Connect(op *operation, r *http.Request, w http.ResponseWriter) error {
+	secret := r.FormValue("secret")
+	if secret == "" {
+		return fmt.Errorf("missing secret")
+	}
+
+	var conn **websocket.Conn
+
+	switch secret {
+	case s.dest.controlSecret:
+		conn = &s.dest.controlConn
+	case s.dest.criuSecret:
+		conn = &s.dest.criuConn
+	case s.dest.fsSecret:
+		conn = &s.dest.fsConn
+	default:
+		/* If we didn't find the right secret, the user provided a bad one,
+		 * which 403, not 404, since this operation actually exists */
+		return os.ErrPermission
+	}
+
+	c, err := shared.WebsocketUpgrader.Upgrade(w, r, nil)
+	if err != nil {
+		return err
+	}
+
+	*conn = c
+
+	if s.dest.controlConn != nil && (!s.dest.live || s.dest.criuConn != nil) && s.dest.fsConn != nil {
+		s.allConnected <- true
+	}
+
+	return nil
 }
 
 func (c *migrationSink) Do(migrateOp *operation) error {
 	var err error
 
-	c.src.controlConn, err = c.connectWithSecret(c.src.controlSecret)
-	if err != nil {
-		c.src.container.Delete()
-		return err
-	}
-	defer c.src.disconnect()
-
-	c.src.fsConn, err = c.connectWithSecret(c.src.fsSecret)
-	if err != nil {
-		c.src.container.Delete()
-		c.src.sendControl(err)
-		return err
+	if c.push {
+		<-c.allConnected
 	}
 
-	if c.src.live {
-		c.src.criuConn, err = c.connectWithSecret(c.src.criuSecret)
+	disconnector := c.src.disconnect
+	if c.push {
+		disconnector = c.dest.disconnect
+	}
+
+	if c.push {
+		defer disconnector()
+	} else {
+		c.src.controlConn, err = c.connectWithSecret(c.src.controlSecret)
+		if err != nil {
+			return err
+		}
+		defer c.src.disconnect()
+
+		c.src.fsConn, err = c.connectWithSecret(c.src.fsSecret)
 		if err != nil {
-			c.src.container.Delete()
 			c.src.sendControl(err)
 			return err
 		}
+
+		if c.src.live {
+			c.src.criuConn, err = c.connectWithSecret(c.src.criuSecret)
+			if err != nil {
+				c.src.sendControl(err)
+				return err
+			}
+		}
+	}
+
+	receiver := c.src.recv
+	if c.push {
+		receiver = c.dest.recv
+	}
+
+	sender := c.src.send
+	if c.push {
+		sender = c.dest.send
+	}
+
+	controller := c.src.sendControl
+	if c.push {
+		controller = c.dest.sendControl
 	}
 
 	header := MigrationHeader{}
-	if err := c.src.recv(&header); err != nil {
-		c.src.container.Delete()
-		c.src.sendControl(err)
+	if err := receiver(&header); err != nil {
+		controller(err)
 		return err
 	}
 
+	live := c.src.live
+	if c.push {
+		live = c.dest.live
+	}
+
 	criuType := CRIUType_CRIU_RSYNC.Enum()
-	if !c.src.live {
+	if !live {
 		criuType = nil
 	}
 
@@ -656,26 +743,24 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 		resp.Fs = &myType
 	}
 
-	err = c.src.send(&resp)
-	if err != nil {
-		c.src.container.Delete()
-		c.src.sendControl(err)
+	if err := sender(&resp); err != nil {
+		controller(err)
 		return err
 	}
 
 	restore := make(chan error)
 	go func(c *migrationSink) {
 		imagesDir := ""
-		srcIdmap := new(idmap.IdmapSet)
+		srcIdmap := new(shared.IdmapSet)
 
-		for _, idmapSet := range header.Idmap {
-			e := idmap.IdmapEntry{
-				Isuid:    *idmapSet.Isuid,
-				Isgid:    *idmapSet.Isgid,
-				Nsid:     int64(*idmapSet.Nsid),
-				Hostid:   int64(*idmapSet.Hostid),
-				Maprange: int64(*idmapSet.Maprange)}
-			srcIdmap.Idmap = idmap.Extend(srcIdmap.Idmap, e)
+		for _, idmap := range header.Idmap {
+			e := shared.IdmapEntry{
+				Isuid:    *idmap.Isuid,
+				Isgid:    *idmap.Isgid,
+				Nsid:     int(*idmap.Nsid),
+				Hostid:   int(*idmap.Hostid),
+				Maprange: int(*idmap.Maprange)}
+			srcIdmap.Idmap = shared.Extend(srcIdmap.Idmap, e)
 		}
 
 		/* We do the fs receive in parallel so we don't have to reason
@@ -702,14 +787,18 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 				snapshots = header.Snapshots
 			}
 
-			err := mySink(c.src.live, c.src.container, snapshots, c.src.fsConn, srcIdmap)
-			if err != nil {
+			var fsConn *websocket.Conn
+			if c.push {
+				fsConn = c.dest.fsConn
+			} else {
+				fsConn = c.src.fsConn
+			}
+			if err := mySink(live, c.src.container, header.Snapshots, fsConn, srcIdmap, migrateOp); err != nil {
 				fsTransfer <- err
 				return
 			}
 
-			err = ShiftIfNecessary(c.src.container, srcIdmap)
-			if err != nil {
+			if err := ShiftIfNecessary(c.src.container, srcIdmap); err != nil {
 				fsTransfer <- err
 				return
 			}
@@ -717,7 +806,7 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			fsTransfer <- nil
 		}()
 
-		if c.src.live {
+		if live {
 			var err error
 			imagesDir, err = ioutil.TempDir("", "lxd_restore_")
 			if err != nil {
@@ -727,8 +816,13 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 
 			defer os.RemoveAll(imagesDir)
 
-			err = RsyncRecv(shared.AddSlash(imagesDir), c.src.criuConn)
-			if err != nil {
+			var criuConn *websocket.Conn
+			if c.push {
+				criuConn = c.dest.criuConn
+			} else {
+				criuConn = c.src.criuConn
+			}
+			if err := RsyncRecv(shared.AddSlash(imagesDir), criuConn, nil); err != nil {
 				restore <- err
 				return
 			}
@@ -740,7 +834,7 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			return
 		}
 
-		if c.src.live {
+		if live {
 			err = c.src.container.Migrate(lxc.MIGRATE_RESTORE, imagesDir, "migration", false, false)
 			if err != nil {
 				restore <- err
@@ -752,37 +846,31 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 		restore <- nil
 	}(c)
 
-	source := c.src.controlChannel()
+	var source <-chan MigrationControl
+	if c.push {
+		source = c.dest.controlChannel()
+	} else {
+		source = c.src.controlChannel()
+	}
 
 	for {
 		select {
 		case err = <-restore:
-			c.src.sendControl(err)
-			if err != nil {
-				c.src.container.Delete()
-				return err
-			}
-			return nil
+			controller(err)
+			return err
 		case msg, ok := <-source:
 			if !ok {
-				c.src.disconnect()
-				c.src.container.Delete()
+				disconnector()
 				return fmt.Errorf("Got error reading source")
 			}
 			if !*msg.Success {
-				c.src.disconnect()
-				c.src.container.Delete()
+				disconnector()
 				return fmt.Errorf(*msg.Message)
 			} else {
 				// The source can only tell us it failed (e.g. if
 				// checkpointing failed). We have to tell the source
 				// whether or not the restore was successful.
-				logger.Debugf("Unknown message %v from source", msg)
-				err = c.src.container.TemplateApply("copy")
-				if err != nil {
-					c.src.container.Delete()
-					return err
-				}
+				shared.LogDebugf("Unknown message %v from source", msg)
 			}
 		}
 	}
