@@ -11,11 +11,7 @@ import (
 	"github.com/gorilla/mux"
 	"github.com/pborman/uuid"
 
-	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/cancel"
-	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/version"
 )
 
@@ -43,13 +39,12 @@ type operation struct {
 	class     operationClass
 	createdAt time.Time
 	updatedAt time.Time
-	status    api.StatusCode
+	status    shared.StatusCode
 	url       string
 	resources map[string][]string
 	metadata  map[string]interface{}
 	err       string
 	readonly  bool
-	canceler  *cancel.Canceler
 
 	// Those functions are called at various points in the operation lifecycle
 	onRun     func(*operation) error
@@ -102,27 +97,27 @@ func (op *operation) done() {
 }
 
 func (op *operation) Run() (chan error, error) {
-	if op.status != api.Pending {
+	if op.status != shared.Pending {
 		return nil, fmt.Errorf("Only pending operations can be started")
 	}
 
 	chanRun := make(chan error, 1)
 
 	op.lock.Lock()
-	op.status = api.Running
+	op.status = shared.Running
 
 	if op.onRun != nil {
 		go func(op *operation, chanRun chan error) {
 			err := op.onRun(op)
 			if err != nil {
 				op.lock.Lock()
-				op.status = api.Failure
+				op.status = shared.Failure
 				op.err = SmartError(err).String()
 				op.lock.Unlock()
 				op.done()
 				chanRun <- err
 
-				logger.Debugf("Failure for %s operation: %s: %s", op.class.String(), op.id, err)
+				shared.LogDebugf("Failure for %s operation: %s: %s", op.class.String(), op.id, err)
 
 				_, md, _ := op.Render()
 				eventSend("operation", md)
@@ -130,13 +125,13 @@ func (op *operation) Run() (chan error, error) {
 			}
 
 			op.lock.Lock()
-			op.status = api.Success
+			op.status = shared.Success
 			op.lock.Unlock()
 			op.done()
 			chanRun <- nil
 
 			op.lock.Lock()
-			logger.Debugf("Success for %s operation: %s", op.class.String(), op.id)
+			shared.LogDebugf("Success for %s operation: %s", op.class.String(), op.id)
 			_, md, _ := op.Render()
 			eventSend("operation", md)
 			op.lock.Unlock()
@@ -144,7 +139,7 @@ func (op *operation) Run() (chan error, error) {
 	}
 	op.lock.Unlock()
 
-	logger.Debugf("Started %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Started %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -152,7 +147,7 @@ func (op *operation) Run() (chan error, error) {
 }
 
 func (op *operation) Cancel() (chan error, error) {
-	if op.status != api.Running {
+	if op.status != shared.Running {
 		return nil, fmt.Errorf("Only running operations can be cancelled")
 	}
 
@@ -164,11 +159,11 @@ func (op *operation) Cancel() (chan error, error) {
 
 	op.lock.Lock()
 	oldStatus := op.status
-	op.status = api.Cancelling
+	op.status = shared.Cancelling
 	op.lock.Unlock()
 
 	if op.onCancel != nil {
-		go func(op *operation, oldStatus api.StatusCode, chanCancel chan error) {
+		go func(op *operation, oldStatus shared.StatusCode, chanCancel chan error) {
 			err := op.onCancel(op)
 			if err != nil {
 				op.lock.Lock()
@@ -176,44 +171,37 @@ func (op *operation) Cancel() (chan error, error) {
 				op.lock.Unlock()
 				chanCancel <- err
 
-				logger.Debugf("Failed to cancel %s operation: %s: %s", op.class.String(), op.id, err)
+				shared.LogDebugf("Failed to cancel %s operation: %s: %s", op.class.String(), op.id, err)
 				_, md, _ := op.Render()
 				eventSend("operation", md)
 				return
 			}
 
 			op.lock.Lock()
-			op.status = api.Cancelled
+			op.status = shared.Cancelled
 			op.lock.Unlock()
 			op.done()
 			chanCancel <- nil
 
-			logger.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
+			shared.LogDebugf("Cancelled %s operation: %s", op.class.String(), op.id)
 			_, md, _ := op.Render()
 			eventSend("operation", md)
 		}(op, oldStatus, chanCancel)
 	}
 
-	logger.Debugf("Cancelling %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Cancelling %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
-	if op.canceler != nil {
-		err := op.canceler.Cancel()
-		if err != nil {
-			return nil, err
-		}
-	}
-
 	if op.onCancel == nil {
 		op.lock.Lock()
-		op.status = api.Cancelled
+		op.status = shared.Cancelled
 		op.lock.Unlock()
 		op.done()
 		chanCancel <- nil
 	}
 
-	logger.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Cancelled %s operation: %s", op.class.String(), op.id)
 	_, md, _ = op.Render()
 	eventSend("operation", md)
 
@@ -225,7 +213,7 @@ func (op *operation) Connect(r *http.Request, w http.ResponseWriter) (chan error
 		return nil, fmt.Errorf("Only websocket operations can be connected")
 	}
 
-	if op.status != api.Running {
+	if op.status != shared.Running {
 		return nil, fmt.Errorf("Only running operations can be connected")
 	}
 
@@ -238,38 +226,26 @@ func (op *operation) Connect(r *http.Request, w http.ResponseWriter) (chan error
 		if err != nil {
 			chanConnect <- err
 
-			logger.Debugf("Failed to handle %s operation: %s: %s", op.class.String(), op.id, err)
+			shared.LogDebugf("Failed to handle %s operation: %s: %s", op.class.String(), op.id, err)
 			return
 		}
 
 		chanConnect <- nil
 
-		logger.Debugf("Handled %s operation: %s", op.class.String(), op.id)
+		shared.LogDebugf("Handled %s operation: %s", op.class.String(), op.id)
 	}(op, chanConnect)
 	op.lock.Unlock()
 
-	logger.Debugf("Connected %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Connected %s operation: %s", op.class.String(), op.id)
 
 	return chanConnect, nil
 }
 
 func (op *operation) mayCancel() bool {
-	if op.class == operationClassToken {
-		return true
-	}
-
-	if op.onCancel != nil {
-		return true
-	}
-
-	if op.canceler != nil && op.canceler.Cancelable() {
-		return true
-	}
-
-	return false
+	return op.onCancel != nil || op.class == operationClassToken
 }
 
-func (op *operation) Render() (string, *api.Operation, error) {
+func (op *operation) Render() (string, *shared.Operation, error) {
 	// Setup the resource URLs
 	resources := op.resources
 	if resources != nil {
@@ -284,15 +260,17 @@ func (op *operation) Render() (string, *api.Operation, error) {
 		resources = tmpResources
 	}
 
-	return op.url, &api.Operation{
-		ID:         op.id,
+	md := shared.Jmap(op.metadata)
+
+	return op.url, &shared.Operation{
+		Id:         op.id,
 		Class:      op.class.String(),
 		CreatedAt:  op.createdAt,
 		UpdatedAt:  op.updatedAt,
 		Status:     op.status.String(),
 		StatusCode: op.status,
 		Resources:  resources,
-		Metadata:   op.metadata,
+		Metadata:   &md,
 		MayCancel:  op.mayCancel(),
 		Err:        op.err,
 	}, nil
@@ -306,19 +284,23 @@ func (op *operation) WaitFinal(timeout int) (bool, error) {
 
 	// Wait indefinitely
 	if timeout == -1 {
-		<-op.chanDone
-		return true, nil
+		for {
+			<-op.chanDone
+			return true, nil
+		}
 	}
 
 	// Wait until timeout
 	if timeout > 0 {
 		timer := time.NewTimer(time.Duration(timeout) * time.Second)
-		select {
-		case <-op.chanDone:
-			return false, nil
+		for {
+			select {
+			case <-op.chanDone:
+				return false, nil
 
-		case <-timer.C:
-			return false, nil
+			case <-timer.C:
+				return false, nil
+			}
 		}
 	}
 
@@ -326,7 +308,7 @@ func (op *operation) WaitFinal(timeout int) (bool, error) {
 }
 
 func (op *operation) UpdateResources(opResources map[string][]string) error {
-	if op.status != api.Pending && op.status != api.Running {
+	if op.status != shared.Pending && op.status != shared.Running {
 		return fmt.Errorf("Only pending or running operations can be updated")
 	}
 
@@ -339,7 +321,7 @@ func (op *operation) UpdateResources(opResources map[string][]string) error {
 	op.resources = opResources
 	op.lock.Unlock()
 
-	logger.Debugf("Updated resources for %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Updated resources for %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -347,7 +329,7 @@ func (op *operation) UpdateResources(opResources map[string][]string) error {
 }
 
 func (op *operation) UpdateMetadata(opMetadata interface{}) error {
-	if op.status != api.Pending && op.status != api.Running {
+	if op.status != shared.Pending && op.status != shared.Running {
 		return fmt.Errorf("Only pending or running operations can be updated")
 	}
 
@@ -365,7 +347,7 @@ func (op *operation) UpdateMetadata(opMetadata interface{}) error {
 	op.metadata = newMetadata
 	op.lock.Unlock()
 
-	logger.Debugf("Updated metadata for %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("Updated metadata for %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -383,7 +365,7 @@ func operationCreate(opClass operationClass, opResources map[string][]string, op
 	op.class = opClass
 	op.createdAt = time.Now()
 	op.updatedAt = op.createdAt
-	op.status = api.Pending
+	op.status = shared.Pending
 	op.url = fmt.Sprintf("/%s/operations/%s", version.APIVersion, op.id)
 	op.resources = opResources
 	op.chanDone = make(chan error)
@@ -420,7 +402,7 @@ func operationCreate(opClass operationClass, opResources map[string][]string, op
 	operations[op.id] = &op
 	operationsLock.Unlock()
 
-	logger.Debugf("New %s operation: %s", op.class.String(), op.id)
+	shared.LogDebugf("New %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -450,7 +432,7 @@ func operationAPIGet(d *Daemon, r *http.Request) Response {
 
 	_, body, err := op.Render()
 	if err != nil {
-		return SmartError(err)
+		return InternalError(err)
 	}
 
 	return SyncResponse(true, body)
@@ -477,7 +459,7 @@ var operationCmd = Command{name: "operations/{id}", get: operationAPIGet, delete
 func operationsAPIGet(d *Daemon, r *http.Request) Response {
 	var md shared.Jmap
 
-	recursion := util.IsRecursionRequest(r)
+	recursion := d.isRecursionRequest(r)
 
 	md = shared.Jmap{}
 
@@ -490,7 +472,7 @@ func operationsAPIGet(d *Daemon, r *http.Request) Response {
 		_, ok := md[status]
 		if !ok {
 			if recursion {
-				md[status] = make([]*api.Operation, 0)
+				md[status] = make([]*shared.Operation, 0)
 			} else {
 				md[status] = make([]string, 0)
 			}
@@ -506,7 +488,7 @@ func operationsAPIGet(d *Daemon, r *http.Request) Response {
 			continue
 		}
 
-		md[status] = append(md[status].([]*api.Operation), body)
+		md[status] = append(md[status].([]*shared.Operation), body)
 	}
 
 	return SyncResponse(true, md)
@@ -533,7 +515,7 @@ func operationAPIWaitGet(d *Daemon, r *http.Request) Response {
 
 	_, body, err := op.Render()
 	if err != nil {
-		return SmartError(err)
+		return InternalError(err)
 	}
 
 	return SyncResponse(true, body)
@@ -562,7 +544,7 @@ func (r *operationWebSocket) String() string {
 		return fmt.Sprintf("error: %s", err)
 	}
 
-	return md.ID
+	return md.Id
 }
 
 func operationAPIWebsocketGet(d *Daemon, r *http.Request) Response {
