@@ -3,27 +3,63 @@ package main
 import (
 	"encoding/json"
 	"fmt"
+	"io"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"reflect"
 	"syscall"
+	"time"
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/types"
-	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/ioprogress"
 	"github.com/lxc/lxd/shared/logging"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
+/* Some interesting filesystems */
+const (
+	filesystemSuperMagicTmpfs = 0x01021994
+	filesystemSuperMagicExt4  = 0xEF53
+	filesystemSuperMagicXfs   = 0x58465342
+	filesystemSuperMagicNfs   = 0x6969
+	filesystemSuperMagicZfs   = 0x2fc12fc1
+)
+
+/*
+ * filesystemDetect returns the filesystem on which
+ * the passed-in path sits
+ */
+func filesystemDetect(path string) (string, error) {
+	fs := syscall.Statfs_t{}
+
+	err := syscall.Statfs(path, &fs)
+	if err != nil {
+		return "", err
+	}
+
+	switch fs.Type {
+	case filesystemSuperMagicBtrfs:
+		return "btrfs", nil
+	case filesystemSuperMagicZfs:
+		return "zfs", nil
+	case filesystemSuperMagicTmpfs:
+		return "tmpfs", nil
+	case filesystemSuperMagicExt4:
+		return "ext4", nil
+	case filesystemSuperMagicXfs:
+		return "xfs", nil
+	case filesystemSuperMagicNfs:
+		return "nfs", nil
+	default:
+		shared.LogDebugf("Unknown backing filesystem type: 0x%x", fs.Type)
+		return string(fs.Type), nil
+	}
+}
+
 // storageRsyncCopy copies a directory using rsync (with the --devices option).
 func storageRsyncCopy(source string, dest string) (string, error) {
 	if err := os.MkdirAll(dest, 0755); err != nil {
@@ -35,33 +71,19 @@ func storageRsyncCopy(source string, dest string) (string, error) {
 		rsyncVerbosity = "-vi"
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"rsync",
 		"-a",
 		"-HAX",
-		"--sparse",
 		"--devices",
 		"--delete",
 		"--checksum",
 		"--numeric-ids",
 		rsyncVerbosity,
 		shared.AddSlash(source),
-		dest)
-	if err != nil {
-		runError, ok := err.(shared.RunError)
-		if ok {
-			exitError, ok := runError.Err.(*exec.ExitError)
-			if ok {
-				waitStatus := exitError.Sys().(syscall.WaitStatus)
-				if waitStatus.ExitStatus() == 24 {
-					return string(output), nil
-				}
-			}
-		}
-		return string(output), err
-	}
+		dest).CombinedOutput()
 
-	return string(output), nil
+	return string(output), err
 }
 
 // storageType defines the type of a storage
@@ -97,7 +119,7 @@ type MigrationStorageSourceDriver interface {
 	/* send any bits of the container/snapshots that are possible while the
 	 * container is still running.
 	 */
-	SendWhileRunning(conn *websocket.Conn) error
+	SendWhileRunning(conn *websocket.Conn, op *operation) error
 
 	/* send the final bits (e.g. a final delta snapshot for zfs, btrfs, or
 	 * do a final rsync) of the fs after the container has been
@@ -172,78 +194,67 @@ type storage interface {
 	// already present on the target instance as an exercise for the
 	// enterprising developer.
 	MigrationSource(container container) (MigrationStorageSourceDriver, error)
-	MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error
+	MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error
 }
 
 func newStorage(d *Daemon, sType storageType) (storage, error) {
 	var nilmap map[string]interface{}
-	return newStorageWithConfig(d.State(), d.Storage, sType, nilmap)
+	return newStorageWithConfig(d, sType, nilmap)
 }
 
-func newStorageWithConfig(s *state.State, st storage, sType storageType, config map[string]interface{}) (storage, error) {
-	if s.OS.MockMode {
-		return st, nil
+func newStorageWithConfig(d *Daemon, sType storageType, config map[string]interface{}) (storage, error) {
+	if d.MockMode {
+		return d.Storage, nil
 	}
 
-	shared := storageShared{s: s}
-	var w storage
+	var s storage
 
 	switch sType {
 	case storageTypeBtrfs:
-		if st != nil && st.GetStorageType() == storageTypeBtrfs {
-			return st, nil
+		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeBtrfs {
+			return d.Storage, nil
 		}
-		btrfs := &storageBtrfs{storageShared: shared}
-		w = &storageLogWrapper{w: btrfs}
-		btrfs.storage = w
+
+		s = &storageLogWrapper{w: &storageBtrfs{d: d}}
 	case storageTypeZfs:
-		if st != nil && st.GetStorageType() == storageTypeZfs {
-			return st, nil
+		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeZfs {
+			return d.Storage, nil
 		}
-		zfs := &storageZfs{storageShared: shared}
-		w = &storageLogWrapper{w: zfs}
-		zfs.storage = w
+
+		s = &storageLogWrapper{w: &storageZfs{d: d}}
 	case storageTypeLvm:
-		if st != nil && st.GetStorageType() == storageTypeLvm {
-			return st, nil
+		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeLvm {
+			return d.Storage, nil
 		}
-		lvm := &storageLvm{storageShared: shared}
-		w = &storageLogWrapper{w: lvm}
-		lvm.storage = w
+
+		s = &storageLogWrapper{w: &storageLvm{d: d}}
 	default:
-		if st != nil && st.GetStorageType() == storageTypeDir {
-			return st, nil
+		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeDir {
+			return d.Storage, nil
 		}
-		dir := &storageDir{storageShared: shared}
-		w = &storageLogWrapper{w: dir}
-		dir.storage = w
+
+		s = &storageLogWrapper{w: &storageDir{d: d}}
 	}
 
-	return w.Init(config)
+	return s.Init(config)
 }
 
-func storageForFilename(s *state.State, storage storage, filename string) (storage, error) {
+func storageForFilename(d *Daemon, filename string) (storage, error) {
 	var filesystem string
 	var err error
 
 	config := make(map[string]interface{})
 	storageType := storageTypeDir
 
-	if s.OS.MockMode {
-		return newStorageWithConfig(s, storage, storageTypeMock, config)
+	if d.MockMode {
+		return newStorageWithConfig(d, storageTypeMock, config)
 	}
 
 	if shared.PathExists(filename) {
-		filesystem, err = util.FilesystemDetect(filename)
+		filesystem, err = filesystemDetect(filename)
 		if err != nil {
 			return nil, fmt.Errorf("couldn't detect filesystem for '%s': %v", filename, err)
 		}
-
-		if filesystem == "btrfs" {
-			if !(*storageBtrfs).isSubvolume(nil, filename) {
-				filesystem = ""
-			}
-		}
 	}
 
 	if shared.PathExists(filename + ".lv") {
@@ -260,12 +271,12 @@ func storageForFilename(s *state.State, storage storage, filename string) (stora
 		storageType = storageTypeBtrfs
 	}
 
-	return newStorageWithConfig(s, storage, storageType, config)
+	return newStorageWithConfig(d, storageType, config)
 }
 
-func storageForImage(s *state.State, storage storage, imgInfo *api.Image) (storage, error) {
+func storageForImage(d *Daemon, imgInfo *shared.ImageInfo) (storage, error) {
 	imageFilename := shared.VarPath("images", imgInfo.Fingerprint)
-	return storageForFilename(s, storage, imageFilename)
+	return storageForFilename(d, imageFilename)
 }
 
 type storageShared struct {
@@ -273,16 +284,12 @@ type storageShared struct {
 	sTypeName    string
 	sTypeVersion string
 
-	s *state.State
-
-	storage storage
-
-	log logger.Logger
+	log shared.Logger
 }
 
 func (ss *storageShared) initShared() error {
 	ss.log = logging.AddContext(
-		logger.Log,
+		shared.Log,
 		log.Ctx{"driver": fmt.Sprintf("storage/%s", ss.sTypeName)},
 	)
 	return nil
@@ -304,21 +311,18 @@ func (ss *storageShared) shiftRootfs(c container) error {
 	dpath := c.Path()
 	rpath := c.RootfsPath()
 
-	logger.Debug("Shifting root filesystem",
+	shared.LogDebug("Shifting root filesystem",
 		log.Ctx{"container": c.Name(), "rootfs": rpath})
 
-	idmapset, err := c.IdmapSet()
-	if err != nil {
-		return err
-	}
+	idmapset := c.IdmapSet()
 
 	if idmapset == nil {
 		return fmt.Errorf("IdmapSet of container '%s' is nil", c.Name())
 	}
 
-	err = idmapset.ShiftRootfs(rpath)
+	err := idmapset.ShiftRootfs(rpath)
 	if err != nil {
-		logger.Debugf("Shift of rootfs %s failed: %s", rpath, err)
+		shared.LogDebugf("Shift of rootfs %s failed: %s", rpath, err)
 		return err
 	}
 
@@ -329,10 +333,7 @@ func (ss *storageShared) shiftRootfs(c container) error {
 }
 
 func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
-	idmapset, err := c.IdmapSet()
-	if err != nil {
-		return err
-	}
+	idmapset := c.IdmapSet()
 
 	// Skip for privileged containers
 	if idmapset == nil {
@@ -350,9 +351,9 @@ func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
 
 	// Attempt to set a POSIX ACL first. Fallback to chmod if the fs doesn't support it.
 	acl := fmt.Sprintf("%d:rx", uid)
-	_, err = shared.RunCommand("setfacl", "-m", acl, destPath)
+	_, err := exec.Command("setfacl", "-m", acl, destPath).CombinedOutput()
 	if err != nil {
-		_, err := shared.RunCommand("chmod", "+x", destPath)
+		_, err := exec.Command("chmod", "+x", destPath).CombinedOutput()
 		if err != nil {
 			return fmt.Errorf("Failed to chmod the container path.")
 		}
@@ -363,13 +364,13 @@ func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
 
 type storageLogWrapper struct {
 	w   storage
-	log logger.Logger
+	log shared.Logger
 }
 
 func (lw *storageLogWrapper) Init(config map[string]interface{}) (storage, error) {
 	_, err := lw.w.Init(config)
 	lw.log = logging.AddContext(
-		logger.Log,
+		shared.Log,
 		log.Ctx{"driver": fmt.Sprintf("storage/%s", lw.w.GetStorageTypeName())},
 	)
 
@@ -557,7 +558,7 @@ func (lw *storageLogWrapper) MigrationSource(container container) (MigrationStor
 	return lw.w.MigrationSource(container)
 }
 
-func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error {
 	objNames := []string{}
 	for _, obj := range objects {
 		objNames = append(objNames, obj.GetName())
@@ -568,19 +569,16 @@ func (lw *storageLogWrapper) MigrationSink(live bool, container container, objec
 		"container": container.Name(),
 		"objects":   objNames,
 		"srcIdmap":  *srcIdmap,
+		"op":        op,
 	})
 
-	return lw.w.MigrationSink(live, container, objects, conn, srcIdmap)
+	return lw.w.MigrationSink(live, container, objects, conn, srcIdmap, op)
 }
 
-func ShiftIfNecessary(container container, srcIdmap *idmap.IdmapSet) error {
-	dstIdmap, err := container.IdmapSet()
-	if err != nil {
-		return err
-	}
-
+func ShiftIfNecessary(container container, srcIdmap *shared.IdmapSet) error {
+	dstIdmap := container.IdmapSet()
 	if dstIdmap == nil {
-		dstIdmap = new(idmap.IdmapSet)
+		dstIdmap = new(shared.IdmapSet)
 	}
 
 	if !reflect.DeepEqual(srcIdmap, dstIdmap) {
@@ -613,8 +611,7 @@ func (s rsyncStorageSourceDriver) Snapshots() []container {
 	return s.snapshots
 }
 
-func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
-	ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
+func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn, op *operation) error {
 	for _, send := range s.snapshots {
 		if err := send.StorageStart(); err != nil {
 			return err
@@ -622,19 +619,19 @@ func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
 		defer send.StorageStop()
 
 		path := send.Path()
-		if err := RsyncSend(ctName, shared.AddSlash(path), conn); err != nil {
+		wrapper := StorageProgressReader(op, "fs_progress", send.Name())
+		if err := RsyncSend(shared.AddSlash(path), conn, wrapper); err != nil {
 			return err
 		}
 	}
 
-	return RsyncSend(ctName, shared.AddSlash(s.container.Path()), conn)
+	wrapper := StorageProgressReader(op, "fs_progress", s.container.Name())
+	return RsyncSend(shared.AddSlash(s.container.Path()), conn, wrapper)
 }
 
 func (s rsyncStorageSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) error {
-	ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
-
 	/* resync anything that changed between our first send and the checkpoint */
-	return RsyncSend(ctName, shared.AddSlash(s.container.Path()), conn)
+	return RsyncSend(shared.AddSlash(s.container.Path()), conn, nil)
 }
 
 func (s rsyncStorageSourceDriver) Cleanup() {
@@ -650,14 +647,14 @@ func rsyncMigrationSource(container container) (MigrationStorageSourceDriver, er
 	return rsyncStorageSourceDriver{container, snapshots}, nil
 }
 
-func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.ContainerArgs {
+func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) containerArgs {
 	config := map[string]string{}
 
 	for _, ent := range snap.LocalConfig {
 		config[ent.GetKey()] = ent.GetValue()
 	}
 
-	devices := types.Devices{}
+	devices := shared.Devices{}
 	for _, ent := range snap.LocalDevices {
 		props := map[string]string{}
 		for _, prop := range ent.Config {
@@ -668,9 +665,9 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.Co
 	}
 
 	name := containerName + shared.SnapshotDelimiter + snap.GetName()
-	return db.ContainerArgs{
+	return containerArgs{
 		Name:         name,
-		Ctype:        db.CTypeSnapshot,
+		Ctype:        cTypeSnapshot,
 		Config:       config,
 		Profiles:     snap.Profiles,
 		Ephemeral:    snap.GetEphemeral(),
@@ -680,7 +677,7 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.Co
 	}
 }
 
-func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error {
 	isDirBackend := container.Storage().GetStorageType() == storageTypeDir
 
 	if isDirBackend {
@@ -692,12 +689,13 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 		}
 		for _, snap := range snapshots {
 			args := snapshotProtobufToContainerArgs(container.Name(), snap)
-			s, err := containerCreateEmptySnapshot(container.StateObject(), container.Storage(), args)
+			s, err := containerCreateEmptySnapshot(container.Daemon(), args)
 			if err != nil {
 				return err
 			}
 
-			if err := RsyncRecv(shared.AddSlash(s.Path()), conn); err != nil {
+			wrapper := StorageProgressWriter(op, "fs_progress", s.Name())
+			if err := RsyncRecv(shared.AddSlash(s.Path()), conn, wrapper); err != nil {
 				return err
 			}
 
@@ -706,7 +704,8 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 			}
 		}
 
-		if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+		if err := RsyncRecv(shared.AddSlash(container.Path()), conn, wrapper); err != nil {
 			return err
 		}
 	} else {
@@ -716,7 +715,9 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 		defer container.StorageStop()
 
 		for _, snap := range snapshots {
-			if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
+			args := snapshotProtobufToContainerArgs(container.Name(), snap)
+			wrapper := StorageProgressWriter(op, "fs_progress", snap.GetName())
+			if err := RsyncRecv(shared.AddSlash(container.Path()), conn, wrapper); err != nil {
 				return err
 			}
 
@@ -724,21 +725,22 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 				return err
 			}
 
-			args := snapshotProtobufToContainerArgs(container.Name(), snap)
-			_, err := containerCreateAsSnapshot(container.StateObject(), container.Storage(), args, container)
+			_, err := containerCreateAsSnapshot(container.Daemon(), args, container)
 			if err != nil {
 				return err
 			}
 		}
 
-		if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+		if err := RsyncRecv(shared.AddSlash(container.Path()), conn, wrapper); err != nil {
 			return err
 		}
 	}
 
 	if live {
 		/* now receive the final sync */
-		if err := RsyncRecv(shared.AddSlash(container.Path()), conn); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+		if err := RsyncRecv(shared.AddSlash(container.Path()), conn, wrapper); err != nil {
 			return err
 		}
 	}
@@ -750,36 +752,116 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 	return nil
 }
 
-func SetupStorageDriver(d *Daemon) error {
+// Useful functions for unreliable backends
+func tryExec(name string, arg ...string) ([]byte, error) {
 	var err error
+	var output []byte
 
-	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
-	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
+	for i := 0; i < 20; i++ {
+		output, err = exec.Command(name, arg...).CombinedOutput()
+		if err == nil {
+			break
+		}
 
-	if lvmVgName != "" {
-		d.Storage, err = newStorage(d, storageTypeLvm)
-		if err != nil {
-			logger.Errorf("Could not initialize storage type LVM: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
-	} else if zfsPoolName != "" {
-		d.Storage, err = newStorage(d, storageTypeZfs)
-		if err != nil {
-			logger.Errorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
-	} else if d.os.BackingFS == "btrfs" {
-		d.Storage, err = newStorage(d, storageTypeBtrfs)
-		if err != nil {
-			logger.Errorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
+		time.Sleep(500 * time.Millisecond)
 	}
 
-	d.Storage, err = newStorage(d, storageTypeDir)
+	return output, err
+}
 
-	return err
+func tryMount(src string, dst string, fs string, flags uintptr, options string) error {
+	var err error
+
+	for i := 0; i < 20; i++ {
+		err = syscall.Mount(src, dst, fs, flags, options)
+		if err == nil {
+			break
+		}
+
+		time.Sleep(500 * time.Millisecond)
+	}
+
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func tryUnmount(path string, flags int) error {
+	var err error
+
+	for i := 0; i < 20; i++ {
+		err = syscall.Unmount(path, flags)
+		if err == nil {
+			break
+		}
+
+		time.Sleep(500 * time.Millisecond)
+	}
+
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func progressWrapperRender(op *operation, key string, description string, progressInt int64, speedInt int64) {
+	meta := op.metadata
+	if meta == nil {
+		meta = make(map[string]interface{})
+	}
+
+	progress := fmt.Sprintf("%s (%s/s)", shared.GetByteSizeString(progressInt), shared.GetByteSizeString(speedInt))
+	if description != "" {
+		progress = fmt.Sprintf("%s: %s (%s/s)", description, shared.GetByteSizeString(progressInt), shared.GetByteSizeString(speedInt))
+	}
+
+	if meta[key] != progress {
+		meta[key] = progress
+		op.UpdateMetadata(meta)
+	}
+}
+
+func StorageProgressReader(op *operation, key string, description string) func(io.ReadCloser) io.ReadCloser {
+	return func(reader io.ReadCloser) io.ReadCloser {
+		if op == nil {
+			return reader
+		}
+
+		progress := func(progressInt int64, speedInt int64) {
+			progressWrapperRender(op, key, description, progressInt, speedInt)
+		}
+
+		readPipe := &ioprogress.ProgressReader{
+			ReadCloser: reader,
+			Tracker: &ioprogress.ProgressTracker{
+				Handler: progress,
+			},
+		}
+
+		return readPipe
+	}
+}
+
+func StorageProgressWriter(op *operation, key string, description string) func(io.WriteCloser) io.WriteCloser {
+	return func(writer io.WriteCloser) io.WriteCloser {
+		if op == nil {
+			return writer
+		}
+
+		progress := func(progressInt int64, speedInt int64) {
+			progressWrapperRender(op, key, description, progressInt, speedInt)
+		}
+
+		writePipe := &ioprogress.ProgressWriter{
+			WriteCloser: writer,
+			Tracker: &ioprogress.ProgressTracker{
+				Handler: progress,
+			},
+		}
+
+		return writePipe
+	}
 }
