@@ -2,12 +2,12 @@ package main
 
 import (
 	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
-	"sort"
 	"strconv"
 	"strings"
 	"syscall"
@@ -16,13 +16,13 @@ import (
 	"github.com/pborman/uuid"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type storageBtrfs struct {
+	d *Daemon
+
 	storageShared
 }
 
@@ -38,7 +38,7 @@ func (s *storageBtrfs) Init(config map[string]interface{}) (storage, error) {
 		return s, fmt.Errorf("The 'btrfs' tool isn't available")
 	}
 
-	output, err := shared.RunCommand("btrfs", "version")
+	output, err := exec.Command("btrfs", "version").CombinedOutput()
 	if err != nil {
 		return s, fmt.Errorf("The 'btrfs' tool isn't working properly")
 	}
@@ -266,12 +266,12 @@ func (s *storageBtrfs) ContainerSetQuota(container container, size int64) error
 		return err
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"btrfs",
 		"qgroup",
 		"limit",
 		"-e", fmt.Sprintf("%d", size),
-		subvol)
+		subvol).CombinedOutput()
 
 	if err != nil {
 		return fmt.Errorf("Failed to set btrfs quota: %s", output)
@@ -414,8 +414,8 @@ func (s *storageBtrfs) ImageCreate(fingerprint string) error {
 		return err
 	}
 
-	if err := unpackImage(imagePath, subvol, s.storage.GetStorageType()); err != nil {
-		s.subvolsDelete(subvol)
+	if err := unpackImage(s.d, imagePath, subvol); err != nil {
+		s.subvolDelete(subvol)
 		return err
 	}
 
@@ -443,11 +443,11 @@ func (s *storageBtrfs) subvolCreate(subvol string) error {
 		}
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"btrfs",
 		"subvolume",
 		"create",
-		subvol)
+		subvol).CombinedOutput()
 	if err != nil {
 		s.log.Debug(
 			"subvolume create failed",
@@ -464,13 +464,13 @@ func (s *storageBtrfs) subvolCreate(subvol string) error {
 }
 
 func (s *storageBtrfs) subvolQGroup(subvol string) (string, error) {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"btrfs",
 		"qgroup",
 		"show",
 		subvol,
 		"-e",
-		"-f")
+		"-f").CombinedOutput()
 
 	if err != nil {
 		return "", fmt.Errorf("btrfs quotas not supported. Try enabling them with 'btrfs quota enable'.")
@@ -498,13 +498,13 @@ func (s *storageBtrfs) subvolQGroup(subvol string) (string, error) {
 }
 
 func (s *storageBtrfs) subvolQGroupUsage(subvol string) (int64, error) {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"btrfs",
 		"qgroup",
 		"show",
 		subvol,
 		"-e",
-		"-f")
+		"-f").CombinedOutput()
 
 	if err != nil {
 		return -1, fmt.Errorf("btrfs quotas not supported. Try enabling them with 'btrfs quota enable'.")
@@ -535,12 +535,12 @@ func (s *storageBtrfs) subvolDelete(subvol string) error {
 	// Attempt (but don't fail on) to delete any qgroup on the subvolume
 	qgroup, err := s.subvolQGroup(subvol)
 	if err == nil {
-		output, err := shared.RunCommand(
+		output, err := exec.Command(
 			"btrfs",
 			"qgroup",
 			"destroy",
 			qgroup,
-			subvol)
+			subvol).CombinedOutput()
 
 		if err != nil {
 			s.log.Warn(
@@ -551,12 +551,12 @@ func (s *storageBtrfs) subvolDelete(subvol string) error {
 	}
 
 	// Delete the subvolume itself
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"btrfs",
 		"subvolume",
 		"delete",
 		subvol,
-	)
+	).CombinedOutput()
 
 	if err != nil {
 		s.log.Warn(
@@ -576,7 +576,6 @@ func (s *storageBtrfs) subvolsDelete(subvol string) error {
 	if err != nil {
 		return err
 	}
-	sort.Sort(sort.Reverse(sort.StringSlice(subsubvols)))
 
 	for _, subsubvol := range subsubvols {
 		s.log.Debug(
@@ -618,23 +617,23 @@ func (s *storageBtrfs) subvolSnapshot(
 		}
 	}
 
-	var output string
+	var output []byte
 	var err error
 	if readonly {
-		output, err = shared.RunCommand(
+		output, err = exec.Command(
 			"btrfs",
 			"subvolume",
 			"snapshot",
 			"-r",
 			source,
-			dest)
+			dest).CombinedOutput()
 	} else {
-		output, err = shared.RunCommand(
+		output, err = exec.Command(
 			"btrfs",
 			"subvolume",
 			"snapshot",
 			source,
-			dest)
+			dest).CombinedOutput()
 	}
 	if err != nil {
 		s.log.Error(
@@ -660,7 +659,6 @@ func (s *storageBtrfs) subvolsSnapshot(
 	if err != nil {
 		return err
 	}
-	sort.Sort(sort.StringSlice(subsubvols))
 
 	if len(subsubvols) > 0 && readonly {
 		// A root with subvolumes can never be readonly,
@@ -673,18 +671,17 @@ func (s *storageBtrfs) subvolsSnapshot(
 	}
 
 	// First snapshot the root
-	err = s.subvolSnapshot(source, dest, readonly)
-	if err != nil {
+	if err := s.subvolSnapshot(source, dest, readonly); err != nil {
 		return err
 	}
 
 	// Now snapshot all subvolumes of the root.
 	for _, subsubvol := range subsubvols {
-		// Clear the target for the subvol to use
-		os.Remove(path.Join(dest, subsubvol))
+		if err := s.subvolSnapshot(
+			path.Join(source, subsubvol),
+			path.Join(dest, subsubvol),
+			readonly); err != nil {
 
-		err := s.subvolSnapshot(path.Join(source, subsubvol), path.Join(dest, subsubvol), readonly)
-		if err != nil {
 			return err
 		}
 	}
@@ -697,14 +694,38 @@ func (s *storageBtrfs) subvolsSnapshot(
  * else false.
  */
 func (s *storageBtrfs) isSubvolume(subvolPath string) bool {
-	fs := syscall.Stat_t{}
-	err := syscall.Lstat(subvolPath, &fs)
-	if err != nil {
-		return false
+	if runningInUserns {
+		// subvolume show is restricted to real root, use a workaround
+
+		fs := syscall.Statfs_t{}
+		err := syscall.Statfs(subvolPath, &fs)
+		if err != nil {
+			return false
+		}
+
+		if fs.Type != filesystemSuperMagicBtrfs {
+			return false
+		}
+
+		parentFs := syscall.Statfs_t{}
+		err = syscall.Statfs(path.Dir(subvolPath), &parentFs)
+		if err != nil {
+			return false
+		}
+
+		if fs.Fsid == parentFs.Fsid {
+			return false
+		}
+
+		return true
 	}
 
-	// Check if BTRFS_FIRST_FREE_OBJECTID
-	if fs.Ino != 256 {
+	output, err := exec.Command(
+		"btrfs",
+		"subvolume",
+		"show",
+		subvolPath).CombinedOutput()
+	if err != nil || strings.HasPrefix(string(output), "ERROR: ") {
 		return false
 	}
 
@@ -715,34 +736,82 @@ func (s *storageBtrfs) isSubvolume(subvolPath string) bool {
 func (s *storageBtrfs) getSubVolumes(path string) ([]string, error) {
 	result := []string{}
 
-	if !strings.HasSuffix(path, "/") {
-		path = path + "/"
+	if runningInUserns {
+		if !strings.HasSuffix(path, "/") {
+			path = path + "/"
+		}
+
+		// Unprivileged users can't get to fs internals
+		filepath.Walk(path, func(fpath string, fi os.FileInfo, err error) error {
+			if strings.TrimRight(fpath, "/") == strings.TrimRight(path, "/") {
+				return nil
+			}
+
+			if err != nil {
+				return nil
+			}
+
+			if !fi.IsDir() {
+				return nil
+			}
+
+			if s.isSubvolume(fpath) {
+				result = append(result, strings.TrimPrefix(fpath, path))
+			}
+			return nil
+		})
+
+		return result, nil
 	}
 
-	// Unprivileged users can't get to fs internals
-	filepath.Walk(path, func(fpath string, fi os.FileInfo, err error) error {
-		// Skip walk errors
-		if err != nil {
-			return nil
+	out, err := exec.Command(
+		"btrfs",
+		"inspect-internal",
+		"rootid",
+		path).CombinedOutput()
+	if err != nil {
+		return result, fmt.Errorf(
+			"Unable to get btrfs rootid, path='%s', err='%s'",
+			path,
+			err)
+	}
+	rootid := strings.TrimRight(string(out), "\n")
+
+	out, err = exec.Command(
+		"btrfs",
+		"inspect-internal",
+		"subvolid-resolve",
+		rootid, path).CombinedOutput()
+	if err != nil {
+		return result, fmt.Errorf(
+			"Unable to resolve btrfs rootid, path='%s', err='%s'",
+			path,
+			err)
+	}
+	basePath := strings.TrimRight(string(out), "\n")
+
+	out, err = exec.Command(
+		"btrfs",
+		"subvolume",
+		"list",
+		"-o",
+		path).CombinedOutput()
+	if err != nil {
+		return result, fmt.Errorf(
+			"Unable to list subvolumes, path='%s', err='%s'",
+			path,
+			err)
+	}
+
+	lines := strings.Split(string(out), "\n")
+	for _, line := range lines {
+		if line == "" {
+			continue
 		}
 
-		// Ignore the base path
-		if strings.TrimRight(fpath, "/") == strings.TrimRight(path, "/") {
-			return nil
-		}
-
-		// Subvolumes can only be directories
-		if !fi.IsDir() {
-			return nil
-		}
-
-		// Check if a btrfs subvolume
-		if s.isSubvolume(fpath) {
-			result = append(result, strings.TrimPrefix(fpath, path))
-		}
-
-		return nil
-	})
+		cols := strings.Fields(line)
+		result = append(result, cols[8][len(basePath):])
+	}
 
 	return result, nil
 }
@@ -760,12 +829,11 @@ func (s *btrfsMigrationSourceDriver) Snapshots() []container {
 	return s.snapshots
 }
 
-func (s *btrfsMigrationSourceDriver) send(conn *websocket.Conn, btrfsPath string, btrfsParent string) error {
-	args := []string{"send"}
+func (s *btrfsMigrationSourceDriver) send(conn *websocket.Conn, btrfsPath string, btrfsParent string, readWrapper func(io.ReadCloser) io.ReadCloser) error {
+	args := []string{"send", btrfsPath}
 	if btrfsParent != "" {
 		args = append(args, "-p", btrfsParent)
 	}
-	args = append(args, btrfsPath)
 
 	cmd := exec.Command("btrfs", args...)
 
@@ -774,48 +842,51 @@ func (s *btrfsMigrationSourceDriver) send(conn *websocket.Conn, btrfsPath string
 		return err
 	}
 
+	readPipe := io.ReadCloser(stdout)
+	if readWrapper != nil {
+		readPipe = readWrapper(stdout)
+	}
+
 	stderr, err := cmd.StderrPipe()
 	if err != nil {
 		return err
 	}
 
-	err = cmd.Start()
-	if err != nil {
+	if err := cmd.Start(); err != nil {
 		return err
 	}
 
-	<-shared.WebsocketSendStream(conn, stdout, 4*1024*1024)
+	<-shared.WebsocketSendStream(conn, readPipe, 4*1024*1024)
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
-		logger.Error("problem reading btrfs send stderr", log.Ctx{"err": err})
+		shared.LogError("problem reading btrfs send stderr", log.Ctx{"err": err})
 	}
 
 	err = cmd.Wait()
 	if err != nil {
-		logger.Error("problem with btrfs send", log.Ctx{"output": string(output)})
+		shared.LogError("problem with btrfs send", log.Ctx{"output": string(output)})
 	}
-
 	return err
 }
 
-func (s *btrfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
+func (s *btrfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn, op *operation) error {
 	if s.container.IsSnapshot() {
 		tmpPath := containerPath(fmt.Sprintf("%s/.migration-send-%s", s.container.Name(), uuid.NewRandom().String()), true)
 		err := os.MkdirAll(tmpPath, 0700)
 		if err != nil {
 			return err
 		}
-		defer os.RemoveAll(tmpPath)
 
 		btrfsPath := fmt.Sprintf("%s/.root", tmpPath)
-		if err := s.btrfs.subvolsSnapshot(s.container.Path(), btrfsPath, true); err != nil {
+		if err := s.btrfs.subvolSnapshot(s.container.Path(), btrfsPath, true); err != nil {
 			return err
 		}
 
-		defer s.btrfs.subvolsDelete(btrfsPath)
+		defer s.btrfs.subvolDelete(btrfsPath)
 
-		return s.send(conn, btrfsPath, "")
+		wrapper := StorageProgressReader(op, "fs_progress", s.container.Name())
+		return s.send(conn, btrfsPath, "", wrapper)
 	}
 
 	for i, snap := range s.snapshots {
@@ -824,7 +895,8 @@ func (s *btrfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) erro
 			prev = s.snapshots[i-1].Path()
 		}
 
-		if err := s.send(conn, snap.Path(), prev); err != nil {
+		wrapper := StorageProgressReader(op, "fs_progress", snap.Name())
+		if err := s.send(conn, snap.Path(), prev, wrapper); err != nil {
 			return err
 		}
 	}
@@ -837,20 +909,19 @@ func (s *btrfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) erro
 	if err != nil {
 		return err
 	}
-	defer os.RemoveAll(tmpPath)
 
 	s.runningSnapName = fmt.Sprintf("%s/.root", tmpPath)
-	if err := s.btrfs.subvolsSnapshot(s.container.Path(), s.runningSnapName, true); err != nil {
+	if err := s.btrfs.subvolSnapshot(s.container.Path(), s.runningSnapName, true); err != nil {
 		return err
 	}
-	defer s.btrfs.subvolsDelete(s.runningSnapName)
 
 	btrfsParent := ""
 	if len(s.btrfsSnapshotNames) > 0 {
 		btrfsParent = s.btrfsSnapshotNames[len(s.btrfsSnapshotNames)-1]
 	}
 
-	return s.send(conn, s.runningSnapName, btrfsParent)
+	wrapper := StorageProgressReader(op, "fs_progress", s.container.Name())
+	return s.send(conn, s.runningSnapName, btrfsParent, wrapper)
 }
 
 func (s *btrfsMigrationSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) error {
@@ -859,33 +930,31 @@ func (s *btrfsMigrationSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) e
 	if err != nil {
 		return err
 	}
-	defer os.RemoveAll(tmpPath)
 
 	s.stoppedSnapName = fmt.Sprintf("%s/.root", tmpPath)
-	err = s.btrfs.subvolsSnapshot(s.container.Path(), s.stoppedSnapName, true)
-	if err != nil {
+	if err := s.btrfs.subvolSnapshot(s.container.Path(), s.stoppedSnapName, true); err != nil {
 		return err
 	}
 
-	return s.send(conn, s.stoppedSnapName, s.runningSnapName)
+	return s.send(conn, s.stoppedSnapName, s.runningSnapName, nil)
 }
 
 func (s *btrfsMigrationSourceDriver) Cleanup() {
 	if s.stoppedSnapName != "" {
-		s.btrfs.subvolsDelete(s.stoppedSnapName)
+		s.btrfs.subvolDelete(s.stoppedSnapName)
 	}
 
 	if s.runningSnapName != "" {
-		s.btrfs.subvolsDelete(s.runningSnapName)
+		s.btrfs.subvolDelete(s.runningSnapName)
 	}
 }
 
 func (s *storageBtrfs) MigrationType() MigrationFSType {
 	if runningInUserns {
 		return MigrationFSType_RSYNC
+	} else {
+		return MigrationFSType_BTRFS
 	}
-
-	return MigrationFSType_BTRFS
 }
 
 func (s *storageBtrfs) PreservesInodes() bool {
@@ -925,9 +994,9 @@ func (s *storageBtrfs) MigrationSource(c container) (MigrationStorageSourceDrive
 	return driver, nil
 }
 
-func (s *storageBtrfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func (s *storageBtrfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error {
 	if runningInUserns {
-		return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
+		return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap, op)
 	}
 
 	cName := container.Name()
@@ -940,14 +1009,13 @@ func (s *storageBtrfs) MigrationSink(live bool, container container, snapshots [
 		}
 	}
 
-	btrfsRecv := func(btrfsPath string, targetPath string, isSnapshot bool) error {
+	btrfsRecv := func(btrfsPath string, targetPath string, isSnapshot bool, writeWrapper func(io.WriteCloser) io.WriteCloser) error {
 		args := []string{"receive", "-e", btrfsPath}
 		cmd := exec.Command("btrfs", args...)
 
 		// Remove the existing pre-created subvolume
 		err := s.subvolsDelete(targetPath)
 		if err != nil {
-			logger.Errorf("Failed to delete pre-created BTRFS subvolume: %s.", btrfsPath)
 			return err
 		}
 
@@ -961,36 +1029,40 @@ func (s *storageBtrfs) MigrationSink(live bool, container container, snapshots [
 			return err
 		}
 
-		err = cmd.Start()
-		if err != nil {
+		if err := cmd.Start(); err != nil {
 			return err
 		}
 
-		<-shared.WebsocketRecvStream(stdin, conn)
+		writePipe := io.WriteCloser(stdin)
+		if writeWrapper != nil {
+			writePipe = writeWrapper(stdin)
+		}
+
+		<-shared.WebsocketRecvStream(writePipe, conn)
 
 		output, err := ioutil.ReadAll(stderr)
 		if err != nil {
-			logger.Debugf("problem reading btrfs receive stderr %s", err)
+			shared.LogDebugf("problem reading btrfs receive stderr %s", err)
 		}
 
 		err = cmd.Wait()
 		if err != nil {
-			logger.Error("problem with btrfs receive", log.Ctx{"output": string(output)})
+			shared.LogError("problem with btrfs receive", log.Ctx{"output": string(output)})
 			return err
 		}
 
 		if !isSnapshot {
 			cPath := containerPath(fmt.Sprintf("%s/.root", cName), true)
 
-			err := s.subvolsSnapshot(cPath, targetPath, false)
+			err := s.subvolSnapshot(cPath, targetPath, false)
 			if err != nil {
-				logger.Error("problem with btrfs snapshot", log.Ctx{"err": err})
+				shared.LogError("problem with btrfs snapshot", log.Ctx{"err": err})
 				return err
 			}
 
 			err = s.subvolsDelete(cPath)
 			if err != nil {
-				logger.Error("problem with btrfs delete", log.Ctx{"err": err})
+				shared.LogError("problem with btrfs delete", log.Ctx{"err": err})
 				return err
 			}
 		}
@@ -1000,23 +1072,26 @@ func (s *storageBtrfs) MigrationSink(live bool, container container, snapshots [
 
 	for _, snap := range snapshots {
 		args := snapshotProtobufToContainerArgs(container.Name(), snap)
-		s, err := containerCreateEmptySnapshot(s.s, s.storage, args)
+		s, err := containerCreateEmptySnapshot(container.Daemon(), args)
 		if err != nil {
 			return err
 		}
 
-		if err := btrfsRecv(containerPath(cName, true), s.Path(), true); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", snap.GetName())
+		if err := btrfsRecv(containerPath(cName, true), s.Path(), true, wrapper); err != nil {
 			return err
 		}
 	}
 
 	/* finally, do the real container */
-	if err := btrfsRecv(containerPath(cName, true), container.Path(), false); err != nil {
+	wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+	if err := btrfsRecv(containerPath(cName, true), container.Path(), false, wrapper); err != nil {
 		return err
 	}
 
 	if live {
-		if err := btrfsRecv(containerPath(cName, true), container.Path(), false); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+		if err := btrfsRecv(containerPath(cName, true), container.Path(), false, wrapper); err != nil {
 			return err
 		}
 	}
