@@ -60,7 +60,7 @@ func (s *storageDir) ContainerCreateFromImage(
 	}
 
 	imagePath := shared.VarPath("images", imageFingerprint)
-	if err := unpackImage(imagePath, container.Path()); err != nil {
+	if err := unpackImage(s.d, imagePath, container.Path()); err != nil {
 		s.ContainerDelete(container)
 		return err
 	}
@@ -123,11 +123,11 @@ func (s *storageDir) ContainerCopy(
 	return container.TemplateApply("copy")
 }
 
-func (s *storageDir) ContainerStart(container container) error {
+func (s *storageDir) ContainerStart(name string, path string) error {
 	return nil
 }
 
-func (s *storageDir) ContainerStop(container container) error {
+func (s *storageDir) ContainerStop(name string, path string) error {
 	return nil
 }
 
@@ -195,13 +195,37 @@ func (s *storageDir) ContainerSnapshotCreate(
 	/*
 	 * Copy by using rsync
 	 */
-	output, err := storageRsyncCopy(oldPath, newPath)
-	if err != nil {
-		s.ContainerDelete(snapshotContainer)
-		s.log.Error("ContainerSnapshotCreate: rsync failed",
-			log.Ctx{"output": string(output)})
+	rsync := func(snapshotContainer container, oldPath string, newPath string) error {
+		output, err := storageRsyncCopy(oldPath, newPath)
+		if err != nil {
+			s.ContainerDelete(snapshotContainer)
+			s.log.Error("ContainerSnapshotCreate: rsync failed",
+				log.Ctx{"output": string(output)})
 
-		return fmt.Errorf("rsync failed: %s", string(output))
+			return fmt.Errorf("rsync failed: %s", string(output))
+		}
+		return nil
+	}
+
+	if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
+		return err
+	}
+
+	if sourceContainer.IsRunning() {
+		/* This is done to ensure consistency when snapshotting. But we
+		 * probably shouldn't fail just because of that.
+		 */
+		s.log.Debug("ContainerSnapshotCreate: trying to freeze and rsync again to ensure consistency.")
+		if err := sourceContainer.Freeze(); err != nil {
+			s.log.Warn("ContainerSnapshotCreate: trying to freeze and rsync again failed.")
+			return nil
+		}
+
+		if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
+			return err
+		}
+
+		defer sourceContainer.Unfreeze()
 	}
 
 	return nil
@@ -282,6 +306,6 @@ func (s *storageDir) MigrationSource(container container) (MigrationStorageSourc
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageDir) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
-	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
+func (s *storageDir) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error {
+	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap, op)
 }
