@@ -2,12 +2,14 @@ package main
 
 import (
 	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"os/exec"
 	"strconv"
 	"strings"
 	"syscall"
+	"time"
 
 	"github.com/gorilla/websocket"
 
@@ -52,7 +54,7 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 	err = s.zfsCheckPool(s.zfsPool)
 	if err != nil {
 		if shared.PathExists(shared.VarPath("zfs.img")) {
-			_ = exec.Command("modprobe", "zfs").Run()
+			_ = loadModule("zfs")
 
 			output, err := exec.Command("zpool", "import",
 				"-d", shared.VarPath(), s.zfsPool).CombinedOutput()
@@ -90,8 +92,8 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 }
 
 // Things we don't need to care about
-func (s *storageZfs) ContainerStart(container container) error {
-	fs := fmt.Sprintf("containers/%s", container.Name())
+func (s *storageZfs) ContainerStart(name string, path string) error {
+	fs := fmt.Sprintf("containers/%s", name)
 
 	// Just in case the container filesystem got unmounted
 	if !shared.IsMountPoint(shared.VarPath(fs)) {
@@ -101,7 +103,7 @@ func (s *storageZfs) ContainerStart(container container) error {
 	return nil
 }
 
-func (s *storageZfs) ContainerStop(container container) error {
+func (s *storageZfs) ContainerStop(name string, path string) error {
 	return nil
 }
 
@@ -659,7 +661,7 @@ func (s *storageZfs) ImageCreate(fingerprint string) error {
 		return err
 	}
 
-	err = unpackImage(imagePath, subvol)
+	err = unpackImage(s.d, imagePath, subvol)
 	if err != nil {
 		return cleanup(err)
 	}
@@ -923,20 +925,33 @@ func (s *storageZfs) zfsGet(path string, key string) (string, error) {
 }
 
 func (s *storageZfs) zfsRename(source string, dest string) error {
-	output, err := tryExec(
-		"zfs",
-		"rename",
-		"-p",
-		fmt.Sprintf("%s/%s", s.zfsPool, source),
-		fmt.Sprintf("%s/%s", s.zfsPool, dest))
-	if err != nil {
-		if s.zfsExists(source) || !s.zfsExists(dest) {
-			s.log.Error("zfs rename failed", log.Ctx{"output": string(output)})
-			return fmt.Errorf("Failed to rename ZFS filesystem: %s", output)
+	var err error
+	var output []byte
+
+	for i := 0; i < 20; i++ {
+		output, err = exec.Command(
+			"zfs",
+			"rename",
+			"-p",
+			fmt.Sprintf("%s/%s", s.zfsPool, source),
+			fmt.Sprintf("%s/%s", s.zfsPool, dest)).CombinedOutput()
+
+		// Success
+		if err == nil {
+			return nil
 		}
+
+		// zfs rename can fail because of descendants, yet still manage the rename
+		if !s.zfsExists(source) && s.zfsExists(dest) {
+			return nil
+		}
+
+		time.Sleep(500 * time.Millisecond)
 	}
 
-	return nil
+	// Timeout
+	s.log.Error("zfs rename failed", log.Ctx{"output": string(output)})
+	return fmt.Errorf("Failed to rename ZFS filesystem: %s", output)
 }
 
 func (s *storageZfs) zfsSet(path string, key string, value string) error {
@@ -1202,17 +1217,17 @@ func storageZFSValidatePoolName(d *Daemon, key string, value string) error {
 		if err != nil {
 			return fmt.Errorf("Invalid ZFS pool: %v", err)
 		}
-	}
 
-	// Confirm that the new pool is empty
-	s.zfsPool = value
-	subvols, err := s.zfsListSubvolumes("")
-	if err != nil {
-		return err
-	}
+		// Confirm that the new pool is empty
+		s.zfsPool = value
+		subvols, err := s.zfsListSubvolumes("")
+		if err != nil {
+			return err
+		}
 
-	if len(subvols) > 0 {
-		return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
+		if len(subvols) > 0 {
+			return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
+		}
 	}
 
 	// Confirm the old pool isn't in use anymore
@@ -1246,7 +1261,7 @@ func (s *zfsMigrationSourceDriver) Snapshots() []container {
 	return s.snapshots
 }
 
-func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zfsParent string) error {
+func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zfsParent string, readWrapper func(io.ReadCloser) io.ReadCloser) error {
 	fields := strings.SplitN(s.container.Name(), shared.SnapshotDelimiter, 2)
 	args := []string{"send", fmt.Sprintf("%s/containers/%s@%s", s.zfs.zfsPool, fields[0], zfsName)}
 	if zfsParent != "" {
@@ -1260,6 +1275,11 @@ func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zf
 		return err
 	}
 
+	readPipe := io.ReadCloser(stdout)
+	if readWrapper != nil {
+		readPipe = readWrapper(stdout)
+	}
+
 	stderr, err := cmd.StderrPipe()
 	if err != nil {
 		return err
@@ -1269,7 +1289,7 @@ func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zf
 		return err
 	}
 
-	<-shared.WebsocketSendStream(conn, stdout, 4*1024*1024)
+	<-shared.WebsocketSendStream(conn, readPipe, 4*1024*1024)
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
@@ -1284,11 +1304,12 @@ func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zf
 	return err
 }
 
-func (s *zfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
+func (s *zfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn, op *operation) error {
 	if s.container.IsSnapshot() {
 		fields := strings.SplitN(s.container.Name(), shared.SnapshotDelimiter, 2)
 		snapshotName := fmt.Sprintf("snapshot-%s", fields[1])
-		return s.send(conn, snapshotName, "")
+		wrapper := StorageProgressReader(op, "fs_progress", s.container.Name())
+		return s.send(conn, snapshotName, "", wrapper)
 	}
 
 	lastSnap := ""
@@ -1301,7 +1322,8 @@ func (s *zfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) error
 
 		lastSnap = snap
 
-		if err := s.send(conn, snap, prev); err != nil {
+		wrapper := StorageProgressReader(op, "fs_progress", snap)
+		if err := s.send(conn, snap, prev, wrapper); err != nil {
 			return err
 		}
 	}
@@ -1311,7 +1333,8 @@ func (s *zfsMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn) error
 		return err
 	}
 
-	if err := s.send(conn, s.runningSnapName, lastSnap); err != nil {
+	wrapper := StorageProgressReader(op, "fs_progress", s.container.Name())
+	if err := s.send(conn, s.runningSnapName, lastSnap, wrapper); err != nil {
 		return err
 	}
 
@@ -1324,7 +1347,7 @@ func (s *zfsMigrationSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) err
 		return err
 	}
 
-	if err := s.send(conn, s.stoppedSnapName, s.runningSnapName); err != nil {
+	if err := s.send(conn, s.stoppedSnapName, s.runningSnapName, nil); err != nil {
 		return err
 	}
 
@@ -1396,8 +1419,8 @@ func (s *storageZfs) MigrationSource(ct container) (MigrationStorageSourceDriver
 	return &driver, nil
 }
 
-func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
-	zfsRecv := func(zfsName string) error {
+func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet, op *operation) error {
+	zfsRecv := func(zfsName string, writeWrapper func(io.WriteCloser) io.WriteCloser) error {
 		zfsFsName := fmt.Sprintf("%s/%s", s.zfsPool, zfsName)
 		args := []string{"receive", "-F", "-u", zfsFsName}
 		cmd := exec.Command("zfs", args...)
@@ -1416,7 +1439,12 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 			return err
 		}
 
-		<-shared.WebsocketRecvStream(stdin, conn)
+		writePipe := io.WriteCloser(stdin)
+		if writeWrapper != nil {
+			writePipe = writeWrapper(stdin)
+		}
+
+		<-shared.WebsocketRecvStream(writePipe, conn)
 
 		output, err := ioutil.ReadAll(stderr)
 		if err != nil {
@@ -1449,8 +1477,9 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 			return err
 		}
 
+		wrapper := StorageProgressWriter(op, "fs_progress", snap.GetName())
 		name := fmt.Sprintf("containers/%s@snapshot-%s", container.Name(), snap.GetName())
-		if err := zfsRecv(name); err != nil {
+		if err := zfsRecv(name, wrapper); err != nil {
 			return err
 		}
 
@@ -1484,13 +1513,15 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 	}()
 
 	/* finally, do the real container */
-	if err := zfsRecv(zfsName); err != nil {
+	wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+	if err := zfsRecv(zfsName, wrapper); err != nil {
 		return err
 	}
 
 	if live {
 		/* and again for the post-running snapshot if this was a live migration */
-		if err := zfsRecv(zfsName); err != nil {
+		wrapper := StorageProgressWriter(op, "fs_progress", container.Name())
+		if err := zfsRecv(zfsName, wrapper); err != nil {
 			return err
 		}
 	}
