@@ -14,15 +14,23 @@ import (
 	"encoding/pem"
 	"fmt"
 	"io/ioutil"
+	"log"
 	"math/big"
 	"net"
-	"net/http"
 	"os"
 	"os/user"
 	"path"
 	"time"
 )
 
+// CertInfo is the representation of a Certificate in the API.
+type CertInfo struct {
+	Certificate string `json:"certificate"`
+	Fingerprint string `json:"fingerprint"`
+	Name        string `json:"name"`
+	Type        string `json:"type"`
+}
+
 /*
  * Generate a list of names for which the certificate will be valid.
  * This will include the hostname and ip address
@@ -94,14 +102,16 @@ func GenCert(certf string, keyf string, certtype bool) error {
 
 	certOut, err := os.Create(certf)
 	if err != nil {
-		return fmt.Errorf("Failed to open %s for writing: %v", certf, err)
+		log.Fatalf("failed to open %s for writing: %s", certf, err)
+		return err
 	}
 	certOut.Write(certBytes)
 	certOut.Close()
 
 	keyOut, err := os.OpenFile(keyf, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
 	if err != nil {
-		return fmt.Errorf("Failed to open %s for writing: %v", keyf, err)
+		log.Printf("failed to open %s for writing: %s", keyf, err)
+		return err
 	}
 	keyOut.Write(keyBytes)
 	keyOut.Close()
@@ -113,12 +123,14 @@ func GenCert(certf string, keyf string, certtype bool) error {
 func GenerateMemCert(client bool) ([]byte, []byte, error) {
 	privk, err := rsa.GenerateKey(rand.Reader, 4096)
 	if err != nil {
-		return nil, nil, fmt.Errorf("Failed to generate key: %v", err)
+		log.Fatalf("failed to generate key")
+		return nil, nil, err
 	}
 
 	hosts, err := mynames()
 	if err != nil {
-		return nil, nil, fmt.Errorf("Failed to get my hostname: %v", err)
+		log.Fatalf("Failed to get my hostname")
+		return nil, nil, err
 	}
 
 	validFrom := time.Now()
@@ -127,7 +139,8 @@ func GenerateMemCert(client bool) ([]byte, []byte, error) {
 	serialNumberLimit := new(big.Int).Lsh(big.NewInt(1), 128)
 	serialNumber, err := rand.Int(rand.Reader, serialNumberLimit)
 	if err != nil {
-		return nil, nil, fmt.Errorf("Failed to generate serial number: %v", err)
+		log.Fatalf("failed to generate serial number: %s", err)
+		return nil, nil, err
 	}
 
 	userEntry, err := user.Current()
@@ -177,7 +190,8 @@ func GenerateMemCert(client bool) ([]byte, []byte, error) {
 
 	derBytes, err := x509.CreateCertificate(rand.Reader, &template, &template, &privk.PublicKey, privk)
 	if err != nil {
-		return nil, nil, fmt.Errorf("Failed to create certificate: %v", err)
+		log.Fatalf("Failed to create certificate: %s", err)
+		return nil, nil, err
 	}
 
 	cert := pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: derBytes})
@@ -216,32 +230,3 @@ func CertFingerprintStr(c string) (string, error) {
 
 	return CertFingerprint(cert), nil
 }
-
-func GetRemoteCertificate(address string) (*x509.Certificate, error) {
-	// Setup a permissive TLS config
-	tlsConfig, err := GetTLSConfig("", "", "", nil)
-	if err != nil {
-		return nil, err
-	}
-
-	tlsConfig.InsecureSkipVerify = true
-	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            RFC3493Dialer,
-		Proxy:           ProxyFromEnvironment,
-	}
-
-	// Connect
-	client := &http.Client{Transport: tr}
-	resp, err := client.Get(address)
-	if err != nil {
-		return nil, err
-	}
-
-	// Retrieve the certificate
-	if resp.TLS == nil || len(resp.TLS.PeerCertificates) == 0 {
-		return nil, fmt.Errorf("Unable to read remote TLS certificate")
-	}
-
-	return resp.TLS.PeerCertificates[0], nil
-}
