@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"strconv"
 	"strings"
+	"syscall"
 	"time"
 )
 
@@ -60,6 +61,7 @@ type ContainerStateNetworkCounters struct {
 type ContainerExecControl struct {
 	Command string            `json:"command"`
 	Args    map[string]string `json:"args"`
+	Signal  syscall.Signal    `json:"signal"`
 }
 
 type SnapshotInfo struct {
@@ -172,6 +174,19 @@ func IsInt64(value string) error {
 	return nil
 }
 
+func IsUint32(value string) error {
+	if value == "" {
+		return nil
+	}
+
+	_, err := strconv.ParseInt(value, 10, 32)
+	if err != nil {
+		return fmt.Errorf("Invalid value for uint32: %s: %v", value, err)
+	}
+
+	return nil
+}
+
 func IsPriority(value string) error {
 	if value == "" {
 		return nil
@@ -226,13 +241,65 @@ var KnownContainerConfigKeys = map[string]func(value string) error{
 	"boot.autostart.priority":    IsInt64,
 	"boot.host_shutdown_timeout": IsInt64,
 
-	"limits.cpu":           IsAny,
-	"limits.cpu.allowance": IsAny,
-	"limits.cpu.priority":  IsPriority,
+	"limits.cpu": IsAny,
+	"limits.cpu.allowance": func(value string) error {
+		if value == "" {
+			return nil
+		}
+
+		if strings.HasSuffix(value, "%") {
+			// Percentage based allocation
+			_, err := strconv.Atoi(strings.TrimSuffix(value, "%"))
+			if err != nil {
+				return err
+			}
+
+			return nil
+		}
+
+		// Time based allocation
+		fields := strings.SplitN(value, "/", 2)
+		if len(fields) != 2 {
+			return fmt.Errorf("Invalid allowance: %s", value)
+		}
+
+		_, err := strconv.Atoi(strings.TrimSuffix(fields[0], "ms"))
+		if err != nil {
+			return err
+		}
+
+		_, err = strconv.Atoi(strings.TrimSuffix(fields[1], "ms"))
+		if err != nil {
+			return err
+		}
+
+		return nil
+	},
+	"limits.cpu.priority": IsPriority,
 
 	"limits.disk.priority": IsPriority,
 
-	"limits.memory": IsAny,
+	"limits.memory": func(value string) error {
+		if value == "" {
+			return nil
+		}
+
+		if strings.HasSuffix(value, "%") {
+			_, err := strconv.ParseInt(strings.TrimSuffix(value, "%"), 10, 64)
+			if err != nil {
+				return err
+			}
+
+			return nil
+		}
+
+		_, err := ParseByteSizeString(value)
+		if err != nil {
+			return err
+		}
+
+		return nil
+	},
 	"limits.memory.enforce": func(value string) error {
 		return IsOneOf(value, []string{"soft", "hard"})
 	},
@@ -248,6 +315,9 @@ var KnownContainerConfigKeys = map[string]func(value string) error{
 	"security.nesting":    IsBool,
 	"security.privileged": IsBool,
 
+	"security.idmap.size":     IsUint32,
+	"security.idmap.isolated": IsBool,
+
 	"security.syscalls.blacklist_default": IsBool,
 	"security.syscalls.blacklist_compat":  IsBool,
 	"security.syscalls.blacklist":         IsAny,
@@ -257,11 +327,14 @@ var KnownContainerConfigKeys = map[string]func(value string) error{
 	"raw.apparmor": IsAny,
 	"raw.lxc":      IsAny,
 	"raw.seccomp":  IsAny,
+	"raw.idmap":    IsAny,
 
 	"volatile.apply_template":   IsAny,
 	"volatile.base_image":       IsAny,
 	"volatile.last_state.idmap": IsAny,
 	"volatile.last_state.power": IsAny,
+	"volatile.idmap.next":       IsAny,
+	"volatile.idmap.base":       IsAny,
 }
 
 // ConfigKeyChecker returns a function that will check whether or not
@@ -293,5 +366,9 @@ func ConfigKeyChecker(key string) (func(value string) error, error) {
 		return IsAny, nil
 	}
 
+	if strings.HasPrefix(key, "image.") {
+		return IsAny, nil
+	}
+
 	return nil, fmt.Errorf("Bad key: %s", key)
 }
