@@ -10,17 +10,15 @@ import (
 	"os"
 	"path/filepath"
 	"sort"
-	"strconv"
 	"strings"
 	"time"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/ioprogress"
 	"github.com/lxc/lxd/shared/osarch"
 )
 
-type ssSortImage []api.Image
+type ssSortImage []shared.ImageInfo
 
 func (a ssSortImage) Len() int {
 	return len(a)
@@ -33,19 +31,15 @@ func (a ssSortImage) Swap(i, j int) {
 func (a ssSortImage) Less(i, j int) bool {
 	if a[i].Properties["os"] == a[j].Properties["os"] {
 		if a[i].Properties["release"] == a[j].Properties["release"] {
-			if !shared.TimeIsSet(a[i].CreatedAt) {
+			if a[i].CreationDate.UTC().Unix() == 0 {
 				return true
 			}
 
-			if !shared.TimeIsSet(a[j].CreatedAt) {
+			if a[j].CreationDate.UTC().Unix() == 0 {
 				return false
 			}
 
-			if a[i].CreatedAt == a[j].CreatedAt {
-				return a[i].Properties["serial"] > a[j].Properties["serial"]
-			}
-
-			return a[i].CreatedAt.UTC().Unix() > a[j].CreatedAt.UTC().Unix()
+			return a[i].CreationDate.UTC().Unix() > a[j].CreationDate.UTC().Unix()
 		}
 
 		if a[i].Properties["release"] == "" {
@@ -82,10 +76,10 @@ type SimpleStreamsManifest struct {
 	Products map[string]SimpleStreamsManifestProduct `json:"products"`
 }
 
-func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
+func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]string) {
 	downloads := map[string][][]string{}
 
-	images := []api.Image{}
+	images := []shared.ImageInfo{}
 	nameLayout := "20060102"
 	eolLayout := "2006-01-02"
 
@@ -112,70 +106,58 @@ func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
 				continue
 			}
 
-			var meta SimpleStreamsManifestProductVersionItem
-			var rootTar SimpleStreamsManifestProductVersionItem
-			var rootSquash SimpleStreamsManifestProductVersionItem
-			deltas := []SimpleStreamsManifestProductVersionItem{}
+			size := int64(0)
+			filename := ""
+			fingerprint := ""
 
+			metaPath := ""
+			metaHash := ""
+			rootfsPath := ""
+			rootfsHash := ""
+
+			found := 0
 			for _, item := range version.Items {
-				// Identify deltas
-				if item.FileType == "squashfs.vcdiff" {
-					deltas = append(deltas, item)
-				}
-
 				// Skip the files we don't care about
 				if !shared.StringInSlice(item.FileType, []string{"root.tar.xz", "lxd.tar.xz", "squashfs"}) {
 					continue
 				}
+				found += 1
+
+				if fingerprint == "" {
+					if item.LXDHashSha256SquashFs != "" {
+						fingerprint = item.LXDHashSha256SquashFs
+					} else if item.LXDHashSha256RootXz != "" {
+						fingerprint = item.LXDHashSha256RootXz
+					} else if item.LXDHashSha256 != "" {
+						fingerprint = item.LXDHashSha256
+					}
+				}
 
 				if item.FileType == "lxd.tar.xz" {
-					meta = item
-				} else if item.FileType == "squashfs" {
-					rootSquash = item
-				} else if item.FileType == "root.tar.xz" {
-					rootTar = item
+					fields := strings.Split(item.Path, "/")
+					filename = fields[len(fields)-1]
+					metaPath = item.Path
+					metaHash = item.HashSha256
+
+					size += item.Size
+				}
+
+				if rootfsPath == "" || rootfsHash == "" {
+					if item.FileType == "squashfs" {
+						rootfsPath = item.Path
+						rootfsHash = item.HashSha256
+					}
+
+					if item.FileType == "root.tar.xz" {
+						rootfsPath = item.Path
+						rootfsHash = item.HashSha256
+					}
+
+					size += item.Size
 				}
 			}
 
-			if meta.FileType == "" || (rootTar.FileType == "" && rootSquash.FileType == "") {
-				// Invalid image
-				continue
-			}
-
-			metaPath := meta.Path
-			metaHash := meta.HashSha256
-			metaSize := meta.Size
-			rootfsPath := ""
-			rootfsHash := ""
-			rootfsSize := int64(0)
-			fields := strings.Split(meta.Path, "/")
-			filename := fields[len(fields)-1]
-			size := meta.Size
-			fingerprint := ""
-
-			if rootSquash.FileType != "" {
-				if meta.LXDHashSha256SquashFs != "" {
-					fingerprint = meta.LXDHashSha256SquashFs
-				} else {
-					fingerprint = meta.LXDHashSha256
-				}
-				size += rootSquash.Size
-				rootfsPath = rootSquash.Path
-				rootfsHash = rootSquash.HashSha256
-				rootfsSize = rootSquash.Size
-			} else {
-				if meta.LXDHashSha256RootXz != "" {
-					fingerprint = meta.LXDHashSha256RootXz
-				} else {
-					fingerprint = meta.LXDHashSha256
-				}
-				size += rootTar.Size
-				rootfsPath = rootTar.Path
-				rootfsHash = rootTar.HashSha256
-				rootfsSize = rootTar.Size
-			}
-
-			if size == 0 || filename == "" || fingerprint == "" {
+			if found < 2 || size == 0 || filename == "" || fingerprint == "" {
 				// Invalid image
 				continue
 			}
@@ -187,12 +169,12 @@ func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
 			}
 			description = fmt.Sprintf("%s (%s)", description, name)
 
-			image := api.Image{}
+			image := shared.ImageInfo{}
 			image.Architecture = architectureName
 			image.Public = true
 			image.Size = size
-			image.CreatedAt = creationDate
-			image.UploadedAt = creationDate
+			image.CreationDate = creationDate
+			image.UploadDate = creationDate
 			image.Filename = filename
 			image.Fingerprint = fingerprint
 			image.Properties = map[string]string{
@@ -207,9 +189,9 @@ func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
 
 			// Add the provided aliases
 			if product.Aliases != "" {
-				image.Aliases = []api.ImageAlias{}
+				image.Aliases = []shared.ImageAlias{}
 				for _, entry := range strings.Split(product.Aliases, ",") {
-					image.Aliases = append(image.Aliases, api.ImageAlias{Name: entry})
+					image.Aliases = append(image.Aliases, shared.ImageAlias{Name: entry})
 				}
 			}
 
@@ -221,47 +203,15 @@ func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
 			}
 
 			// Attempt to parse the EOL
-			image.ExpiresAt = time.Unix(0, 0).UTC()
+			image.ExpiryDate = time.Unix(0, 0).UTC()
 			if product.SupportedEOL != "" {
 				eolDate, err := time.Parse(eolLayout, product.SupportedEOL)
 				if err == nil {
-					image.ExpiresAt = eolDate
+					image.ExpiryDate = eolDate
 				}
 			}
 
-			imgDownloads := [][]string{
-				{metaPath, metaHash, "meta", fmt.Sprintf("%d", metaSize)},
-				{rootfsPath, rootfsHash, "root", fmt.Sprintf("%d", rootfsSize)}}
-
-			// Add the deltas
-			for _, delta := range deltas {
-				srcImage, ok := product.Versions[delta.DeltaBase]
-				if !ok {
-					continue
-				}
-
-				var srcFingerprint string
-				for _, item := range srcImage.Items {
-					if item.FileType != "lxd.tar.xz" {
-						continue
-					}
-
-					srcFingerprint = item.LXDHashSha256SquashFs
-					break
-				}
-
-				if srcFingerprint == "" {
-					continue
-				}
-
-				imgDownloads = append(imgDownloads, []string{
-					delta.Path,
-					delta.HashSha256,
-					fmt.Sprintf("root.delta-%s", srcFingerprint),
-					fmt.Sprintf("%d", delta.Size)})
-			}
-
-			downloads[fingerprint] = imgDownloads
+			downloads[fingerprint] = [][]string{[]string{metaPath, metaHash, "meta"}, []string{rootfsPath, rootfsHash, "root"}}
 			images = append(images, image)
 		}
 	}
@@ -297,7 +247,6 @@ type SimpleStreamsManifestProductVersionItem struct {
 	LXDHashSha256RootXz   string `json:"combined_rootxz_sha256"`
 	LXDHashSha256SquashFs string `json:"combined_squashfs_sha256"`
 	Size                  int64  `json:"size"`
-	DeltaBase             string `json:"delta_base"`
 }
 
 type SimpleStreamsIndex struct {
@@ -313,12 +262,6 @@ type SimpleStreamsIndexStream struct {
 	Products []string `json:"products"`
 }
 
-type SimpleStreamsFile struct {
-	Path   string
-	Sha256 string
-	Size   int64
-}
-
 func NewClient(url string, httpClient http.Client, useragent string) *SimpleStreams {
 	return &SimpleStreams{
 		http:           &httpClient,
@@ -335,8 +278,8 @@ type SimpleStreams struct {
 
 	cachedIndex    *SimpleStreamsIndex
 	cachedManifest map[string]*SimpleStreamsManifest
-	cachedImages   []api.Image
-	cachedAliases  map[string]*api.ImageAliasesEntry
+	cachedImages   []shared.ImageInfo
+	cachedAliases  map[string]*shared.ImageAliasesEntry
 }
 
 func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
@@ -344,8 +287,7 @@ func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
 		return s.cachedIndex, nil
 	}
 
-	url := fmt.Sprintf("%s/streams/v1/index.json", s.url)
-	req, err := http.NewRequest("GET", url, nil)
+	req, err := http.NewRequest("GET", fmt.Sprintf("%s/streams/v1/index.json", s.url), nil)
 	if err != nil {
 		return nil, err
 	}
@@ -360,10 +302,6 @@ func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
 	}
 	defer r.Body.Close()
 
-	if r.StatusCode != http.StatusOK {
-		return nil, fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
-	}
-
 	body, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return nil, err
@@ -386,8 +324,7 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 		return s.cachedManifest[path], nil
 	}
 
-	url := fmt.Sprintf("%s/%s", s.url, path)
-	req, err := http.NewRequest("GET", url, nil)
+	req, err := http.NewRequest("GET", fmt.Sprintf("%s/%s", s.url, path), nil)
 	if err != nil {
 		return nil, err
 	}
@@ -402,10 +339,6 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 	}
 	defer r.Body.Close()
 
-	if r.StatusCode != http.StatusOK {
-		return nil, fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
-	}
-
 	body, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return nil, err
@@ -423,8 +356,8 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 	return &ssManifest, nil
 }
 
-func (s *SimpleStreams) applyAliases(images []api.Image) ([]api.Image, map[string]*api.ImageAliasesEntry, error) {
-	aliases := map[string]*api.ImageAliasesEntry{}
+func (s *SimpleStreams) applyAliases(images []shared.ImageInfo) ([]shared.ImageInfo, map[string]*shared.ImageAliasesEntry, error) {
+	aliases := map[string]*shared.ImageAliasesEntry{}
 
 	sort.Sort(ssSortImage(images))
 
@@ -436,7 +369,7 @@ func (s *SimpleStreams) applyAliases(images []api.Image) ([]api.Image, map[strin
 		}
 	}
 
-	addAlias := func(name string, fingerprint string) *api.ImageAlias {
+	addAlias := func(name string, fingerprint string) *shared.ImageAlias {
 		if defaultOS != "" {
 			name = strings.TrimPrefix(name, fmt.Sprintf("%s/", defaultOS))
 		}
@@ -445,17 +378,17 @@ func (s *SimpleStreams) applyAliases(images []api.Image) ([]api.Image, map[strin
 			return nil
 		}
 
-		alias := api.ImageAliasesEntry{}
+		alias := shared.ImageAliasesEntry{}
 		alias.Name = name
 		alias.Target = fingerprint
 		aliases[name] = &alias
 
-		return &api.ImageAlias{Name: name}
+		return &shared.ImageAlias{Name: name}
 	}
 
 	architectureName, _ := osarch.ArchitectureGetLocal()
 
-	newImages := []api.Image{}
+	newImages := []shared.ImageInfo{}
 	for _, image := range images {
 		if image.Aliases != nil {
 			// Build a new list of aliases from the provided ones
@@ -485,12 +418,12 @@ func (s *SimpleStreams) applyAliases(images []api.Image) ([]api.Image, map[strin
 	return newImages, aliases, nil
 }
 
-func (s *SimpleStreams) getImages() ([]api.Image, map[string]*api.ImageAliasesEntry, error) {
+func (s *SimpleStreams) getImages() ([]shared.ImageInfo, map[string]*shared.ImageAliasesEntry, error) {
 	if s.cachedImages != nil && s.cachedAliases != nil {
 		return s.cachedImages, s.cachedAliases, nil
 	}
 
-	images := []api.Image{}
+	images := []shared.ImageInfo{}
 
 	// Load the main index
 	ssIndex, err := s.parseIndex()
@@ -534,7 +467,7 @@ func (s *SimpleStreams) getImages() ([]api.Image, map[string]*api.ImageAliasesEn
 	return images, aliases, nil
 }
 
-func (s *SimpleStreams) GetFiles(fingerprint string) (map[string]SimpleStreamsFile, error) {
+func (s *SimpleStreams) getPaths(fingerprint string) ([][]string, error) {
 	// Load the main index
 	ssIndex, err := s.parseIndex()
 	if err != nil {
@@ -562,21 +495,11 @@ func (s *SimpleStreams) GetFiles(fingerprint string) (map[string]SimpleStreamsFi
 
 		for _, image := range manifestImages {
 			if strings.HasPrefix(image.Fingerprint, fingerprint) {
-				files := map[string]SimpleStreamsFile{}
-
+				urls := [][]string{}
 				for _, path := range downloads[image.Fingerprint] {
-					size, err := strconv.ParseInt(path[3], 10, 64)
-					if err != nil {
-						return nil, err
-					}
-
-					files[path[2]] = SimpleStreamsFile{
-						Path:   path[0],
-						Sha256: path[1],
-						Size:   size}
+					urls = append(urls, []string{path[0], path[1], path[2]})
 				}
-
-				return files, nil
+				return urls, nil
 			}
 		}
 	}
@@ -601,20 +524,20 @@ func (s *SimpleStreams) downloadFile(path string, hash string, target string, pr
 			req.Header.Set("User-Agent", s.useragent)
 		}
 
-		r, err := s.http.Do(req)
+		resp, err := s.http.Do(req)
 		if err != nil {
 			return err
 		}
-		defer r.Body.Close()
+		defer resp.Body.Close()
 
-		if r.StatusCode != http.StatusOK {
-			return fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
+		if resp.StatusCode != http.StatusOK {
+			return fmt.Errorf("invalid simplestreams source: got %d looking for %s", resp.StatusCode, path)
 		}
 
 		body := &ioprogress.ProgressReader{
-			ReadCloser: r.Body,
+			ReadCloser: resp.Body,
 			Tracker: &ioprogress.ProgressTracker{
-				Length:  r.ContentLength,
+				Length:  resp.ContentLength,
 				Handler: progress,
 			},
 		}
@@ -650,13 +573,13 @@ func (s *SimpleStreams) downloadFile(path string, hash string, target string, pr
 	return nil
 }
 
-func (s *SimpleStreams) ListAliases() ([]api.ImageAliasesEntry, error) {
+func (s *SimpleStreams) ListAliases() (shared.ImageAliases, error) {
 	_, aliasesMap, err := s.getImages()
 	if err != nil {
 		return nil, err
 	}
 
-	aliases := []api.ImageAliasesEntry{}
+	aliases := shared.ImageAliases{}
 
 	for _, alias := range aliasesMap {
 		aliases = append(aliases, *alias)
@@ -665,46 +588,38 @@ func (s *SimpleStreams) ListAliases() ([]api.ImageAliasesEntry, error) {
 	return aliases, nil
 }
 
-func (s *SimpleStreams) ListImages() ([]api.Image, error) {
+func (s *SimpleStreams) ListImages() ([]shared.ImageInfo, error) {
 	images, _, err := s.getImages()
 	return images, err
 }
 
-func (s *SimpleStreams) GetAlias(name string) (*api.ImageAliasesEntry, error) {
+func (s *SimpleStreams) GetAlias(name string) string {
 	_, aliasesMap, err := s.getImages()
 	if err != nil {
-		return nil, err
+		return ""
 	}
 
 	alias, ok := aliasesMap[name]
 	if !ok {
-		return nil, fmt.Errorf("Alias '%s' doesn't exist", name)
+		return ""
 	}
 
-	return alias, nil
+	return alias.Target
 }
 
-func (s *SimpleStreams) GetImage(fingerprint string) (*api.Image, error) {
+func (s *SimpleStreams) GetImageInfo(fingerprint string) (*shared.ImageInfo, error) {
 	images, _, err := s.getImages()
 	if err != nil {
 		return nil, err
 	}
 
-	matches := []api.Image{}
-
 	for _, image := range images {
 		if strings.HasPrefix(image.Fingerprint, fingerprint) {
-			matches = append(matches, image)
+			return &image, nil
 		}
 	}
 
-	if len(matches) == 0 {
-		return nil, fmt.Errorf("The requested image couldn't be found.")
-	} else if len(matches) > 1 {
-		return nil, fmt.Errorf("More than one match for the provided partial fingerprint.")
-	}
-
-	return &matches[0], nil
+	return nil, fmt.Errorf("The requested image couldn't be found.")
 }
 
 func (s *SimpleStreams) ExportImage(image string, target string) (string, error) {
@@ -712,16 +627,16 @@ func (s *SimpleStreams) ExportImage(image string, target string) (string, error)
 		return "", fmt.Errorf("Split images can only be written to a directory.")
 	}
 
-	files, err := s.GetFiles(image)
+	paths, err := s.getPaths(image)
 	if err != nil {
 		return "", err
 	}
 
-	for _, file := range files {
-		fields := strings.Split(file.Path, "/")
+	for _, path := range paths {
+		fields := strings.Split(path[0], "/")
 		targetFile := filepath.Join(target, fields[len(fields)-1])
 
-		err := s.downloadFile(file.Path, file.Sha256, targetFile, nil)
+		err := s.downloadFile(path[0], path[1], targetFile, nil)
 		if err != nil {
 			return "", err
 		}
@@ -730,15 +645,18 @@ func (s *SimpleStreams) ExportImage(image string, target string) (string, error)
 	return target, nil
 }
 
-func (s *SimpleStreams) Download(image string, fileType string, target string, progress func(int64, int64)) error {
-	files, err := s.GetFiles(image)
+func (s *SimpleStreams) Download(image string, file string, target string, progress func(int64, int64)) error {
+	paths, err := s.getPaths(image)
 	if err != nil {
 		return err
 	}
 
-	file, ok := files[fileType]
-	if ok {
-		return s.downloadFile(file.Path, file.Sha256, target, progress)
+	for _, path := range paths {
+		if file != path[2] {
+			continue
+		}
+
+		return s.downloadFile(path[0], path[1], target, progress)
 	}
 
 	return fmt.Errorf("The file couldn't be found.")
