@@ -11,38 +11,19 @@ import (
 	"io"
 	"io/ioutil"
 	"net/http"
-	"net/url"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
 	"reflect"
 	"regexp"
-	"runtime"
 	"strconv"
 	"strings"
-	"time"
-	"unicode"
 )
 
 const SnapshotDelimiter = "/"
 const DefaultPort = "8443"
 
-// URLEncode encodes a path and query parameters to a URL.
-func URLEncode(path string, query map[string]string) (string, error) {
-	u, err := url.Parse(path)
-	if err != nil {
-		return "", err
-	}
-
-	params := url.Values{}
-	for key, value := range query {
-		params.Add(key, value)
-	}
-	u.RawQuery = params.Encode()
-	return u.String(), nil
-}
-
 // AddSlash adds a slash to the end of paths if they don't already have one.
 // This can be useful for rsyncing things, since rsync has behavior present on
 // the presence or absence of a trailing slash.
@@ -99,47 +80,6 @@ func IsUnixSocket(path string) bool {
 	return (stat.Mode() & os.ModeSocket) == os.ModeSocket
 }
 
-// HostPath returns the host path for the provided path
-// On a normal system, this does nothing
-// When inside of a snap environment, returns the real path
-func HostPath(path string) string {
-	// Ignore empty paths
-	if len(path) == 0 {
-		return path
-	}
-
-	// Check if we're running in a snap package
-	snap := os.Getenv("SNAP")
-	if snap == "" {
-		return path
-	}
-
-	// Handle relative paths
-	if path[0] != os.PathSeparator {
-		// Use the cwd of the parent as snap-confine alters our own cwd on launch
-		ppid := os.Getppid()
-		if ppid < 1 {
-			return path
-		}
-
-		pwd, err := os.Readlink(fmt.Sprintf("/proc/%d/cwd", ppid))
-		if err != nil {
-			return path
-		}
-
-		path = filepath.Clean(strings.Join([]string{pwd, path}, string(os.PathSeparator)))
-	}
-
-	// Check if the path is already snap-aware
-	for _, prefix := range []string{"/snap", "/var/snap", "/var/lib/snapd"} {
-		if strings.HasPrefix(path, prefix) {
-			return path
-		}
-	}
-
-	return fmt.Sprintf("/var/lib/snapd/hostfs%s", path)
-}
-
 // VarPath returns the provided path elements joined by a slash and
 // appended to the end of $LXD_DIR, which defaults to /var/lib/lxd.
 func VarPath(path ...string) string {
@@ -179,13 +119,13 @@ func LogPath(path ...string) string {
 	return filepath.Join(items...)
 }
 
-func ParseLXDFileHeaders(headers http.Header) (uid int64, gid int64, mode int, type_ string, write string) {
-	uid, err := strconv.ParseInt(headers.Get("X-LXD-uid"), 10, 64)
+func ParseLXDFileHeaders(headers http.Header) (uid int, gid int, mode int, type_ string) {
+	uid, err := strconv.Atoi(headers.Get("X-LXD-uid"))
 	if err != nil {
 		uid = -1
 	}
 
-	gid, err = strconv.ParseInt(headers.Get("X-LXD-gid"), 10, 64)
+	gid, err = strconv.Atoi(headers.Get("X-LXD-gid"))
 	if err != nil {
 		gid = -1
 	}
@@ -208,15 +148,7 @@ func ParseLXDFileHeaders(headers http.Header) (uid int64, gid int64, mode int, t
 		type_ = "file"
 	}
 
-	write = headers.Get("X-LXD-write")
-	/* backwards compat: before "write" was introduced, we could only
-	 * overwrite files
-	 */
-	if write == "" {
-		write = "overwrite"
-	}
-
-	return uid, gid, mode, type_, write
+	return uid, gid, mode, type_
 }
 
 func ReadToJSON(r io.Reader, req interface{}) error {
@@ -320,13 +252,11 @@ func WriteAllBuf(w io.Writer, buf *bytes.Buffer) error {
 // FileMove tries to move a file by using os.Rename,
 // if that fails it tries to copy the file and remove the source.
 func FileMove(oldPath string, newPath string) error {
-	err := os.Rename(oldPath, newPath)
-	if err == nil {
+	if err := os.Rename(oldPath, newPath); err == nil {
 		return nil
 	}
 
-	err = FileCopy(oldPath, newPath)
-	if err != nil {
+	if err := FileCopy(oldPath, newPath); err != nil {
 		return err
 	}
 
@@ -343,15 +273,10 @@ func FileCopy(source string, dest string) error {
 	}
 	defer s.Close()
 
-	fi, err := s.Stat()
-	if err != nil {
-		return err
-	}
-
 	d, err := os.Create(dest)
 	if err != nil {
 		if os.IsExist(err) {
-			d, err = os.OpenFile(dest, os.O_WRONLY, fi.Mode())
+			d, err = os.OpenFile(dest, os.O_WRONLY, 0700)
 			if err != nil {
 				return err
 			}
@@ -362,17 +287,7 @@ func FileCopy(source string, dest string) error {
 	defer d.Close()
 
 	_, err = io.Copy(d, s)
-	if err != nil {
-		return err
-	}
-
-	/* chown not supported on windows */
-	if runtime.GOOS != "windows" {
-		_, uid, gid := GetOwnerMode(fi)
-		return d.Chown(uid, gid)
-	}
-
-	return nil
+	return err
 }
 
 type BytesReadCloser struct {
@@ -479,15 +394,6 @@ func IntInSlice(key int, list []int) bool {
 	return false
 }
 
-func Int64InSlice(key int64, list []int64) bool {
-	for _, entry := range list {
-		if entry == key {
-			return true
-		}
-	}
-	return false
-}
-
 func IsTrue(value string) bool {
 	if StringInSlice(strings.ToLower(value), []string{"true", "1", "yes", "on"}) {
 		return true
@@ -496,6 +402,42 @@ func IsTrue(value string) bool {
 	return false
 }
 
+func IsOnSharedMount(pathName string) (bool, error) {
+	file, err := os.Open("/proc/self/mountinfo")
+	if err != nil {
+		return false, err
+	}
+	defer file.Close()
+
+	absPath, err := filepath.Abs(pathName)
+	if err != nil {
+		return false, err
+	}
+
+	expPath, err := os.Readlink(absPath)
+	if err != nil {
+		expPath = absPath
+	}
+
+	scanner := bufio.NewScanner(file)
+	for scanner.Scan() {
+		line := scanner.Text()
+		rows := strings.Fields(line)
+
+		if rows[4] != expPath {
+			continue
+		}
+
+		if strings.HasPrefix(rows[6], "shared:") {
+			return true, nil
+		} else {
+			return false, nil
+		}
+	}
+
+	return false, nil
+}
+
 func IsBlockdev(fm os.FileMode) bool {
 	return ((fm&os.ModeDevice != 0) && (fm&os.ModeCharDevice == 0))
 }
@@ -511,12 +453,12 @@ func IsBlockdevPath(pathName string) bool {
 }
 
 func BlockFsDetect(dev string) (string, error) {
-	out, err := RunCommand("blkid", "-s", "TYPE", "-o", "value", dev)
+	out, err := exec.Command("blkid", "-s", "TYPE", "-o", "value", dev).Output()
 	if err != nil {
-		return "", err
+		return "", fmt.Errorf("Failed to run blkid on: %s", dev)
 	}
 
-	return strings.TrimSpace(out), nil
+	return strings.TrimSpace(string(out)), nil
 }
 
 // DeepCopy copies src to dest by using encoding/gob so its not that fast.
@@ -586,7 +528,6 @@ func ValidHostname(name string) bool {
 	return true
 }
 
-// Spawn the editor with a temporary YAML file for editing configs
 func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 	var f *os.File
 	var err error
@@ -617,8 +558,7 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 			return []byte{}, err
 		}
 
-		err = os.Chmod(f.Name(), 0600)
-		if err != nil {
+		if err = f.Chmod(0600); err != nil {
 			f.Close()
 			os.Remove(f.Name())
 			return []byte{}, err
@@ -627,8 +567,7 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 		f.Write(inContent)
 		f.Close()
 
-		path = fmt.Sprintf("%s.yaml", f.Name())
-		os.Rename(f.Name(), path)
+		path = f.Name()
 		defer os.Remove(path)
 	} else {
 		path = inPath
@@ -678,32 +617,19 @@ func ParseMetadata(metadata interface{}) (map[string]interface{}, error) {
 // Parse a size string in bytes (e.g. 200kB or 5GB) into the number of bytes it
 // represents. Supports suffixes up to EB. "" == 0.
 func ParseByteSizeString(input string) (int64, error) {
-	suffixLen := 2
-
 	if input == "" {
 		return 0, nil
 	}
 
-	if unicode.IsNumber(rune(input[len(input)-1])) {
-		// COMMENT(brauner): No suffix --> bytes.
-		suffixLen = 0
-	} else if (len(input) >= 2) && (input[len(input)-1] == 'B') && unicode.IsNumber(rune(input[len(input)-2])) {
-		// COMMENT(brauner): "B" suffix --> bytes.
-		suffixLen = 1
-	} else if strings.HasSuffix(input, " bytes") {
-		// COMMENT(brauner): Backward compatible behaviour in case we
-		// talk to a LXD that still uses GetByteSizeString() that
-		// returns "n bytes".
-		suffixLen = 6
-	} else if (len(input) < 3) && (suffixLen == 2) {
+	if len(input) < 3 {
 		return -1, fmt.Errorf("Invalid value: %s", input)
 	}
 
 	// Extract the suffix
-	suffix := input[len(input)-suffixLen:]
+	suffix := input[len(input)-2:]
 
 	// Extract the value
-	value := input[0 : len(input)-suffixLen]
+	value := input[0 : len(input)-2]
 	valueInt, err := strconv.ParseInt(value, 10, 64)
 	if err != nil {
 		return -1, fmt.Errorf("Invalid integer: %s", input)
@@ -713,11 +639,6 @@ func ParseByteSizeString(input string) (int64, error) {
 		return -1, fmt.Errorf("Invalid value: %d", valueInt)
 	}
 
-	// COMMENT(brauner): The value is already in bytes.
-	if suffixLen != 2 {
-		return valueInt, nil
-	}
-
 	// Figure out the multiplicator
 	multiplicator := int64(0)
 	switch suffix {
@@ -787,9 +708,9 @@ func ParseBitSizeString(input string) (int64, error) {
 	return valueInt * multiplicator, nil
 }
 
-func GetByteSizeString(input int64, precision uint) string {
+func GetByteSizeString(input int64) string {
 	if input < 1024 {
-		return fmt.Sprintf("%dB", input)
+		return fmt.Sprintf("%d bytes", input)
 	}
 
 	value := float64(input)
@@ -797,71 +718,30 @@ func GetByteSizeString(input int64, precision uint) string {
 	for _, unit := range []string{"kB", "MB", "GB", "TB", "PB", "EB"} {
 		value = value / 1024
 		if value < 1024 {
-			return fmt.Sprintf("%.*f%s", precision, value, unit)
+			return fmt.Sprintf("%.2f%s", value, unit)
 		}
 	}
 
-	return fmt.Sprintf("%.*fEB", precision, value)
+	return fmt.Sprintf("%.2fEB", value)
 }
 
-type RunError struct {
-	msg string
-	Err error
+// RemoveDuplicatesFromString removes all duplicates of the string 'sep'
+// from the specified string 's'.  Leading and trailing occurences of sep
+// are NOT removed (duplicate leading/trailing are).  Performs poorly if
+// there are multiple consecutive redundant separators.
+func RemoveDuplicatesFromString(s string, sep string) string {
+	dup := sep + sep
+	for s = strings.Replace(s, dup, sep, -1); strings.Contains(s, dup); s = strings.Replace(s, dup, sep, -1) {
+
+	}
+	return s
 }
 
-func (e RunError) Error() string {
-	return e.msg
-}
-
-func RunCommand(name string, arg ...string) (string, error) {
+func RunCommand(name string, arg ...string) error {
 	output, err := exec.Command(name, arg...).CombinedOutput()
 	if err != nil {
-		err := RunError{
-			msg: fmt.Sprintf("Failed to run: %s %s: %s", name, strings.Join(arg, " "), strings.TrimSpace(string(output))),
-			Err: err,
-		}
-		return string(output), err
+		return fmt.Errorf("Failed to run: %s %s: %s", name, strings.Join(arg, " "), strings.TrimSpace(string(output)))
 	}
 
-	return string(output), nil
-}
-
-func TryRunCommand(name string, arg ...string) (string, error) {
-	var err error
-	var output string
-
-	for i := 0; i < 20; i++ {
-		output, err = RunCommand(name, arg...)
-		if err == nil {
-			break
-		}
-
-		time.Sleep(500 * time.Millisecond)
-	}
-
-	return output, err
-}
-
-func TimeIsSet(ts time.Time) bool {
-	if ts.Unix() <= 0 {
-		return false
-	}
-
-	if ts.UTC().Unix() <= 0 {
-		return false
-	}
-
-	return true
-}
-
-// WriteTempFile creates a temp file with the specified content
-func WriteTempFile(dir string, prefix string, content string) (string, error) {
-	f, err := ioutil.TempFile(dir, prefix)
-	if err != nil {
-		return "", err
-	}
-	defer f.Close()
-
-	_, err = f.WriteString(content)
-	return f.Name(), err
+	return nil
 }
