@@ -4,39 +4,33 @@
 package shared
 
 import (
-	"bufio"
 	"errors"
 	"fmt"
 	"io"
 	"os"
-	"path/filepath"
-	"reflect"
+	"os/exec"
 	"strings"
 	"sync"
 	"sync/atomic"
 	"syscall"
 	"unsafe"
-
-	"github.com/lxc/lxd/shared/logger"
 )
 
 // #cgo LDFLAGS: -lutil -lpthread
 /*
 #define _GNU_SOURCE
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <grp.h>
+#include <pty.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <grp.h>
 #include <limits.h>
 #include <poll.h>
-#include <pty.h>
-#include <pwd.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/un.h>
+#include <stdio.h>
 
 #ifndef AT_SYMLINK_FOLLOW
 #define AT_SYMLINK_FOLLOW    0x400
@@ -46,8 +40,6 @@ import (
 #define AT_EMPTY_PATH       0x1000
 #endif
 
-#define ABSTRACT_UNIX_SOCK_LEN sizeof(((struct sockaddr_un *)0)->sun_path)
-
 // This is an adaption from https://codereview.appspot.com/4589049, to be
 // included in the stdlib with the stdlib's license.
 
@@ -98,7 +90,7 @@ void configure_pty(int fd) {
 	return;
 }
 
-void create_pty(int *master, int *slave, uid_t uid, gid_t gid) {
+void create_pty(int *master, int *slave, int uid, int gid) {
 	if (openpty(master, slave, NULL, NULL, NULL) < 0) {
 		fprintf(stderr, "Failed to openpty: %s\n", strerror(errno));
 		return;
@@ -129,6 +121,71 @@ void create_pipe(int *master, int *slave) {
 	*slave = pipefd[1];
 }
 
+int shiftowner(char *basepath, char *path, int uid, int gid) {
+	struct stat sb;
+	int fd, r;
+	char fdpath[PATH_MAX];
+	char realpath[PATH_MAX];
+
+	fd = open(path, O_PATH|O_NOFOLLOW);
+	if (fd < 0 ) {
+		perror("Failed open");
+		return 1;
+	}
+
+	r = sprintf(fdpath, "/proc/self/fd/%d", fd);
+	if (r < 0) {
+		perror("Failed sprintf");
+		close(fd);
+		return 1;
+	}
+
+	r = readlink(fdpath, realpath, PATH_MAX);
+	if (r < 0) {
+		perror("Failed readlink");
+		close(fd);
+		return 1;
+	}
+
+	if (strlen(realpath) < strlen(basepath)) {
+		printf("Invalid path, source (%s) is outside of basepath (%s).\n", realpath, basepath);
+		close(fd);
+		return 1;
+	}
+
+	if (strncmp(realpath, basepath, strlen(basepath))) {
+		printf("Invalid path, source (%s) is outside of basepath (%s).\n", realpath, basepath);
+		close(fd);
+		return 1;
+	}
+
+	r = fstat(fd, &sb);
+	if (r < 0) {
+		perror("Failed fstat");
+		close(fd);
+		return 1;
+	}
+
+	r = fchownat(fd, "", uid, gid, AT_EMPTY_PATH|AT_SYMLINK_NOFOLLOW);
+	if (r < 0) {
+		perror("Failed chown");
+		close(fd);
+		return 1;
+	}
+
+	if (!S_ISLNK(sb.st_mode)) {
+		r = chmod(fdpath, sb.st_mode);
+		if (r < 0) {
+			perror("Failed chmod");
+			close(fd);
+			return 1;
+		}
+	}
+
+	close(fd);
+	return 0;
+}
+
 int get_poll_revents(int lfd, int timeout, int flags, int *revents, int *saved_errno)
 {
 	int ret;
@@ -152,8 +209,6 @@ again:
 */
 import "C"
 
-const ABSTRACT_UNIX_SOCK_LEN int = C.ABSTRACT_UNIX_SOCK_LEN
-
 const POLLIN int = C.POLLIN
 const POLLPRI int = C.POLLPRI
 const POLLNVAL int = C.POLLNVAL
@@ -174,11 +229,25 @@ func GetPollRevents(fd int, timeout int, flags int) (int, int, error) {
 	return int(ret), int(revents), err
 }
 
-func OpenPty(uid, gid int64) (master *os.File, slave *os.File, err error) {
+func ShiftOwner(basepath string, path string, uid int, gid int) error {
+	cbasepath := C.CString(basepath)
+	defer C.free(unsafe.Pointer(cbasepath))
+
+	cpath := C.CString(path)
+	defer C.free(unsafe.Pointer(cpath))
+
+	r := C.shiftowner(cbasepath, cpath, C.int(uid), C.int(gid))
+	if r != 0 {
+		return fmt.Errorf("Failed to change ownership of: %s", path)
+	}
+	return nil
+}
+
+func OpenPty(uid, gid int) (master *os.File, slave *os.File, err error) {
 	fd_master := C.int(-1)
 	fd_slave := C.int(-1)
-	rootUid := C.uid_t(uid)
-	rootGid := C.gid_t(gid)
+	rootUid := C.int(uid)
+	rootGid := C.int(gid)
 
 	C.create_pty(&fd_master, &fd_slave, rootUid, rootGid)
 
@@ -208,51 +277,36 @@ func Pipe() (master *os.File, slave *os.File, err error) {
 	return master, slave, nil
 }
 
-// UserId is an adaption from https://codereview.appspot.com/4589049.
-func UserId(name string) (int, error) {
-	var pw C.struct_passwd
-	var result *C.struct_passwd
+// GroupName is an adaption from https://codereview.appspot.com/4589049.
+func GroupName(gid int) (string, error) {
+	var grp C.struct_group
+	var result *C.struct_group
 
-	bufSize := C.sysconf(C._SC_GETPW_R_SIZE_MAX)
-	if bufSize < 0 {
-		bufSize = 4096
-	}
-
-	buf := C.malloc(C.size_t(bufSize))
+	bufSize := C.size_t(C.sysconf(C._SC_GETGR_R_SIZE_MAX))
+	buf := C.malloc(bufSize)
 	if buf == nil {
-		return -1, fmt.Errorf("allocation failed")
+		return "", fmt.Errorf("allocation failed")
 	}
 	defer C.free(buf)
 
-	cname := C.CString(name)
-	defer C.free(unsafe.Pointer(cname))
-
-again:
-	rv, errno := C.getpwnam_r(cname,
-		&pw,
+	// mygetgrgid_r is a wrapper around getgrgid_r to
+	// to avoid using gid_t because C.gid_t(gid) for
+	// unknown reasons doesn't work on linux.
+	rv := C.mygetgrgid_r(C.int(gid),
+		&grp,
 		(*C.char)(buf),
-		C.size_t(bufSize),
+		bufSize,
 		&result)
-	if rv < 0 {
-		// OOM killer will take care of us if we end up doing this too
-		// often.
-		if errno == syscall.ERANGE {
-			bufSize *= 2
-			tmp := C.realloc(buf, C.size_t(bufSize))
-			if tmp == nil {
-				return -1, fmt.Errorf("allocation failed")
-			}
-			buf = tmp
-			goto again
-		}
-		return -1, fmt.Errorf("failed user lookup: %s", syscall.Errno(rv))
+
+	if rv != 0 {
+		return "", fmt.Errorf("failed group lookup: %s", syscall.Errno(rv))
 	}
 
 	if result == nil {
-		return -1, fmt.Errorf("unknown user %s", name)
+		return "", fmt.Errorf("unknown group %d", gid)
 	}
 
-	return int(C.int(result.pw_uid)), nil
+	return C.GoString(result.gr_name), nil
 }
 
 // GroupId is an adaption from https://codereview.appspot.com/4589049.
@@ -260,42 +314,28 @@ func GroupId(name string) (int, error) {
 	var grp C.struct_group
 	var result *C.struct_group
 
-	bufSize := C.sysconf(C._SC_GETGR_R_SIZE_MAX)
-	if bufSize < 0 {
-		bufSize = 4096
-	}
-
-	buf := C.malloc(C.size_t(bufSize))
+	bufSize := C.size_t(C.sysconf(C._SC_GETGR_R_SIZE_MAX))
+	buf := C.malloc(bufSize)
 	if buf == nil {
 		return -1, fmt.Errorf("allocation failed")
 	}
+	defer C.free(buf)
 
+	// mygetgrgid_r is a wrapper around getgrgid_r to
+	// to avoid using gid_t because C.gid_t(gid) for
+	// unknown reasons doesn't work on linux.
 	cname := C.CString(name)
 	defer C.free(unsafe.Pointer(cname))
 
-again:
-	rv, errno := C.getgrnam_r(cname,
+	rv := C.getgrnam_r(cname,
 		&grp,
 		(*C.char)(buf),
-		C.size_t(bufSize),
+		bufSize,
 		&result)
-	if rv != 0 {
-		// OOM killer will take care of us if we end up doing this too
-		// often.
-		if errno == syscall.ERANGE {
-			bufSize *= 2
-			tmp := C.realloc(buf, C.size_t(bufSize))
-			if tmp == nil {
-				return -1, fmt.Errorf("allocation failed")
-			}
-			buf = tmp
-			goto again
-		}
 
-		C.free(buf)
+	if rv != 0 {
 		return -1, fmt.Errorf("failed group lookup: %s", syscall.Errno(rv))
 	}
-	C.free(buf)
 
 	if result == nil {
 		return -1, fmt.Errorf("unknown group %s", name)
@@ -306,14 +346,6 @@ again:
 
 // --- pure Go functions ---
 
-func Major(dev uint64) int {
-	return int(((dev >> 8) & 0xfff) | ((dev >> 32) & (0xfffff000)))
-}
-
-func Minor(dev uint64) int {
-	return int((dev & 0xff) | ((dev >> 12) & (0xffffff00)))
-}
-
 func GetFileStat(p string) (uid int, gid int, major int, minor int,
 	inode uint64, nlink int, err error) {
 	var stat syscall.Stat_t
@@ -328,41 +360,22 @@ func GetFileStat(p string) (uid int, gid int, major int, minor int,
 	major = -1
 	minor = -1
 	if stat.Mode&syscall.S_IFBLK != 0 || stat.Mode&syscall.S_IFCHR != 0 {
-		major = Major(stat.Rdev)
-		minor = Minor(stat.Rdev)
+		major = int(stat.Rdev / 256)
+		minor = int(stat.Rdev % 256)
 	}
 
 	return
 }
 
-func parseMountinfo(name string) int {
-	f, err := os.Open("/proc/self/mountinfo")
-	if err != nil {
-		return -1
-	}
-	defer f.Close()
-
-	scanner := bufio.NewScanner(f)
-	for scanner.Scan() {
-		line := scanner.Text()
-		tokens := strings.Fields(line)
-		if len(tokens) < 5 {
-			return -1
-		}
-		cleanPath := filepath.Clean(tokens[4])
-		cleanName := filepath.Clean(name)
-		if cleanPath == cleanName {
-			return 1
-		}
-	}
-
-	return 0
-}
-
 func IsMountPoint(name string) bool {
-	ret := parseMountinfo(name)
-	if ret >= 0 {
-		return (ret == 1)
+	_, err := exec.LookPath("mountpoint")
+	if err == nil {
+		err = exec.Command("mountpoint", "-q", name).Run()
+		if err != nil {
+			return false
+		}
+
+		return true
 	}
 
 	stat, err := os.Stat(name)
@@ -400,7 +413,7 @@ func ReadLastNLines(f *os.File, lines int) (string, error) {
 		}
 
 		if lines < 0 {
-			return string(data[i+1:]), nil
+			return string(data[i+1 : len(data)]), nil
 		}
 	}
 
@@ -552,17 +565,15 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 
 		atomic.StoreInt32(&attachedChildIsDead, 1)
 
-		ret, revents, err := GetPollRevents(fd, 0, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP | POLLNVAL))
+		ret, revents, err := GetPollRevents(fd, 0, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP))
 		if ret < 0 {
-			logger.Errorf("Failed to poll(POLLIN | POLLPRI | POLLHUP | POLLRDHUP) on file descriptor: %s.", err)
+			LogErrorf("Failed to poll(POLLIN | POLLPRI | POLLHUP | POLLRDHUP) on file descriptor: %s.", err)
 		} else if ret > 0 {
 			if (revents & POLLERR) > 0 {
-				logger.Warnf("Detected poll(POLLERR) event.")
-			} else if (revents & POLLNVAL) > 0 {
-				logger.Warnf("Detected poll(POLLNVAL) event.")
+				LogWarnf("Detected poll(POLLERR) event.")
 			}
 		} else if ret == 0 {
-			logger.Debugf("No data in stdout: exiting.")
+			LogDebugf("No data in stdout: exiting.")
 			once.Do(closeChannel)
 			return
 		}
@@ -579,12 +590,12 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 			nr := 0
 			var err error
 
-			ret, revents, err := GetPollRevents(fd, -1, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP | POLLNVAL))
+			ret, revents, err := GetPollRevents(fd, -1, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP))
 			if ret < 0 {
 				// COMMENT(brauner):
 				// This condition is only reached in cases where we are massively f*cked since we even handle
 				// EINTR in the underlying C wrapper around poll(). So let's exit here.
-				logger.Errorf("Failed to poll(POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP) on file descriptor: %s. Exiting.", err)
+				LogErrorf("Failed to poll(POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP) on file descriptor: %s. Exiting.", err)
 				return
 			}
 
@@ -594,16 +605,13 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 			// keep on reading from the pty file descriptor until we get a simple POLLHUP back.
 			both := ((revents & (POLLIN | POLLPRI)) > 0) && ((revents & (POLLHUP | POLLRDHUP)) > 0)
 			if both {
-				logger.Debugf("Detected poll(POLLIN | POLLPRI | POLLHUP | POLLRDHUP) event.")
+				LogDebugf("Detected poll(POLLIN | POLLPRI | POLLHUP | POLLRDHUP) event.")
 				read := buf[offset : offset+readSize]
 				nr, err = r.Read(read)
 			}
 
 			if (revents & POLLERR) > 0 {
-				logger.Warnf("Detected poll(POLLERR) event: exiting.")
-				return
-			} else if (revents & POLLNVAL) > 0 {
-				logger.Warnf("Detected poll(POLLNVAL) event: exiting.")
+				LogWarnf("Detected poll(POLLERR) event: exiting.")
 				return
 			}
 
@@ -634,7 +642,7 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 					// still be handling a pure POLLIN event from a write prior to the childs
 					// exit. But the child might have exited right before and performed
 					// atomic.StoreInt32() to update attachedChildIsDead before we
-					// performed our atomic.LoadInt32(). This means we accidentally hit this
+					// performed our atomic.LoadInt32(). This means we accidently hit this
 					// codepath and are misinformed about the available poll() events. So we
 					// need to perform a non-blocking poll() again to exclude that case:
 					//
@@ -662,12 +670,12 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 					//   or (POLLHUP | POLLRDHUP). Both will trigger another codepath (See [2].)
 					//   that takes care that all data of the child that is buffered in
 					//   stdout is written out.
-					ret, revents, err := GetPollRevents(fd, 0, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP | POLLNVAL))
+					ret, revents, err := GetPollRevents(fd, 0, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP))
 					if ret < 0 {
-						logger.Errorf("Failed to poll(POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP) on file descriptor: %s. Exiting.", err)
+						LogErrorf("Failed to poll(POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP) on file descriptor: %s. Exiting.", err)
 						return
-					} else if (revents & (POLLHUP | POLLRDHUP | POLLERR | POLLNVAL)) == 0 {
-						logger.Debugf("Exiting but background processes are still running.")
+					} else if (revents & (POLLHUP | POLLRDHUP)) == 0 {
+						LogDebugf("Exiting but background processes are still running.")
 						return
 					}
 				}
@@ -679,7 +687,7 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 			// The attached process has exited and we have read all data that may have
 			// been buffered.
 			if ((revents & (POLLHUP | POLLRDHUP)) > 0) && !both {
-				logger.Debugf("Detected poll(POLLHUP) event: exiting.")
+				LogDebugf("Detected poll(POLLHUP) event: exiting.")
 				return
 			}
 
@@ -694,87 +702,3 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 
 	return ch
 }
-
-// Detect whether err is an errno.
-func GetErrno(err error) (errno error, iserrno bool) {
-	sysErr, ok := err.(*os.SyscallError)
-	if ok {
-		return sysErr.Err, true
-	}
-
-	pathErr, ok := err.(*os.PathError)
-	if ok {
-		return pathErr.Err, true
-	}
-
-	tmpErrno, ok := err.(syscall.Errno)
-	if ok {
-		return tmpErrno, true
-	}
-
-	return nil, false
-}
-
-// Utsname returns the same info as syscall.Utsname, as strings
-type Utsname struct {
-	Sysname    string
-	Nodename   string
-	Release    string
-	Version    string
-	Machine    string
-	Domainname string
-}
-
-// Uname returns Utsname as strings
-func Uname() (*Utsname, error) {
-	/*
-	 * Based on: https://groups.google.com/forum/#!topic/golang-nuts/Jel8Bb-YwX8
-	 * there is really no better way to do this, which is
-	 * unfortunate. Also, we ditch the more accepted CharsToString
-	 * version in that thread, since it doesn't seem as portable,
-	 * viz. github issue #206.
-	 */
-
-	uname := syscall.Utsname{}
-	err := syscall.Uname(&uname)
-	if err != nil {
-		return nil, err
-	}
-
-	return &Utsname{
-		Sysname:    intArrayToString(uname.Sysname),
-		Nodename:   intArrayToString(uname.Nodename),
-		Release:    intArrayToString(uname.Release),
-		Version:    intArrayToString(uname.Version),
-		Machine:    intArrayToString(uname.Machine),
-		Domainname: intArrayToString(uname.Domainname),
-	}, nil
-}
-
-func intArrayToString(arr interface{}) string {
-	slice := reflect.ValueOf(arr)
-	s := ""
-	for i := 0; i < slice.Len(); i++ {
-		val := slice.Index(i)
-		valInt := int64(-1)
-
-		switch val.Kind() {
-		case reflect.Int:
-		case reflect.Int8:
-			valInt = int64(val.Int())
-		case reflect.Uint:
-		case reflect.Uint8:
-			valInt = int64(val.Uint())
-		default:
-			continue
-		}
-
-		if valInt == 0 {
-			break
-		}
-
-		s += string(byte(valInt))
-	}
-
-	return s
-}
