@@ -1,19 +1,6 @@
-gen_third_cert() {
-  [ -f "${LXD_CONF}/client3.crt" ] && return
-  mv "${LXD_CONF}/client.crt" "${LXD_CONF}/client.crt.bak"
-  mv "${LXD_CONF}/client.key" "${LXD_CONF}/client.key.bak"
-  lxc_remote list > /dev/null 2>&1
-  mv "${LXD_CONF}/client.crt" "${LXD_CONF}/client3.crt"
-  mv "${LXD_CONF}/client.key" "${LXD_CONF}/client3.key"
-  mv "${LXD_CONF}/client.crt.bak" "${LXD_CONF}/client.crt"
-  mv "${LXD_CONF}/client.key.bak" "${LXD_CONF}/client.key"
-}
+#!/bin/sh
 
 test_basic_usage() {
-  # shellcheck disable=2039
-  local lxd_backend
-  lxd_backend=$(storage_backend "$LXD_DIR")
-
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
@@ -38,6 +25,12 @@ test_basic_usage() {
   lxc image alias delete foo
   lxc image alias delete bar
 
+  # Test image list output formats (table & json)
+  lxc image list --format table | grep -q testimage
+  lxc image list --format json \
+    | jq '.[]|select(.alias[0].name="testimage")' \
+    | grep -q '"name": "testimage"'
+
   # Test image delete
   lxc image delete testimage
 
@@ -47,9 +40,6 @@ test_basic_usage() {
 
   # Re-import the image
   mv "${LXD_DIR}/${sum}.tar.xz" "${LXD_DIR}/testimage.tar.xz"
-  lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage user.foo=bar
-  lxc image show testimage | grep -q "user.foo: bar"
-  lxc image delete testimage
   lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage
   rm "${LXD_DIR}/testimage.tar.xz"
 
@@ -99,9 +89,6 @@ test_basic_usage() {
   # Test list json format
   lxc list --format json | jq '.[]|select(.name="foo")' | grep '"name": "foo"'
 
-  # Test list with --columns and --fast
-  ! lxc list --columns=nsp --fast
-
   # Test container rename
   lxc move foo bar
   lxc list | grep -v foo
@@ -112,7 +99,7 @@ test_basic_usage() {
   lxc delete foo
 
   # gen untrusted cert
-  gen_third_cert
+  gen_cert client3
 
   # don't allow requests without a cert to get trusted data
   curl -k -s -X GET "https://${LXD_ADDR}/1.0/containers/foo" | grep 403
@@ -123,16 +110,12 @@ test_basic_usage() {
   curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/" && false
   lxc image delete foo-image
 
-  # Test container publish with existing alias
-  lxc publish bar --alias=foo-image --alias=foo-image2
-  lxc launch testimage baz
-  # change the container filesystem so the resulting image is different
-  lxc exec baz touch /somefile
-  lxc stop baz --force
-  # publishing another image with same alias doesn't fail
-  lxc publish baz --alias=foo-image
-  lxc delete baz
-  lxc image delete foo-image foo-image2
+  # Test image compression on publish
+  lxc publish bar --alias=foo-image-compressed --compression=bzip2 prop=val1
+  lxc image show foo-image-compressed | grep val1
+  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/" && false
+  lxc image delete foo-image-compressed
+
 
   # Test privileged container publish
   lxc profile create priv
@@ -150,9 +133,9 @@ test_basic_usage() {
   # the container's filesystem. That's ok though: the logic we're trying to
   # test here is independent of storage backend, so running it for just one
   # backend (or all non-lvm backends) is enough.
-  if [ "$lxd_backend" = "lvm" ]; then
+  if [ "${LXD_BACKEND}" != "lvm" ]; then
     lxc init testimage nometadata
-    rm -f "${LXD_DIR}/containers/nometadata/metadata.yaml"
+    rm "${LXD_DIR}/containers/nometadata/metadata.yaml"
     lxc publish nometadata --alias=nometadata-image
     lxc image delete nometadata-image
     lxc delete nometadata
@@ -179,9 +162,35 @@ test_basic_usage() {
   lxc delete bar2
   lxc image delete foo
 
-  # test basic alias support
-  printf "aliases:\n  ls: list" >> "${LXD_CONF}/config.yml"
-  lxc ls
+  # Test alias support
+  cp "${LXD_CONF}/config.yml" "${LXD_CONF}/config.yml.bak"
+
+  #   1. Basic built-in alias functionality
+  [ "$(lxc ls)" = "$(lxc list)" ]
+  #   2. Basic user-defined alias functionality
+  printf "aliases:\n  l: list\n" >> "${LXD_CONF}/config.yml"
+  [ "$(lxc l)" = "$(lxc list)" ]
+  #   3. Built-in aliases and user-defined aliases can coexist
+  [ "$(lxc ls)" = "$(lxc l)" ]
+  #   4. Multi-argument alias keys and values
+  printf "  i ls: image list\n" >> "${LXD_CONF}/config.yml"
+  [ "$(lxc i ls)" = "$(lxc image list)" ]
+  #   5. Aliases where len(keys) != len(values) (expansion/contraction of number of arguments)
+  printf "  ils: image list\n  container ls: list\n" >> "${LXD_CONF}/config.yml"
+  [ "$(lxc ils)" = "$(lxc image list)" ]
+  [ "$(lxc container ls)" = "$(lxc list)" ]
+  #   6. User-defined aliases override built-in aliases
+  printf "  cp: list\n" >> "${LXD_CONF}/config.yml"
+  [ "$(lxc ls)" = "$(lxc cp)" ]
+  #   7. User-defined aliases override commands and don't recurse
+  lxc init testimage foo
+  LXC_CONFIG_SHOW=$(lxc config show foo --expanded)
+  printf "  config show: config show --expanded\n" >> "${LXD_CONF}/config.yml"
+  [ "$(lxc config show foo)" = "$LXC_CONFIG_SHOW" ]
+  lxc delete foo
+
+  # Restore the config to remove the aliases
+  mv "${LXD_CONF}/config.yml.bak" "${LXD_CONF}/config.yml"
 
   # Delete the bar container we've used for several tests
   lxc delete bar
@@ -190,9 +199,9 @@ test_basic_usage() {
   [ ! -d "${LXD_DIR}/snapshots/bar" ]
 
   # Test randomly named container creation
-  lxc launch testimage
+  lxc init testimage
   RDNAME=$(lxc list | tail -n2 | grep ^\| | awk '{print $2}')
-  lxc delete -f "${RDNAME}"
+  lxc delete "${RDNAME}"
 
   # Test "nonetype" container creation
   wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers" \
@@ -228,13 +237,17 @@ test_basic_usage() {
 
     lxc start autostart --force-local
     PID=$(lxc info autostart --force-local | grep ^Pid | awk '{print $2}')
-    shutdown_lxd "${LXD_DIR}"
+    lxd shutdown
     [ -d "/proc/${PID}" ] && false
 
     lxd activateifneeded --debug 2>&1 | grep -q "Daemon has auto-started containers, activating..."
 
-    # shellcheck disable=SC2031
-    respawn_lxd "${LXD_DIR}"
+    # shellcheck disable=SC2086
+    lxd --logfile "${LXD_DIR}/lxd.log" ${DEBUG-} "$@" 2>&1 &
+    LXD_PID=$!
+    echo "${LXD_PID}" > "${LXD_DIR}/lxd.pid"
+    echo "${LXD_DIR}" >> "${TEST_DIR}/daemons"
+    lxd waitready --timeout=300
 
     lxc list --force-local autostart | grep -q RUNNING
 
@@ -261,12 +274,12 @@ test_basic_usage() {
     false
   fi
 
-  # Test instance types
-  lxc launch testimage test-limits -t c0.5-m0.2
-  [ "$(lxc config get test-limits limits.cpu)" = "1" ]
-  [ "$(lxc config get test-limits limits.cpu.allowance)" = "50%" ]
-  [ "$(lxc config get test-limits limits.memory)" = "204MB" ]
-  lxc delete -f test-limits
+  # Test last_used_at field is working properly
+  lxc init testimage last-used-at-test
+  lxc list last-used-at-test  --format json | jq -r '.[].last_used_at' | grep '1970-01-01T00:00:00Z'
+  lxc start last-used-at-test
+  lxc list last-used-at-test  --format json | jq -r '.[].last_used_at' | grep -v '1970-01-01T00:00:00Z'
+  lxc delete last-used-at-test --force
 
   # check that we can set the environment
   lxc exec foo pwd | grep /root
@@ -307,7 +320,7 @@ test_basic_usage() {
   rm "${LXD_DIR}/out"
 
   # FIXME: make this backend agnostic
-  if [ "$lxd_backend" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     content=$(cat "${LXD_DIR}/containers/foo/rootfs/tmp/foo")
     [ "${content}" = "foo" ]
   fi
@@ -319,34 +332,40 @@ test_basic_usage() {
   # cleanup
   lxc delete foo -f
 
-  if [ -e /sys/module/apparmor/ ]; then
-    # check that an apparmor profile is created for this container, that it is
-    # unloaded on stop, and that it is deleted when the container is deleted
-    lxc launch testimage lxd-apparmor-test
+  # check that an apparmor profile is created for this container, that it is
+  # unloaded on stop, and that it is deleted when the container is deleted
+  lxc launch testimage lxd-apparmor-test
 
-    MAJOR=0
-    MINOR=0
-    if [ -f /sys/kernel/security/apparmor/features/domain/version ]; then
-      MAJOR=$(awk -F. '{print $1}' < /sys/kernel/security/apparmor/features/domain/version)
-      MINOR=$(awk -F. '{print $2}' < /sys/kernel/security/apparmor/features/domain/version)
-    fi
-
-    if [ "${MAJOR}" -gt "1" ] || ([ "${MAJOR}" = "1" ] && [ "${MINOR}" -ge "2" ]); then
-      aa_namespace="lxd-lxd-apparmor-test_<$(echo "${LXD_DIR}" | sed -e 's/\//-/g' -e 's/^.//')>"
-      aa-status | grep -q ":${aa_namespace}:unconfined" || aa-status | grep -q ":${aa_namespace}://unconfined"
-      lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q ":${aa_namespace}:" || false
-    else
-      aa-status | grep "lxd-lxd-apparmor-test_<${LXD_DIR}>"
-      lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
-    fi
-    lxc delete lxd-apparmor-test
-    [ ! -f "${LXD_DIR}/security/apparmor/profiles/lxd-lxd-apparmor-test" ]
-  else
-    echo "==> SKIP: apparmor tests (missing kernel support)"
+  MAJOR=0
+  MINOR=0
+  if [ -f /sys/kernel/security/apparmor/features/domain/version ]; then
+    MAJOR=$(awk -F. '{print $1}' < /sys/kernel/security/apparmor/features/domain/version)
+    MINOR=$(awk -F. '{print $2}' < /sys/kernel/security/apparmor/features/domain/version)
   fi
 
+  if [ "${MAJOR}" -gt "1" ] || ([ "${MAJOR}" = "1" ] && [ "${MINOR}" -ge "2" ]); then
+    aa_namespace="lxd-lxd-apparmor-test_<$(echo "${LXD_DIR}" | sed -e 's/\//-/g' -e 's/^.//')>"
+    aa-status | grep ":${aa_namespace}://unconfined"
+    lxc stop lxd-apparmor-test --force
+    ! aa-status | grep -q ":${aa_namespace}:"
+  else
+    aa-status | grep "lxd-lxd-apparmor-test_<${LXD_DIR}>"
+    lxc stop lxd-apparmor-test --force
+    ! aa-status | grep -q "lxd-lxd-apparmor-test_<${LXD_DIR}>"
+  fi
+  lxc delete lxd-apparmor-test
+  [ ! -f "${LXD_DIR}/security/apparmor/profiles/lxd-lxd-apparmor-test" ]
+
+  lxc launch testimage lxd-seccomp-test
+  init=$(lxc info lxd-seccomp-test | grep Pid | cut -f2 -d" ")
+  [ "$(grep Seccomp "/proc/${init}/status" | cut -f2)" -eq "2" ]
+  lxc stop --force lxd-seccomp-test
+  lxc config set lxd-seccomp-test security.syscalls.blacklist_default false
+  lxc start lxd-seccomp-test
+  init=$(lxc info lxd-seccomp-test | grep Pid | cut -f2 -d" ")
+  [ "$(grep Seccomp "/proc/${init}/status" | cut -f2)" -eq "0" ]
+  lxc delete --force lxd-seccomp-test
+
   # make sure that privileged containers are not world-readable
   lxc profile create unconfined
   lxc profile set unconfined security.privileged true
@@ -355,16 +374,42 @@ test_basic_usage() {
   lxc delete foo2
   lxc profile delete unconfined
 
+  # Test boot.host_shutdown_timeout config setting
+  lxc init testimage configtest --config boot.host_shutdown_timeout=45
+  [ "$(lxc config get configtest boot.host_shutdown_timeout)" -eq 45 ]
+  lxc config set configtest boot.host_shutdown_timeout 15
+  [ "$(lxc config get configtest boot.host_shutdown_timeout)" -eq 15 ]
+  lxc delete configtest
+
+  # Test deleting multiple images
+  # Start 3 containers to create 3 different images
+  lxc launch testimage c1
+  lxc launch testimage c2
+  lxc launch testimage c3
+  lxc exec c1 -- touch /tmp/c1
+  lxc exec c2 -- touch /tmp/c2
+  lxc exec c3 -- touch /tmp/c3
+  lxc publish --force c1 --alias=image1
+  lxc publish --force c2 --alias=image2
+  lxc publish --force c3 --alias=image3
+  # Delete multiple images with lxc delete and confirm they're deleted
+  lxc image delete local:image1 local:image2 local:image3
+  ! lxc image list | grep -q image1
+  ! lxc image list | grep -q image2
+  ! lxc image list | grep -q image3
+  # Cleanup the containers
+  lxc delete --force c1 c2 c3
+
   # Ephemeral
   lxc launch testimage foo -e
 
   OLD_INIT=$(lxc info foo | grep ^Pid)
-  lxc exec foo reboot || true
+  lxc exec foo reboot
 
   REBOOTED="false"
 
   # shellcheck disable=SC2034
-  for i in $(seq 20); do
+  for i in $(seq 10); do
     NEW_INIT=$(lxc info foo | grep ^Pid || true)
 
     if [ -n "${NEW_INIT}" ] && [ "${OLD_INIT}" != "${NEW_INIT}" ]; then
@@ -382,5 +427,5 @@ test_basic_usage() {
   sleep 2
 
   lxc stop foo --force || true
-  ! lxc list | grep -q foo || false
+  ! lxc list | grep -q foo
 }
