@@ -1,3 +1,4 @@
+#!/bin/sh
 ensure_removed() {
   bad=0
   lxc exec foo -- stat /dev/ttyS0 && bad=1
@@ -11,13 +12,13 @@ dounixdevtest() {
     lxc start foo
     lxc config device add foo tty unix-char "$@"
     lxc exec foo -- stat /dev/ttyS0
-    lxc restart foo
+    lxc exec foo reboot
     lxc exec foo -- stat /dev/ttyS0
     lxc restart foo --force
     lxc exec foo -- stat /dev/ttyS0
     lxc config device remove foo tty
     ensure_removed "was not hot-removed"
-    lxc restart foo
+    lxc exec foo reboot
     ensure_removed "removed device re-appeared after container reboot"
     lxc restart foo --force
     ensure_removed "removed device re-appaared after lxc reboot"
@@ -69,7 +70,7 @@ testloopmounts() {
   lxc exec foo stat /mnt/hello
   lxc config device remove foo mnt
   ensure_fs_unmounted "fs should have been hot-unmounted"
-  lxc restart foo
+  lxc exec foo reboot
   ensure_fs_unmounted "removed fs re-appeared after reboot"
   lxc restart foo --force
   ensure_fs_unmounted "removed fs re-appeared after restart"
@@ -107,7 +108,22 @@ test_config_profiles() {
 
   # setting an invalid config item should error out when setting it, not get
   # into the database and never let the user edit the container again.
-  ! lxc config set foo raw.lxc lxc.notaconfigkey=invalid
+  ! lxc config set foo raw.lxc "lxc.notaconfigkey = invalid"
+
+  # check that various profile application mechanisms work
+  lxc profile create one
+  lxc profile create two
+  lxc profile assign foo one,two
+  [ "$(lxc info foo | grep Profiles)" = "Profiles: one, two" ]
+  lxc profile assign foo ""
+  [ "$(lxc info foo | grep Profiles)" = "Profiles: " ]
+  lxc profile apply foo one # backwards compat check with `lxc profile apply`
+  [ "$(lxc info foo | grep Profiles)" = "Profiles: one" ]
+  lxc profile assign foo ""
+  lxc profile add foo one
+  [ "$(lxc info foo | grep Profiles)" = "Profiles: one" ]
+  lxc profile remove foo one
+  [ "$(lxc info foo | grep Profiles)" = "Profiles: " ]
 
   lxc profile create stdintest
   echo "BADCONF" | lxc profile set stdintest user.user_data -
@@ -122,10 +138,10 @@ test_config_profiles() {
   lxc config device add foo mnt1 disk source="${TEST_DIR}/mnt1" path=/mnt1 readonly=true
   lxc profile create onenic
   lxc profile device add onenic eth0 nic nictype=bridged parent=lxdbr0
-  lxc profile apply foo onenic
+  lxc profile assign foo onenic
   lxc profile create unconfined
   lxc profile set unconfined raw.lxc "lxc.aa_profile=unconfined"
-  lxc profile apply foo onenic,unconfined
+  lxc profile assign foo onenic,unconfined
 
   lxc config device list foo | grep mnt1
   lxc config device show foo | grep "/mnt1"
@@ -136,10 +152,9 @@ test_config_profiles() {
 
   # test live-adding a nic
   lxc start foo
-  lxc exec foo -- cat /proc/self/mountinfo | grep -q "/mnt1.*ro,"
-  ! lxc config show foo | grep -q "raw.lxc" || false
+  ! lxc config show foo | grep -q "raw.lxc"
   lxc config show foo --expanded | grep -q "raw.lxc"
-  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0" || false
+  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0"
   lxc config show foo --expanded | grep -v "volatile.eth0" | grep -q "eth0"
   lxc config device add foo eth2 nic nictype=bridged parent=lxdbr0 name=eth10
   lxc exec foo -- /sbin/ifconfig -a | grep eth0
@@ -151,7 +166,6 @@ test_config_profiles() {
   mkdir "${TEST_DIR}/mnt2"
   touch "${TEST_DIR}/mnt2/hosts"
   lxc config device add foo mnt2 disk source="${TEST_DIR}/mnt2" path=/mnt2 readonly=true
-  lxc exec foo -- cat /proc/self/mountinfo | grep -q "/mnt2.*ro,"
   lxc exec foo -- ls /mnt2/hosts
   lxc stop foo --force
   lxc start foo
@@ -194,12 +208,10 @@ test_config_profiles() {
   lxc delete foo
 
   lxc init testimage foo
-  lxc profile apply foo onenic,unconfined
+  lxc profile assign foo onenic,unconfined
   lxc start foo
 
-  if [ -e /sys/module/apparmor ]; then
-    lxc exec foo -- cat /proc/self/attr/current | grep unconfined
-  fi
+  lxc exec foo -- cat /proc/self/attr/current | grep unconfined
   lxc exec foo -- ls /sys/class/net | grep eth0
 
   lxc stop foo --force
